' The script executes a given command line on a remote machine using WMI
' Requires admin privileges on the remote host and domain-based authentication
' Supply two parameters: remote host name and command line
' Optionally provide third parameter - the working directory

' To debug the script - comment out DeleteTempRemoteFolder call below

' The script works by creating a temp command wrapper (bat) in a temp folder on 
' a remote machine, invoking the wrapper and waiting for the flag file indicating 
' that the remote command has completed. The remote command is writing to a log file
' which is read by the script and displayed locally. 

Option Explicit

On Error Goto 0 

private const WaitTimeout = 600000, PollInterval = 500
private const ForReading = 1, ForWriting = 2

QuitWithErrorIfNotCSript()

if WScript.Arguments.Count < 2 or WScript.Arguments.Count > 3 then 
	WScript.Echo "Wrong number of command line arguments. Please provide host name, command line to execute and base directory (optional)"
end if

Dim	fso
	Set fso = CreateObject( "Scripting.FileSystemObject" )		

Dim	TargetHost, TargetCommandLine, TargetWorkDir
	TargetHost = WScript.Arguments(0)
	TargetCommandLine = WScript.Arguments(1)

	if WScript.Arguments.Count = 3 then 
		TargetWorkDir = WScript.Arguments(2)
	else
		TargetWorkDir = null
	end if

Dim	UncTempDir, UncDriverScript, UncFlagFile, UncLogFile

	UncTempDir = SetupTempRemoteFolder()
	UncDriverScript = UncTempDir & "\Remote Script Driver.cmd"
	UncFlagFile = UncTempDir & "\ProcessFinished.flag"
	UncLogFile = UncTempDir & "\ProcessExecution.log"

WScript.Quit( ExecuteRemoteScript() )

'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

private function SetupTempRemoteFolder()
	Dim tempRoot
	
	tempRoot = "\\" & TargetHost & "\c$\temp"

	SetupTempRemoteFolder = tempRoot & "\" & GetNewGuid()

	if not fso.FolderExists( tempRoot ) then fso.CreateFolder( tempRoot )
	if not fso.FolderExists( SetupTempRemoteFolder ) then fso.CreateFolder( SetupTempRemoteFolder )	
end function

private sub DeleteTempRemoteFolder()
	if fso.FolderExists( UncTempDir ) then fso.DeleteFolder UncTempDir, true 
end sub


private function ExecuteRemoteScript()	
    WScript.Echo( "[Executing the following command line on \\" & TargetHost & ": " & TargetCommandLine & "]..." )
	
	DeployDriverScript

	StartRemoteDriverScript

	Dim timedOutFlag, remoteReturnCode

	timedOutFlag = not WaitForFinishedFlag

	WSCript.Echo( "[Remote console output log (both stdout and stderr)]:" )
	WScript.Echo( "" )

	WScript.Echo( ReadRemoteLog() )

	if timedOutFlag then 
		FatalErrorExitWithCleanup "Timed out while waiting for the remote process to finish" 
	else
		ExecuteRemoteScript = ReadRemoteReturnCode
	end if

 	DeleteTempRemoteFolder
end function


private function WaitForFinishedFlag()
	Dim timeoutCounter 

	do
		WScript.Sleep( PollInterval )
		timeoutCounter = timeoutCounter + PollInterval
	loop  until fso.FileExists( UncFlagFile ) or timeoutCounter > WaitTimeout

	WaitForFinishedFlag = fso.FileExists( UncFlagFile )	
end function


private function ReadRemoteLog()
	if not fso.FileExists( UncLogFile ) then
		ReadRemoteLog = "[Remote log file '"	+ UncLogFile + "' does not exist]"
	end if

	Dim file

	set file = fso.OpenTextFile( UncLogFile, ForReading )
	ReadRemoteLog = file.ReadAll
	file.Close		

	if isnull(ReadRemoteLog) or len(trim(ReadRemoteLog)) = 0 then
		ReadRemoteLog = "[Remote process log file was empty]"
	end if
end function


private function ReadRemoteReturnCode()
	if not fso.FileExists( UncFlagFile ) then 
		FatalErrorExitWithCleanup "The flag file '" + UncFlagFile + "' does not exist"
	end if
	
	Dim flagFileContents, file
	
	set file = fso.OpenTextFile( UncFlagFile, ForReading )
	flagFileContents = file.ReadLine
	file.Close

	if len( flagFileContents ) = 0 or not IsNumeric( flagFileContents ) then
	  WSCript.Echo( "Flag file contents did not contain the numeric return code [sleep 500 milliseconds and try again]" )
	  WScript.Sleep( PollInterval )
	  set file = fso.OpenTextFile( UncFlagFile, ForReading )
	  flagFileContents = file.ReadLine
	  file.Close
	  if len( flagFileContents ) = 0 or not IsNumeric( flagFileContents ) then
		  FatalErrorExitWithCleanup "Flag file contents did not contain the numeric return code [" & flagFileContents & "]" 
		end if
	end if

	ReadRemoteReturnCode = CInt( flagFileContents )
end function


private sub StartRemoteDriverScript()
	Dim objWMIService, objProcess
	Dim strShell, objProgram, strInput

	' Connect to WMI
	set objWMIService = GetObject( "winmgmts://" & TargetHost & "/root/cimv2" )

	' Obtain the Win32_Process class of object.
	Set objProcess = objWMIService.Get( "Win32_Process" )
	Set objProgram = objProcess.Methods_( "Create" ).InParameters.SpawnInstance_

	objProgram.CommandLine = UncDriverScript

	'Execute the program now at the command line.
	Set strShell = objWMIService.ExecMethod( "Win32_Process", "Create", objProgram )
end sub


private function DeployDriverScript()
	Dim file 

	Set file = fso.OpenTextFile( UncDriverScript, ForWriting, True )
	file.WriteLine( GetDriverScript() )
	file.Close
end function


private function GetDriverScript()
	Dim script

	script = script & "@if exist """ & UncFlagFile & """ del /f """ & UncFlagFile & """" & vbCrLf
	script = script & "@if exist """ & UncLogFile & """ del /f """ & UncLogFile & """" & vbCrLf

	if not isnull( TargetWorkDir ) then script = script & "pushd """ & TargetWorkDir & """" & vbCrLf

	script = script & "@cmd /c """ & TargetCommandLine & """ > """ & UncLogFile & """ 2>&1" & vbCrLf
	script = script & "@echo %errorlevel% > """ & UncFlagFile & """" & vbCrLf

	GetDriverScript = script
end function


private sub QuitWithErrorIfNotCSript()
	If IsCscript() Then exit sub

	WScript.Echo "Please run this script from a command line using cscript."
	WScript.Quit(1)	
end sub


private function IsCscript()
  IsCscript = False

  If InStrRev(LCase(WScript.FullName), "cscript.exe", -1) Then
    IsCscript = True
  End If
end function    	


private function GetNewGuid()
	Dim TypeLib, guid

	Set TypeLib = CreateObject("Scriptlet.TypeLib")
	guid = TypeLib.Guid
	GetNewGuid = replace( replace( left(guid, len(guid)-2), "{", "" ), "}", "")
	Set TypeLib = Nothing
end function


private sub FatalErrorExitWithCleanup( message )
	Dim convertedMessage

	if IsNull( message ) or len( trim( message ) ) = 0 then
		convertedMessage = "[Error message not specified]"
	else
		convertedMessage = message
	end if		

	WScript.Echo "ERROR OCCURED: " & convertedMessage
	
	DeleteTempRemoteFolder

	WScript.Quit( 1 )
end sub
