/*****************************************************************************************
* VBECS Patch	(Build 1.5.2.2)
*****************************************************************************************/

USE VBECS_V1_TEST
GO

/*****************************************************************************************
**	Change the BloodUnitStatusCodeCurrent trigger name by dropping and readding.
*****************************************************************************************/
if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[trBloodUnitStatusCodeCurrent]') and OBJECTPROPERTY(id, N'IsTrigger') = 1) 
drop trigger [dbo].[trBloodUnitStatusCodeCurrent]
GO

if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[trMirrorBloodUnitStatusCodeCurrent]') and OBJECTPROPERTY(id, N'IsTrigger') = 1) 
drop trigger [dbo].[trMirrorBloodUnitStatusCodeCurrent]
GO
SET QUOTED_IDENTIFIER ON
GO
SET ANSI_NULLS ON
GO

CREATE TRIGGER dbo.trMirrorBloodUnitStatusCodeCurrent
ON dbo.BloodUnitStatusCodeCurrent
FOR UPDATE
AS
BEGIN
	INSERT INTO VBECS_V1_TEST_MIRROR.dbo.BloodUnitStatusCodeCurrent
	SELECT *, GETUTCDATE()
	FROM DELETED
END

GO

/*****************************************************************************************
**	Stored procedure changes.
*****************************************************************************************/
if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[spGetBloodUnitAntigens]') and OBJECTPROPERTY(id, N'IsProcedure') = 1) 
drop procedure [dbo].[spGetBloodUnitAntigens]
GO
SET QUOTED_IDENTIFIER OFF
GO

/*******************************************************************************************************************************************************************************
**	Desc:	Gets Blood Unit Antigens based on Blood Unit GUID and DivisionCode
**
**	Author:	Cameron Taylor
**
**	Date: 	03/10/2003
**
*******************************************************************************************************************************************************************************
**	Modified:
**	Date		Name		Description
**	--------------------	-----------------------	------------------------------------------------------------------------------------------------------------------------------------------------------------------------
**	02/27/2004	M.Jablonski	Added row version to SELECT
**	07/14/2005	N.Paterson	** Reviewed for compliance with database standards and recommendations **
**					Removed Name/input/output from comment header, added SET NOCOUNT, formatted SQL, specified object owners
**	10/13/2010	M. Smith	Changed the input parameter to allow multiple blood unit guids.  (CR 2899)
**	12/22/2010	K. Adu-Poku	** Reviewed for compliance with database standards and recommendations **
*******************************************************************************************************************************************************************************/
CREATE PROCEDURE dbo.spGetBloodUnitAntigens
(
	@BloodUnitGuidList	VARCHAR(8000),  -- Allows multiple copies of (38-character guid + 1-char delimiter)
	@divisioncode		CHAR(5)
)
AS

DECLARE @ERROR INT

SET NOCOUNT ON

	SELECT

		BUA.BloodUnitGUID,
		BUA.AntigenTypeID,
		ANT.AntigenTypeName,
		BUA.Positive,
		BUA.RecordStatusCode,
		BUA.DivisionCode,
		BUA.[RowVersion]
	
	FROM
		dbo.BloodUnitAntigen BUA
			JOIN dbo.fnSplit(@BloodUnitGuidList, ',')
				ON BUA.BloodUnitGUID = Item
			JOIN dbo.AntigenType ANT 
				ON ANT.AntigenTypeID = BUA.AntigenTypeID
		
	WHERE DivisionCode = @divisioncode
	AND	RecordStatusCode != 'X'


SET @ERROR = @@ERROR

SET NOCOUNT OFF

RETURN(@ERROR)
GO

if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[spGetOrderedUnitByPatientAndUnit]') and OBJECTPROPERTY(id, N'IsProcedure') = 1) 
drop procedure [dbo].[spGetOrderedUnitByPatientAndUnit]
GO
SET QUOTED_IDENTIFIER OFF
GO
/*****************************************************************************************************************
**	Desc:	Gets all Ordered Unit based on patient and unit
**
**	Author:	Carl Jensen
**
**	Date: 	4/15/2004
**
******************************************************************************************************************
**	Modified:
**	Date		Name		Description
**	--------------------	-----------------------	----------------------------------------------------------------------------------------------------------------------------------------------------
**	04/16/2004	C Taylor	Added RecordStatusCode
**	04/23/2004	M. Jablonski	Call fnTimeZoneConversionDateTime function on all dates
**	05/25/2004	C. Jensen	Added SelectedDate and SelectedTechId
**	06/07/2004	C. Taylor	Replaced PatientGuid clause with fnWherePatientGuid function, to include possible merged patients
**	10/27/2004	C. Jensen	Added CrossmatchTechID
**	02/01/2005	Lohse		Adding BloodUnitGuid, DivisionCode for UC 24-26
**	03/28/2005	C. Jensen	Added TimeZone conversion to OU.SelectedDate
**	07/20/2005	N.Paterson	** Reviewed for compliance with database standards and recommendations **
**					Removed Name/input/output from comment header, added SET NOCOUNT, formatted SQL, specified object owners
**	07/25/2005	Lohse		Adding ReleaseDate, ReleaseTechId			
**	09/09/2005	D. Askew	Removed fnWherePatientGuid via sub-query using new PatientMergeGroup
**	10/13/2010	M. Smith	Changed the input parameter to allow multiple blood unit guids.  (CR 2899)
**	12/22/2010	K. Adu-Poku	** Reviewed for compliance with database standards and recommendations **
******************************************************************************************************************/
CREATE PROCEDURE dbo.spGetOrderedUnitByPatientAndUnit
(
	@patientguid		UNIQUEIDENTIFIER,
	@BloodUnitGuidList	VARCHAR(8000),  -- Allows multiple copies of (38-character guid + 1-char delimiter)
	@divisioncode		CHAR(5)
)
AS


SET NOCOUNT ON
		
    SELECT DISTINCT

	OU.BloodUnitGuid,
	OU.DivisionCode,
	OU.OrderedUnitGUID,
	OU.CrossmatchRequiredIndicator,

	dbo.fnTimeZoneConversionDateTime(@divisioncode, 0, OU.SelectedDate ) AS SelectedDate,

	OU.SelectedTechId,

	dbo.fnTimeZoneConversionDateTime(@divisioncode, 0, OU.CrossmatchDate ) AS CrossmatchDate,

	OU.CrossmatchResultCode,
	OU.CrossmatchTechID,
	dbo.fnTimeZoneConversionDateTime(@divisioncode, 0, OU.ReleaseDate ) AS ReleaseDate,
	OU.ReleaseTechId,
	OU.RecordStatusCode,
	OU.OrderedComponentGUID,
	PT.PatientLocation,
	PS.Specimenuid,
	CR.CrossmatchResultText,
	OU.LastUpdateUser,
	OU.[Rowversion]

    FROM 

	dbo.OrderedUnit OU
		JOIN dbo.fnSplit(@BloodUnitGuidList, ',')
			ON OU.BloodUnitGuid = Item

   	JOIN dbo.OrderedComponent OC ON OC.OrderedComponentGUID = OU.OrderedComponentGUID

    	JOIN dbo.ComponentClassParameter CCC ON OC.ComponentClassID = CCC.ComponentClassID AND OC.DivisionCode = CCC.DivisionCode

   	JOIN dbo.PatientOrder PO ON OC.PatientOrderGUID = PO.PatientOrderGUID

    	JOIN dbo.PatientTreatment PT ON PT.PatientTreatmentGUID = PO.PatientTreatmentGUID

	LEFT JOIN dbo.CrossmatchResult CR ON OU.CrossmatchResultCode = CR.CrossmatchResultCode

	LEFT JOIN dbo.PatientSpecimen PS on OC.PatientSpecimenGUID = PS.PatientSpecimenGUID

    WHERE 

	PT.PatientGUID IN 
	(
		SELECT 
			P.PatientGuid
		FROM
			dbo.Patient P
		JOIN 
		    	dbo.Patient LinkPatient ON  P.PatientMergeGroup = LinkPatient.PatientMergeGroup OR P.PatientGuid = LinkPatient.PatientGuid
		WHERE
			LinkPatient.PatientGuid = @patientguid
	)

	AND OU.ReleaseDate IS NULL


SET NOCOUNT OFF
GO

if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[spGetBloodUnitTestResults]') and OBJECTPROPERTY(id, N'IsProcedure') = 1) 
drop procedure [dbo].[spGetBloodUnitTestResults]
GO
SET ANSI_NULLS OFF
GO

/*******************************************************************************************************************************************************************************
**
**	Desc:	Gets Blood Unit Test Results
**
**	Author:	John Vrooland
**
**	Date: 	05/15/2003
**
*******************************************************************************************************************************************************************************
**	Modified:
**	Date		Name		Description
**	04/27/04	M. Jablonski	Call fnTimeZoneConversionDateTime function on all dates
**	07/08/04	Greg Lohse	Adding columns (denoted by --)
**	10/01/04 	C.Jensen	Added RecordStatusCode
**	07/15/05 	D. Askew	** Reviewed for compliance with database standards and recommendations **
**					NO COUNT set ON/OFF; comments updated
**	06/30/05	Lohse		Adding more fields for Modification test duplication (denoted by --*)
**	08/19/05	N.Paterson	Implemented differences between Review_Start and DEV before porting changes to DEV, SEE ABOVE
**	04/04/06	Lohse		Adding DateTimeConversion to InactivateDate
**	11/29/06	Lohse		Adding fitler to only include Active records - this affects UC_4, UC_41 - CR 2099
**	02/08/07	D. Askew     	** Reviewed for compliance with database standards and recommendations **
**					Added dbo prefix
**	10/13/10	M. Smith	Changed the input parameter to allow multiple blood unit guids.  (CR 2899)
**	12/22/10	K. Adu-Poku	** Reviewed for compliance with database standards and recommendations **
*******************************************************************************************************************************************************************************/
CREATE PROCEDURE dbo.spGetBloodUnitTestResults
(
	@BloodUnitGuidList	VARCHAR(8000),  -- Allows multiple copies of (38-character guid + 1-char delimiter)
	@divisioncode		CHAR(5)
)

AS

SET NOCOUNT ON

	SELECT
		BUT.BloodUnitTestGuid,
		BUT.BloodUnitGuid,
		BUT.BloodTestTypeId,
		dbo.fnTimeZoneConversionDateTime(@divisioncode, 0, BUT.TestDate) AS TestDate,
		BUT.TestTechId,--
		BUT.OrderedTestGuid, --
		BUT.TestingMethodCode, --
		BUT.EntryTechId, --
		BUT.CorrectedResultIndicator, --
		BUT.AntiseraWorklistTypingGuid, --
		BUT.ConfirmationWorklistUnitGuid, --
		BUT.TestResultId,
		dbo.fnTimeZoneConversionDateTime(@divisioncode, 0, BUT.InactivateDate) AS InactivateDate,
		BUT.TestComments,
		BUT.EntryMethodCode,
		BUT.DivisionCode,
		BUT.RecordStatusCode,
		BUT.[RowVersion],
		TR.TestResultText,
		BUT.AutoInstrumentName,--*
		BUT.RackGuid--*
		
	FROM	dbo.BloodUnitTest BUT
			JOIN dbo.fnSplit(@BloodUnitGuidList, ',')
				ON BUT.BloodUnitGUID = Item
			JOIN dbo.TestResult TR 
				ON TR.TestResultID = BUT.TestResultID
	
	WHERE
		BUT.DivisionCode = @divisioncode
	AND	BUT.RecordStatusCode = 'A'
	AND	BUT.TestResultId IS NOT NULL		--partially completed antigen tests will have null values, but A recordstatuscodes

RETURN @@ERROR

SET NOCOUNT OFF
GO

/*****************************************************************************************
**	Grant permissions on stored procedures.
*****************************************************************************************/
GRANT EXECUTE ON  dbo.spGetBloodUnitAntigens TO VbecsAppUser
GRANT EXECUTE ON  dbo.spGetOrderedUnitByPatientAndUnit TO VbecsAppUser
GRANT EXECUTE ON  dbo.spGetBloodUnitTestResults TO VbecsAppUser

/***************************************************************************************
**	Blood product updates
***************************************************************************************/


-- Update 2 rows in dbo.ProductType
UPDATE dbo.ProductType SET RetireDate='20100503 00:00:00.000', LastUpdateDate='20100820 14:05:43.000' WHERE ProductTypeCode=N'E044'
UPDATE dbo.ProductType SET RetireDate='20100503 00:00:00.000', LastUpdateDate='20100820 14:05:43.000' WHERE ProductTypeCode=N'E045'

-- Update 321 rows in dbo.ProductAttribute
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@00'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@01'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@02'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@03'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@04'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@05'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@06'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@07'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@08'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@09'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@10'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@11'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@12'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@13'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100826 14:39:37.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@14'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@15'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@16'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@17'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@18'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@19'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@20'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@21'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@22'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@23'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100826 14:39:37.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@26'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@28'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@29'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@30'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@31'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@32'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@33'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@34'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@35'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@36'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@37'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@38'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@39'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@40'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@41'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@42'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@46'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@47'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@48'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@49'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@50'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@51'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@52'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@53'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@54'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@55'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@56'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@57'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@58'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@59'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@60'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@61'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@62'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@63'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@64'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@65'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@66'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@67'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@68'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@69'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@70'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@71'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@72'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@73'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@74'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@75'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@76'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@77'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@78'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@79'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@80'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@81'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@82'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@83'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@84'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@85'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@86'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@87'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@88'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@89'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@90'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@91'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@92'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@93'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@94'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@95'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@97'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@98'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@99'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@A0'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@A1'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@A2'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@A3'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@A4'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@A5'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@A6'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@A7'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@A8'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AA'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AB'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AC'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AD'
UPDATE dbo.ProductAttribute SET ProductAttributeText=N'PAS-B/XX/20-24C', ProductAttributePrintName=N'PAS-B', LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AE'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AF'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AG'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AH'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AI'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AJ'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@AK'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AL'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AM'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AN'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AO'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AP'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AQ'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'@AS'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AT'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AU'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AW'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@AX'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@AY'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@AZ'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@B1'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@B2'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@B3'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@B4'
UPDATE dbo.ProductAttribute SET ProductAttributeText=N'PAS-C/XX/20-24C', ProductAttributePrintName=N'PAS-C', ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@B5'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@B6'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@B7'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@B8'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@B9'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@BA'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@BB'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'@BC'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BD'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BE'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BF'
UPDATE dbo.ProductAttribute SET ProductAttributeText=N'CPD>PAS-C/XX/20-24C', ProductAttributePrintName=N'CPD>PAS-C', ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BG'
UPDATE dbo.ProductAttribute SET ProductAttributeText=N'ACD-A>PAS-C/XX/20-24C', ProductAttributePrintName=N'ACD-A>PAS-C', ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BH'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BI'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BJ'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BK'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BL'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BN'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BO'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BP'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BQ'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BR'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BS'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BT'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BU'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BV'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BW'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BX'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BY'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@BZ'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@C1'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@C2'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@C3'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@C4'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@C5'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@C8'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@C9'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CA'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CB'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CC'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CD'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CE'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CF'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CG'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CH'
UPDATE dbo.ProductAttribute SET ProductAttributeText=N'PAS-D/XX/20-24C', ProductAttributePrintName=N'PAS-D', LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CI'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CJ'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CK'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CN'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CO'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CP'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CQ'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, RetireDate='20100301 00:00:00.000', LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CR'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CS'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CT'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CU'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CV'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CW'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@CZ'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@D0'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@D1'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@D2'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100820 14:52:17.000' WHERE ProductAttributeId=N'@D3'
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100825 11:30:50.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'A0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100825 11:30:50.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'A1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'B0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'B1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'B2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'C0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'C1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'C2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'D0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'D1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'D2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'D3 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'D4 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'D5 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'D6 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'D7 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'D8 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'D9 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'DA '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'DB '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'DC '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'E0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'E1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'E2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'E3 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'E4 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'E5 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'E6 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'E7 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'E8 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'E9 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'EA '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'EB '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'EC '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'ED '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'EE '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'EF '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'F0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'F1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'F2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'F3 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'F4 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'F5 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'F6 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'F7 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'F8 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'F9 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'FA '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'G0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'G1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'G2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'G3 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'G4 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'G5 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'G6 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'G7 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'G8 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'G9 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'GA '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'GB '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'GC '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'GD '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'GE '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'H0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'H1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'H2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'H3 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'H4 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'H5 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'H6 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'H7 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'H8 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'H9 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'HA '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'HB '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'HC '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'I0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'I1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'I2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'I3 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'I4 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'I5 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'I6 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'I7 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'J0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'J1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'J2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'J3 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'J4 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'J5 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'J6 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'J7 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'J8 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'J9 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JA '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JB '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JC '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JD '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JE '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JF '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JG '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JH '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'JI '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JJ '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JK '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JL '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JM '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'JN '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'K0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'K1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'K2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'K3 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'K4 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'K5 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'K6 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'L0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'L1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'L2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'L3 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'L4 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'L5 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'M0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'M1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'M2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'N0 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'N1 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000', LastUpdateUser=N'vhaishsmithm' WHERE ProductAttributeId=N'N2 '
UPDATE dbo.ProductAttribute SET ModificationAllowedIndicator=1, LastUpdateDate='20100823 14:03:45.000' WHERE ProductAttributeId=N'N3 '

-- Update 2597 rows in dbo.BloodProduct
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'00025457-b774-4406-991b-2d6d4175abff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0016fd31-9ac0-409f-8448-ba5b5098d0dd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'002b6ca1-3712-43fc-b4fa-6e0fe6c39bda'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'00422b4b-de4f-4f8a-a2b0-eecd38f8c285'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0059ce70-aad2-48d9-a61d-0fdb6c5c7923'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'005e093c-1a1e-4c18-9e4c-cbe75bcb3da5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0076d7ff-d832-4468-877d-16914b02baf5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'007cb83c-91e5-4fb8-99bc-aac7b8865ad4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'00a20666-d95f-4eea-a0a6-08e93e9289f5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'00a9169d-0e17-4764-a507-0e03cb86d243'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'00abc6f5-438b-482a-b335-2766ab1d2473'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 B2 C2 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'00ac9a13-b5cb-480d-a435-a9180e2a82ac'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'00e4b0b5-344b-44a7-a902-4e94a270993b'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84 B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'011889be-1578-4428-a362-32a41cf61211'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'013a3968-1f41-44fc-aebf-4741620a1800'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS' WHERE BloodProductGuid=N'013d66b5-cf80-40bf-9ce9-2d63a6ef9f3c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'017e081d-a7c2-49a0-8415-e7e2754129e6'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|1st container|Bacterial test', ProductShortName=N'PLT AFR PAS-B LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'018aae74-f6e1-446f-85ac-a6552bb51126'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0191485c-6993-41a5-91f4-f20d88e17b2a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'01d253d6-b884-4f30-be64-0f71044bdf9c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'01f2a471-7ce3-4525-b111-8994f492dd17'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|1st container', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'01f5e7b7-9ea9-40ec-8007-cc4c6440e57d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0235cc74-6dc1-4113-9569-9085b790fcda'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0240f466-ea92-49b0-a129-7b2bb77cc59d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'024e0933-a9ca-4b7d-9cca-c0bbeaade9e6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'02526682-3c1d-4240-bbfb-0d71e779818e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'025403cc-983e-4b4a-a45b-d6b35713cb44'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'02751ffa-6b02-4aff-9e84-fe51d13f1aba'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0279432f-6f7b-40df-b046-9255e107ce2a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0284fb94-3a9f-4802-8ae0-e0fe1d7d9caf'
UPDATE dbo.BloodProduct SET ProductName=N'Washed PLATELETS|PAS-B/XX/20-24C|Buffy coat plts prep', ProductShortName=N'PLT Wash PAS-B BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'0293132f-ca64-41e5-bd93-435476de5f2f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'02b43809-342a-430c-b205-36e82f5894fc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'02ce1e6d-0439-4001-bb61-68c826fd4c7b'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Plasma reduced|Buffy coat plts prep|Approx 240log9 plts', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'02d0ed90-11b6-4b4b-9cad-8fe73a5ed8e5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'02de27c1-6168-4494-8bfa-0dfa600bda5f'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'02e7e58a-cd93-4c06-8c08-aa4e4182e70d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'02ebadf8-3bd2-4ffc-9c42-c3de62757e64'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'02ed5a05-e993-4e6f-9c2a-81fe1e512f29'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'02fafe1c-8f76-486c-925b-1d92f47b5a3e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'032d1112-2160-49c6-b26a-6e9606fdb4e0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'035bae1f-bd61-4ff7-8325-faa820b24bf7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0364e181-27cd-40af-9a9f-008e1c526fb1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0381a2f8-9a0e-4345-85fe-baf5cc236a06'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 B2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'0388e487-8bff-4169-bb25-93692974a426'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0397977b-a84d-4abe-aa0c-5b73d6afd830'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'039d585d-c64d-4d5e-9c99-94ead03cc586'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'03a026d0-93b2-4e15-abe3-610cd64f235d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'03fafed1-9ae6-41e9-9567-5ed2affea344'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'04156061-d1c8-4f80-9525-a5a16a1cf20b'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Buffy coat plts prep|From 2 donors', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BUFCT 2DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'043a0d25-9dca-4adf-8470-dfe66e798f5f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'044e5423-e072-4509-90a5-da0e31d67dac'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83 C2', LastUpdateDate='20101201 15:05:41.827' WHERE BloodProductGuid=N'04502597-4dde-4c06-bef9-f2c8c7b3b93a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'04597948-053b-48c6-9bfc-f4b7043b5f8e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0465fe3b-eee6-476a-a7ea-63e8ad1445ab'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'047a2208-9261-4002-b0ee-8e11367c53a3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0496a077-534e-4787-8120-53ad4046f956'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'049906f8-8531-4aef-b90b-3a3ea41b889c'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'049bf494-7cbf-4a6f-9179-8a1f18123410'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'04bcbb56-349c-47b5-b36e-075728c11dae'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'04bf80dc-6cd8-4ef8-a461-279b26f4a5ff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'04c6f87b-8495-417b-9058-a588d5227e7d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'04d4d9b2-1f94-4231-b11c-5aa81448e744'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'04e180ec-5bdc-4435-8d2f-18dcf24afd55'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'04e3688d-b99c-47a0-b78a-ba64820a45b6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0507ba4c-45d1-4538-86d0-771cc8f5dfe6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'050efe97-cc36-48a7-8428-aa6b125b48b1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'05244089-e384-4fb4-b6f7-8f68a2c1ece4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'05b4a598-8af2-434b-b498-1cb48315fc4f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'05c50772-210a-4730-b0c4-85a0af1a9680'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'05c8c694-4fd1-4975-b084-5530550f5cb8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'05db268f-6308-4f18-866a-89256db5774a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'05fc0037-d661-431b-bd21-4c72cc258e1b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'062cb67d-152b-4618-8f72-b773a4335e37'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'0633b0d7-a713-4df3-98ae-a67850f7df02'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0649a1f5-52b7-4581-91c0-ae43163a7b48'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'064af6e8-d16f-4fe0-ad62-1177b7e52e12'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'065e907d-5edc-468b-92e1-886d287c0e42'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'066ad462-89e8-475a-9c57-1dbd7f472956'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'06756f09-258e-41a0-8061-735cfb4b21ce'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'06a909d0-df6e-491f-921b-22cfb9301a86'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'06b21389-2977-4d71-bcac-0546a5c4526a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'06b9a8e8-70f9-4a13-8e5d-e50316c77156'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'06c43405-b375-4335-b587-59650e83b6c5'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Plasma reduced|Buffy coat plts prep|Approx 180log9 plts', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'06cd5211-b352-4b02-bb65-0be4e9be2b5a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'06dc2cf3-9ab5-4b84-a855-b5eb1d1cc8e2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'06e7ffdc-9587-4644-b63a-7a2455e0722e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'06f0977e-90e2-4d5b-97e3-a3677689641a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'06fc627e-da65-443f-a592-8c965b17e4fb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'070708af-4559-4ab5-822d-31984ad96122'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'070993ce-9947-4863-a987-5f47ebd8bb3a'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 C2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'072275d1-8157-4db9-bbd5-438c1fee5bca'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0729748e-864c-4392-99fe-93807c560762'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'07456b83-79fa-45c2-ae09-6a5425bf2783'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0778109c-efe1-44cb-8f1c-cd2ed884056e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'077e0471-1cbd-4b80-b6d3-7ff5d14f1f1a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'07a2014a-c448-4db2-a841-378d5d115bd8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'080a492f-e8dd-48d8-a57d-753be696f9a2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0817350b-6465-4c88-b842-2a143ac350e9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'082c68ae-cbfd-47ba-9f6d-04da93e753e7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'085954db-7890-4c7e-aa74-56990a7cf7f8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'085f9d60-d696-4300-be64-b29fd1ef733b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'08b963ae-448b-4739-a0d3-0a402e291954'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|4th container|Psoralen-treated', ProductShortName=N'PLT AFR PAS-C LUKOPR Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'08eec6e3-19d2-4593-a5d9-4bcc090c7c23'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0943ec68-3b14-4099-ba9d-9340ee5788c1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0944c792-cee9-4c09-9a59-89a29b22b96d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0950f63a-34fc-4698-bf71-c1b379484baf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'095b460e-b3f8-4396-a371-72574e2ddbef'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'099208a6-e354-40c1-a33e-2ddeaacc8bf0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'09c5654c-c5f3-4b00-921d-b16917826819'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'09cc3bbe-9fca-4540-8fa0-85b2789ef919'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'09de9a0b-15c9-4d07-9382-4b1c111a1899'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0a2bc861-db6c-4629-b79b-b901abd69f28'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0a6cf86a-cfb4-439b-8090-3c215f38160e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'0a8fc800-3ac7-4d45-9e3b-fbe315258c9e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0aaf75ab-94c3-4429-bf86-cf3d180662c4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0ace6249-ffc3-41d0-8d57-4516108ac63c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0acf5b91-98ea-4ac7-9c96-a62cd3fa7ce4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0af35230-ba93-45ec-803b-840e177c48ec'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0afa0efa-b256-4220-b485-8f83be054656'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0b1b8000-da34-4b60-bdd1-a3da69e8ce5e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0b3add9a-0477-4b28-88d5-f3d1fbc24a15'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Approx 180log9 plts', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'0b7e3d3b-c107-4983-a111-38482eb1f0b7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0b869f4c-d6cd-41db-b5b9-2bf17c513931'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0b9faf1e-9413-43ad-9954-2e8a7cdf611e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'0bbe7109-89ac-41c9-9495-afc9b9b2889f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0bec07bd-1a3c-4d28-a4d3-c176fa32c08f'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@71 B2 C2 D2 JI', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'0bfa350a-0ecb-4f12-9685-eb7a6f4133e7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0c00eac6-34eb-4638-b03b-f2fb7c8ce536'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0c0fccbf-a89f-424f-8dda-96298336c160'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|4th container', ProductShortName=N'PLT AFR PAS-B LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'0c16019a-7df4-444e-a446-3f8e79073c5e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0c1bdefb-75d0-4217-a0bf-a128a2726769'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0c51c51f-0c4f-466b-a12b-57b09d63077b'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@CB' WHERE BloodProductGuid=N'0c5d316c-a3f3-4a3a-93c9-b93b933fb5eb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0c7ac9dd-513a-423e-af4c-83e2e134c5f9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0ca318d0-6de4-43ef-baaf-8fc46c368ea4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0cb088bd-a195-4b8e-bd5e-af716f5a41e0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0cc3d421-308b-44b8-9875-9ae23dd552b5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0cc635f9-6f2b-4007-b735-3bb4fb87bc30'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0cd789f1-8a80-4ee4-9fb9-c28954c188da'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0cde016b-de20-4cc3-9ddd-795addd57c61'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0ce82e23-43ff-459a-8ef7-a5898b869876'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0d06c25c-538d-4330-83c6-571f75e9cab7'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@BW' WHERE BloodProductGuid=N'0d3ab82c-b6d5-4ee8-8829-c503515bb548'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0d41dce1-9160-47e2-b1f6-325efdfcb1c0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0d5155b0-16b4-42c7-b5f8-4a441caf85a7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0d522fa6-6e0e-4295-91aa-7a870c879fdd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0d61466e-ad69-4471-88bd-30b1da76e16a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0d6263d5-e37a-4e34-be0c-b063ed21d987'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0d742a06-62f3-4ee2-a774-d80879beb70e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0d75cd4e-558e-44c4-8b9b-d6173a4e637d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0d90115f-7e42-4f72-93ab-805bb5c07d51'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0d93d8b2-9fc9-4b28-ba85-232832b4df1c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0df8383a-f81f-4d9d-b019-06e272987e33'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0e6af11c-db8c-4057-8ead-8249b78867fb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0e7f3d70-e68e-4fa9-aa39-4c31917ad06c'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@71 B2 C2 JI', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'0e9da76f-e5b2-4a81-a1cc-3baf0dd6660c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0e9e22cd-6bde-43ca-bda9-7a195d9a2e62'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0ea4dfcb-3eaa-4d66-a989-1117b485ff2e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0ea94c73-f4ad-48a6-9788-38975c633f8e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0ebc4cc9-9c24-484e-97de-dd4cafa3c403'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0eddc40d-e6e5-4c65-b085-febef1408c21'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0f1c906b-97f8-4f34-97ba-fb507838ae24'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0f20e6d5-a096-4c3c-bbfb-1f66bb014c13'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0f3c7a33-0ec4-4b61-8c29-1fd16fe67d61'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', ProductShortName=N'PLT Pool PAS-D IRD LUKOPR BUFCT 4DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'0f428cd1-eee0-4abd-a2ba-8395b5c5ff2c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0f6cd25d-0901-4f66-b38d-88a335f8f21c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0f8cc761-051d-4d49-b9b3-5ba2df7ce04b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0f9dc254-4377-446b-af84-b006728f6e9d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0fc1e452-faee-4b80-94f1-63e718f49814'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'0ff0d154-130d-49ba-85a7-df78df041909'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Bacterial monitoring', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BUFCT BacMtr', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.560' WHERE BloodProductGuid=N'0ff344f2-aee2-4dc1-998a-75d0c4b1f6de'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|2nd container', ProductShortName=N'PLT AFR PAS-B LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'100d6e4b-10a0-4ef3-b76d-74e21b5cccf7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'101a4ee5-b215-48b3-a030-d66f2950cbee'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@C9' WHERE BloodProductGuid=N'1029b180-8c26-49f2-803c-6fb4993c454f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'10662e01-3661-4297-bf70-134b4a269cf8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1069bda2-3abc-4030-b5a1-0d6f6ee44d56'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'106beffe-62ff-48b4-8048-dfb3e0248eed'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'106dcd58-ecda-4b01-b0a5-cb4e2fc5edfa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'107065fd-dd58-4f54-bf8e-5ec3be9d3fc4'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6', ProductShortName=N'PLT Pool PAS-B LUKOPR ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1074c283-04ed-4e1b-8b3b-4a1a9ee4d5f7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1084231c-3b46-49c4-955a-780abfb919e4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'10a96980-6e9b-4e41-90fa-3c5074206f36'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'10eed16f-1cd4-4d25-802a-0af1d79887e0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'110e9fad-810b-4027-91a1-cb52e6d27117'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1127d9f9-1197-410b-abaf-e4bd89a6cf6d'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced|3rd container', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'11456e2d-bc94-48ef-9710-9aa6fc987386'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container', ProductShortName=N'PLT AFR PAS-C IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'117368e9-e12e-49d9-9a36-143b10c670c5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1180fc0b-de14-4995-bea7-3a6287c2a673'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'118372cd-65e9-4992-89f7-c863286f7924'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1186cb05-f876-446f-b288-21e9bd70fdd8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'11a1fbff-6435-464e-831e-a4f91460f115'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'11adad87-b981-4151-bb3c-94e72ce66047'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'11b71298-3ba5-4d6c-9001-3b6b688c3d0d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'12113fe1-c24e-4898-97d7-305d146f3bf9'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced|2nd container', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'127b328c-0a03-4b80-87cf-a5ce47a56391'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1286007c-1c25-475b-bfec-30cca21037c8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'128fff9a-9887-4a4c-a7bc-1883ecb224f1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'12a04836-2e21-48c6-997f-18f3ea906c03'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'12a2b1c6-b972-428d-a761-086c50aad3a4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'12b6f813-4562-4122-9a89-a0a8cb7bbd9b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'12dc317a-6861-4ece-81fc-342273c1c192'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'130761d9-b4d7-4d26-adf0-102130211394'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'13320291-4e07-495c-bbc1-080b69d0cb0f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1351032f-2f89-43f1-87e0-ace8c8265240'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'13743654-83ef-496e-8741-37bc62510d92'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 C2 E5', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1394604d-90a6-4a30-b4e2-c4a6fc91bd37'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'13b3791c-dda3-4c5f-ac97-b0d1a75b821b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'13b61f93-eab2-42db-94d9-364009636d4e'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|1st container|Bacterial test', ProductShortName=N'PLT AFR PAS-D LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'13b67ec8-6409-4f42-920b-8154461d3784'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'13dc22a6-3688-4452-9c67-d211575b7926'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'14085b86-7a72-4f94-b11c-66686f7fccdb'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1412cbc9-696d-4b30-bd56-9f860130ef5d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'14182210-e040-4817-b534-33bdef54044b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'141c88ff-d3fe-40d1-a7af-51566cfa4c05'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|2nd container', ProductShortName=N'PLT AFR Wash PAS-D LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'14709128-474c-4dcd-8921-59555c791f54'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'14d7b56c-3950-47b3-b4ee-b14052039488'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'14eb34b2-0b46-4b5f-8fb6-7d5b55f93b0e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'15364fb7-1de5-40e1-9be8-1320d74f3e7c'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'153a3ad0-a746-430b-ab24-05ffda814e57'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'155fd515-7c17-46ed-ad28-f4755b871528'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1566e03d-dcf5-4104-af51-4ffb23ab7280'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'157562f1-a41d-4671-a49e-ccef4d0f6ea6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'15a61afc-fa3c-407f-806e-a62838867e22'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'15b9f538-00ae-4458-976d-5dfd24a0a04c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'15c6c05e-bb42-4377-acd8-40ad90f8e8a1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'15c940e3-3b3b-41e2-9774-a161937d2c7d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'15cac4be-17a7-40d7-9a97-b6d0143c719a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'15caf660-d200-4632-b1f0-54586322f851'
UPDATE dbo.BloodProduct SET RetireDate='20100201 00:00:00.000', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'15e68013-6150-473a-bbad-ea22782818fc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'15fa5e3e-42a8-40db-9ccf-076a94694dd9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'162135fe-945d-475e-a035-292d0d15159e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1633c52e-fc08-40b0-8fe3-7659e6f5522a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'16532084-5a3b-4582-9556-85fcd73631b8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'165ed9c1-30fc-4d9e-9448-6515d8b1b235'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'166c7e31-2ef5-4976-afac-ecdf106eda02'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1695a9de-1287-4ee6-b769-91d83c8f25ed'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'16d74f5b-6e99-4bc4-a918-3cbb5034682e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'17197e8f-14f3-4e7d-b21d-6921ce3ad31e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1757e914-d4ef-4595-ac76-02e05f6e0b56'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'17618b58-088b-4dcc-9fe1-41fcf70c7584'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'17965621-fcc3-4ba7-9119-56e8ed38a7c8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'17b2006a-e7d3-47ac-8aed-d5ae3c2a4dbc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'17b528fc-331c-4cfb-8b42-f2e47dd40116'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'17c38809-9f76-4144-b40d-48b845be6178'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'17c52a68-bc55-47ab-b9ed-c08853672534'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'17dba687-dec0-45d5-af28-2676c5b1b293'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Bacterial test', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BUFCT BacTest', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.577' WHERE BloodProductGuid=N'18002e5f-a595-4f50-b0f7-0f086ce5b20b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1801b881-7672-4262-9738-29bf2d845c40'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|1st container', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'18155b78-1915-40af-9e44-2f854e3fe8c4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'182333bb-d3b5-43bc-b931-ca42764840bb'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|Approx 240 log9 plts', ProductShortName=N'PLT Pool PAS-B LUKOPR SR/PA BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1845103c-c1b7-406f-b6ee-90883d542aa6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'185a7aee-11ef-4faf-8d91-cc0adafe18d4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'185e4d28-6bfc-4a12-b387-ceb6251786f1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'18776dbc-cce7-4c81-ba45-09fa4815f6f0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'188f0149-794a-4468-a3ee-6fccf7cb62ec'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Supernat reduced|Approx 240 log9 plts', ProductShortName=N'PLT AFR Wash PAS-B IRD LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'18989835-bdc7-400f-ae4a-649ccb8bdf8d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'189951ac-b636-40d2-9e9b-c6f77e0661ec'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'189cbb8b-8af3-49b1-98c4-12049a2d2465'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Bacterial monitoring', ProductShortName=N'PLT Pool PAS-D IRD LUKOPR BUFCT BacMtr', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.700' WHERE BloodProductGuid=N'189dd813-e848-4590-a836-0722c89e824c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'18d33287-f04b-44fe-bd74-98efa0a82363'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'18e27f36-917d-4fa8-820d-473afbd8ad7d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'18eb955c-5825-4bb3-b28a-eb276a05d2b4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'18f1a6b1-b75b-4117-81f0-32416dcbd117'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Approx 240 log9 plts', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1921b85b-59ee-49d1-b19a-f7cbba6d901f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1986d9cc-7b8d-4d5d-9f1c-cfc329a0c256'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'198d0929-2a42-46f7-b841-dd0b863329c6'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Buffy coat plts prep|Approx 120log9 plts', ProductShortName=N'PLT Pool PAS-B BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'19b6cc5b-4cca-4853-b9e9-67c7334ebb32'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'19b7b417-d315-4aba-ab33-a527a9c4a0dd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'19c2084f-8b52-46fc-a77e-3284f47df59e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'19c49ace-c1bb-4edc-bc4b-01e653771bec'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'19d23a91-718f-49a4-bd65-09700529481c'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Supernat reduced|2nd container', ProductShortName=N'PLT AFR PAS-B LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'19d642c6-9352-48cd-ac36-c482b66cbdbf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'19d94820-0c21-4b72-920c-3b1bcda8cc82'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Psoralen-treated', ProductShortName=N'PLT Pool PAS-C LUKOPR BUFCT Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'19ea31d8-a00b-4ceb-aecd-e123842fb167'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1a0afa4d-b338-4538-a0ba-669c3920c044'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@28 B2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1a0d467c-eaad-4c80-a002-92fd8944ddea'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@B3 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1a0fce37-dc57-49f6-a01b-eb219df6a735'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1a2b8832-313e-4083-9a38-c2e71ff70cce'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Supernat reduced', ProductShortName=N'PLT AFR PAS-D LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1a47c66c-a8ee-4d4d-bf99-5e25b2a5f000'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Supernat reduced|Approx 240 log9 plts', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1ad8b1c1-3290-40f1-b238-c829cdc10c0c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1af2564c-9c6f-48f9-a68b-26f5d18dec62'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1b07471a-8b08-4fad-9081-c315f4153e62'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|For mnf:injectable|ResLeu:<1log6|4th container', ProductShortName=N'PLT AFR PAS-B MNFI LUKOPR ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1b15c401-5fa1-4656-a392-32f1e5241dc0'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|3rd container', ProductShortName=N'PLT AFR Wash PAS-D LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1b173789-1ac5-40af-8c39-cfbcbd564dc2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1b5a3355-17d8-4406-979c-e7293875538a'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Plasma reduced|Buffy coat plts prep|Approx 120log9 plts', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1b5edff7-c219-4737-bae4-3ba59b41481d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1b777e04-2871-4f8e-a325-75cb2fc2779e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@30', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1baa8f36-7e18-4e29-a739-fe6eccdc3731'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1bc064c4-2e55-43ac-986e-a164f61b2d5d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1bd788b6-f0d8-4399-943c-5740f76dea1f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1bd966c6-554d-4731-ac7b-42ef570a7035'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1c153172-9fe6-4942-b1fb-2eb0a3b9b11b'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1c1b1cf7-d4b8-4ac1-a86b-d61f6319f822'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1c2b9664-db31-4e07-ab10-6b0ffb6808ff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1c2f9773-2b50-4c5c-bcde-5b9a1e4234d8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1c552948-0c72-48bd-bf63-a065443556e6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1c638525-1ea1-43a7-a9a7-9039cadad954'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@BV' WHERE BloodProductGuid=N'1c7a3056-5e3a-4b52-b886-524e0b2b1e35'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1cdce61d-e2de-4dab-a823-52f3a6268116'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1ce1ec0d-f53a-4cc4-8d5d-d8d8e13e424c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1ce680ec-290d-4a0f-b224-176daa50bd40'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1d052c80-e0c6-43e2-8c21-27aac9c582fd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1d1021b3-36d6-49e0-8164-8f47e9cdc3eb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1d170951-db9e-4b9c-b6ad-61e01076691b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1d19296b-cfad-4b55-a0dc-7806e07b8786'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1d3ca635-9027-4e86-8354-40eeecb1b01e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1d3f6443-0f93-44f9-beb3-182526cc5778'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1d428224-ce79-4a53-8846-70ac61aec25d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1d4d76dd-2816-4a57-b1e4-b1906e6e7307'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1d71d92e-0e36-4d5d-ad6d-896c68b8d012'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|Plasma reduced|Buffy coat plts prep|Approx 180log9 plts', ProductShortName=N'PLT Pool PAS-B IRD PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1da05e6d-4b35-4323-a84e-889c60de98f5'
UPDATE dbo.BloodProduct SET ProductName=N'Washed POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Buffy coat plts prep|From 3 donors', ProductShortName=N'PLT Pool Wash PAS-B LUKOPR BUFCT 3DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1dc7c9c5-572d-467d-9be6-e4296ee45bd9'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|For mnf:injectable|ResLeu:<1log6|3rd container', ProductShortName=N'PLT AFR PAS-C MNFI LUKOPR ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1dd0a140-3810-4860-ab03-eee4ee4ae965'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@33', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1dec6bdc-d87b-4e85-ad23-78c67d7ffe36'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1dfa73a8-c7fc-4598-8725-98352ddfe25f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1e057ad3-10e8-4702-96ee-0ea4acc15075'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1e198730-e33f-4e98-8287-2c04d527a6f2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1e2f7670-8d02-4370-ac69-9669f0abecd3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1e72708c-58ef-405f-a5e2-43b9c22a973b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1e819c13-062b-4299-859c-769001d26a8f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1e90c2cd-3dd7-4b03-b51b-fc6e56ebaf78'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1ec781d2-be49-44d2-8323-485fc7866fe0'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|3rd container|Bacterial test', ProductShortName=N'PLT AFR PAS-D LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1ee47702-f5d4-4c39-b652-f1502ecc8822'
UPDATE dbo.BloodProduct SET ProductName=N'Washed PLATELETS|PAS-B/XX/20-24C|Plasma reduced|Buffy coat plts prep', ProductShortName=N'PLT Wash PAS-B PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'1ee651d4-e689-4d99-895d-63bf050caeb6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1efebe6d-3dd5-4907-a16d-a365d3d339b2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1effcfb4-672a-4472-b7a4-34963d0682f4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1f02bc07-616f-4257-a607-c50f648a2118'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1f04fca0-4d4d-4fe9-9b29-ad6606686ef0'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1f1418eb-c616-4ad5-9e4a-10efaf6a9995'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1f1fda47-186f-4702-920c-16cbd7443978'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1f4ffa2b-e288-4148-925d-159bbbe3b358'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1f5143d7-c111-4447-9114-6f618e62fe0c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1f6a022c-c2a0-4b39-a8c7-5f8c26214dca'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1f7074a0-07f4-404c-bbbd-fa1e6c8f7412'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1f8e93e4-5ad0-4d61-9968-e08e5ec937f4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1f9ec33b-96b2-4045-a3be-560d0dc43df2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1fda0674-f85f-4b8b-879f-6dfc84cf55f5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'1fe050b2-2415-458b-a096-ff92c829d876'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|1st container', ProductShortName=N'PLT AFR Wash PAS-D IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2005cb12-112c-42e2-aebc-e4ae3fbc348e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'20158901-62dc-48bc-aa18-b0b63a0c4e19'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'202f6123-1fd9-4712-8df6-550cd4318325'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'203d40d2-32cf-476c-b9ba-c0ed01033b93'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'204b1bde-41a1-4a1f-9b15-56908ba2868a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'206b44d6-2029-4a97-8beb-990e9688941c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'207d4fe8-5958-4f8f-bac6-e5a3b0a8ca8e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'20e6cebf-78fb-4a5c-bd31-4e86b57c6c4d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'21053b6d-d86f-45da-b68b-fa7ac1cb3037'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2105d78b-912e-4d03-9f90-b07cae25f885'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2112ffa1-27be-408a-9286-c7031a33ed3f'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|Plasma reduced|Buffy coat plts prep|Approx 240log9 plts', ProductShortName=N'PLT Pool PAS-B IRD PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'211c064a-35a2-4658-933b-ce2737a6a871'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2141c9eb-4288-40c3-920e-def4d99f7bbe'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'21666c19-1e14-4593-8547-96fd297a5064'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'21812ad6-6e83-4bbb-aea1-e5fde7180da0'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced|Buffy coat plts prep', ProductShortName=N'PLT Pool PAS-D IRD LUKOPR SUPRed BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'218dd7c1-947a-482c-8787-e30f71e5b033'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'21a23093-bbd8-4857-8144-b7e57b130508'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'21a6eb5e-b3fc-4371-b842-ab5718e12e5b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'22045022-4609-4e5b-b231-a429f64d08f5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2205992d-bf34-4a39-a2c2-9cea40208856'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'220d73f7-f248-4adf-9ef3-a67dd24bfae3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'221e7807-eb7d-4e7b-81d6-4102da71f59f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2247349b-da6f-4b10-b80e-8818e62328bf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'22577100-dd39-46b8-9360-458eb3a5a522'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'22654020-f8ff-4467-bc74-a17cf0ca2f35'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'229ea480-7a91-49cc-ae18-c683c7b464b3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'229ead56-8692-4473-9b3e-cb2554efa02f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'22a35c3b-8f9c-4128-a62a-45b9b1f0f971'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'22b6942b-8b19-4f3c-929d-7df0cf4a0941'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'23221036-c095-477b-95a7-9a6a398e29d3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'23269258-6abd-4856-92c1-a9b8cd89fa3e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'232c85cc-e8d2-4e1c-a46d-2c675c1bde29'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'23593a8f-6d9c-4cb3-aece-b0759570b192'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'238e0673-1855-4dd4-9430-188cc914d9b1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'23df5bf6-3834-4a97-a202-5595f8097e19'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'23ea3629-be0e-4cbb-81b8-c33bc04e472d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'240a0d07-8535-4f6b-8ed6-a0390ba3f621'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'245ea715-8697-4517-ab73-72ec69f271a1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'249e1ded-8295-4485-9bc9-d8cf8d39cb69'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'24aca56e-dae0-4d60-8461-9b7cfdd2f3aa'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS B2 C2 D4', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'24cdf95c-4c1e-468a-8e42-93dfccf47bff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'24f4daa8-62d8-46e4-bab6-2a5c02d6c3d4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'25003ee1-c780-4ae7-aeb7-4ae65f8a90c0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'254d0eb0-6a56-429e-812a-cd98d78a1048'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2574d576-265e-41e9-85aa-22552186a374'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'25aa977d-21fe-40c5-84a2-00d53e9bfbf3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'25d9b867-87eb-47fb-8423-5eb880370da5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'25e3233e-bc53-4ba2-b852-9d97a0a0857e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'261251f3-7f07-4a7f-bbeb-a5092b29dc23'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'26144af9-e7a8-464b-b54e-6c936f37d77b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'264ce7c1-e2b1-4a38-b9bb-cadebe2c0466'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'264e5325-fcd0-4b85-8195-157c746c6051'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'265475c0-7e11-4f85-a1ed-0f34a02e7d3b'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@91 B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2660aaa8-1d1b-4688-aad8-6c20a215fab3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2673bcb6-e958-4fc8-ad3f-5cb7eeb334dc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'267a99dc-d61b-4d9d-9542-dc2c86e2e546'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'268bd72b-6f71-48e9-a41a-d49484d19715'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'26b87e3d-894b-4fbe-812e-4c7c943d8f84'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'26f1823e-f613-40f7-92ce-0464944b660c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'26f58b4f-6527-4d5f-b087-8350418b84b0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'272f2c08-3c3b-4762-8f67-9db8a85a6ee4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'274ce8c8-d014-434d-b5a8-37cdac1832a2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'27cc91cd-ae3e-4b0d-a2e9-d94283c3382f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'27d2a603-262b-4a45-9d1e-57df55402535'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'27d919b8-a58e-4094-a5e6-051fb7ff02be'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83', LastUpdateDate='20101201 15:05:41.810' WHERE BloodProductGuid=N'27dcae09-01df-46b2-a096-db17657b0c55'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'27e0d3fa-f3cd-4e22-a7b2-e6c1b0ab4fc3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'27e72916-ba03-4ef9-954e-fac68579317c'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'282b0805-c562-4792-9de7-18ae10b3be96'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2859c702-c8b7-4175-b9e3-40cfd7ba1b45'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'28adfc7b-10de-4016-83cd-a37c487f3825'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'28bbf6cb-1974-4631-bfd2-b9da07e8eeff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'28c028eb-9b34-49c0-a427-6aa8be5e22c7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'28dee929-2774-443e-b525-a8299afd7f9e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@BV' WHERE BloodProductGuid=N'28deffd4-b158-4807-886d-1ecdab19cef1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'28f82bfc-b752-49ed-9bcb-eeb62ed8c6f3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'292c8bcd-5e5d-4d3c-9248-92de12f7c526'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'293972a2-a3d5-4983-b020-4b3c466f6ba0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2941a2cf-84b9-4a43-b538-0fa06964e094'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'29474a29-22a7-4baf-8acc-e45f99e45a3d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'296535e9-92f2-475e-9884-aeb0c179dde9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'296a67c6-59dc-4ead-abfc-942051de2e57'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'29802d92-9e23-4c2e-9c0e-55c5f5c5e2e3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'29832e50-d63f-46a0-8a93-3c705b416c40'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|3rd container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-B LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2984c243-991e-4f20-a988-a779bfd97c58'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|1st container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2994ac1e-10b8-464d-bb34-a32f030cac57'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Supernat reduced|Buffy coat plts prep', ProductShortName=N'PLT Pool PAS-D LUKOPR SUPRed BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2996ca90-c390-4e2b-ac17-3444e42f05c8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'29a04eb9-154f-4e8a-a271-37fdba6fe2d2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'29b31f36-86fd-4e6c-94d9-3ddd376c3925'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'29bb1f00-3d89-43d0-a4a4-85dd88bd40ef'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2a579ec7-8c5d-49bb-beea-9487d0212569'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2a8c3d35-311f-4fcd-999d-23aa3f31f522'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2a98bacb-d59e-4f38-8efb-a353c954448d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2aa3dbe4-1235-47b0-8ad8-b030bc11010a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2aa535de-56fa-443d-bc4b-324c5267582e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2aac641e-4044-4b5e-8628-cf359bce23f1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2ad79c06-33f8-4e32-bfe4-385c5206c3c5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2afbedf9-f56b-466e-a7dd-9c5e7884a0ea'
UPDATE dbo.BloodProduct SET ProductName=N'Washed PLATELETS|PAS-B/XX/20-24C|Irradiated|Plasma reduced|Buffy coat plts prep', ProductShortName=N'PLT Wash PAS-B IRD PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2b40acdd-80bb-45f3-b45c-5f285bb22794'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2b478fe8-9513-4ee9-a9f8-ba2f01f7a627'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2b4eaff8-b166-4a36-8a7d-fe9d73994531'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2b5c14fc-1b91-46b0-bf46-2f3b86a61eb9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2b61b737-c1ce-4f26-b9ff-6e5b0e26423a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2b65d5b3-463f-42f6-9a89-00ff458e5ff9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2b6e4b4e-4687-4f54-a2ba-e3848efc86dc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2b6eac79-7025-4166-b18e-b9eb795e86bd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2bb528e3-ac7d-4d2e-a8d8-ac4033df2cc6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2bca9206-881a-4d5e-ab80-02b6a17b34fa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2bd168c3-d0a8-4b69-9616-003c5a24bae7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2be0d6f0-a111-4aae-8bb6-8b1c2e42e11f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2bfa7fe1-8884-4fd7-934f-639e3ee42849'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2c2565dc-f7cf-4fb9-b747-bc57b930874d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2c3e68da-2c3a-49da-b190-a017b353c67e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2c407b07-7923-4073-bcbe-8409c2121884'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2c48ef69-6120-4d4d-a6bb-94a4137527ca'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2c508663-ed19-4891-a68f-b3a6a8c294a4'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|1st container', ProductShortName=N'PLT AFR PAS-D LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2c855716-3aab-4129-bcec-640164fc33f1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2caa6b05-550a-4b8d-b513-3dd365d31de5'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Bacterial test', ProductShortName=N'PLT Pool PAS-B LUKOPR BUFCT BacTest', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.560' WHERE BloodProductGuid=N'2caee226-7e71-40ac-8a8c-d930f72cc668'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2cd71d3e-1148-4a7f-b6e7-c7acef30e2b0'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@BY' WHERE BloodProductGuid=N'2cdc43bc-9a6c-464c-9e5d-4aeb01421831'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container', ProductShortName=N'PLT AFR PAS-C IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2cdc51a6-0f6b-449e-aaf3-8a8b1e5838a1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2cec4c9c-40f4-4857-a4f7-31e351819424'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.653' WHERE BloodProductGuid=N'2cec8fad-440d-4be9-b851-dbcae43b9e6a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2d05e203-6a69-4145-9670-a1ea78bf60d5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2d1c426a-94c7-4e68-aaef-5f3c56758bd2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2d28426c-5bac-4a17-91d2-9e466ec422e8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2d48156a-25e2-4d79-a175-95a87a6d9c5a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2d57a7f8-6431-4369-be6e-89e6772cfb03'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2d6223d2-e185-42c7-be4a-c3fb0cca3217'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2d813e55-2ce3-41e5-b6db-00129a6467b2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2da08ad0-92da-4e80-ad5f-7aa535c86901'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2deb4d14-3fe0-48a0-b110-fb818c28f843'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2e2f000b-307b-4441-9264-e8d3083be5a2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2e878018-940d-447a-8c37-a89c396a8132'
UPDATE dbo.BloodProduct SET ProductName=N'Washed PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep', ProductShortName=N'PLT Wash PAS-B IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2e91fdf8-fe95-4f67-9580-3385cb73f349'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2ea783b0-92a7-4627-877b-83be9f6f04e1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2eaecd73-c6a3-4ad7-a9b9-dc8c5edfa97c'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 B2 D2 E5', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2eb0e0dd-0ed3-4296-bcf3-ab59da3ffd99'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2eb29654-b37a-47bb-b249-6f25fe972f25'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2ec9674e-7e76-45ff-b3f7-c1cdcb29b4c8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2ecbe557-5a35-4d41-a0b2-e8d44bafc9e6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2ecfd3c7-20fd-4a2b-94d9-3fbea0f52beb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2ed7646b-83b2-4112-9f1d-bc021e79d3df'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@33 B2 C2 E7', LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2edb531f-a44f-4da6-8b69-5288919b2696'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2eea71d7-8a69-4329-b1a5-9fc6f1ca8c88'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2efd8127-2fe0-4b20-aa28-599e4185aaa6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2f163092-d080-4e28-a987-07feb40f40b8'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|2nd container|Bacterial test', ProductShortName=N'PLT AFR PAS-D LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2f1e330d-45e2-4a0c-b26d-2d68a5ea9cde'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|4th container|Bacterial test', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2f3015eb-3e1d-48bb-a798-140409b38367'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|2nd container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-D LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2f334487-1c28-4698-a2e1-b186553ef65b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2f397523-2966-44fc-85de-03c1a47007df'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2f3b7ecf-675b-475a-a097-77f99d6710a6'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|For mnf:injectable|ResLeu:<1log6|1st container', ProductShortName=N'PLT AFR PAS-C MNFI LUKOPR ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2f41d463-8e01-41b8-af7e-bb6747f8eeb5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2f5e2a9e-0f56-40bf-a72e-bf59507b8686'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2f6e56d5-7c6e-4e5e-bea0-198740c6507d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2f76d8dc-33e9-40c8-990c-a40404acf5f4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2f85405c-54e7-4fb1-b1ea-4a6995a3a103'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2fad6f6b-deda-40d0-aaa7-562f55b7161b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2fc432c3-e5c7-4e6b-ad9b-1ab37d10c4c4'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|ResLeu:<1log6|1st container|Psoralen-treated', ProductShortName=N'PLT AFR ACD-A>PAS-C LUKOPR Psoraln', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'2fcdbffb-9ac9-4511-9dc4-3fa19de17280'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2fd3771a-c774-40a7-a29f-192e5c36be87'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.547' WHERE BloodProductGuid=N'2fe14a2a-5147-456b-99ca-99e944c09a0a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'2fef18d3-b0d4-4c32-a2c9-3c4767895d89'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'30027f88-86e3-4d10-8252-d48dbbe3067b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3006fd1e-bc91-40a8-b043-31a5fe4afb25'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'302c38cb-a026-4c97-b89e-69d1ace5d85e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'30483d1d-ca14-4b1c-818f-44e52cb68ca6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3074df72-0f66-4b35-9c79-91b42d4eab69'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'30a49be8-b309-46a8-a3fe-b47431ce450f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'30ab7c8a-3658-4eca-ad92-7041d6faa9e1'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'30f8d008-dfc1-4a94-a2d4-5cfa36cc21f4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3142be1b-b0e2-4619-91d0-248722903a93'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'31810906-edbe-4458-990d-888b86df2f1f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'31a817d3-080b-443d-84fa-3be8f77ad810'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'31c1c4fd-7ac5-4ea4-9c87-ba7163778bb5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'31e3a7bc-cb84-4584-9a9a-ef901dbdc4b7'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.687' WHERE BloodProductGuid=N'31edff07-e143-492d-8256-d889642fa51d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'320ce080-6902-43df-bc7e-6cd47836bb89'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'32312174-f2d0-4447-ace7-6c969c236fe5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'32396fa1-d38f-4c82-8f4a-559aa68ddf5d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'32639b20-3ff4-4025-aa90-afb15fa1be22'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3272894f-b971-4aa9-919d-4b0b3fe7069d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3286b870-0f16-42ce-964c-e3c5125ff7ce'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'32992e70-0ac6-4f58-86b3-f218fca9a3e5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'32aa43d6-8ba9-412e-8fd2-7e2b0b92df27'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'32c33ed2-a36b-43d8-a13f-3abae7352f54'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'32d7c23e-87a5-4cc9-ad46-c23581b96ecb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'32e0cb4d-cd0c-4022-91ee-c367d36e3ee5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'331bf566-646e-4b63-84e4-cf3d962f8348'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3321d571-2959-4db1-96c2-f8ebd81166ac'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'333f725a-d062-424c-a38e-b746ef95cf5e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'33514860-9391-4cbe-88d9-2efc51dadadd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'335575c4-91a4-4d5f-8644-cc59c3ecdb50'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3355ac27-6a95-4254-85aa-4371180a9411'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@B1 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'33568303-e837-4424-bf7c-c75a2d335caf'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Supernat reduced|3rd container', ProductShortName=N'PLT AFR PAS-B LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'33a9f2de-8c0d-4192-a814-de37cb79c5c6'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.733' WHERE BloodProductGuid=N'33ae431a-7cb4-4355-a1c1-a034ee54b537'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'33c41065-194f-4a58-80f4-ffa8629b56d7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'33ccd329-420c-41a3-a1bc-b46a422952b7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'33d0ef8b-33c7-4dec-bd60-693d41fe720b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'33d6cf37-4501-41c5-ad6e-2d54621007ba'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'33de86a1-e01c-4b6a-8071-0b3bcdbe486a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3407aa67-1ae3-423e-a4fb-4c9ebc8d7554'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3433f54a-ed90-4839-b348-48738d523ddf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'343f48bf-c131-47f3-8204-baac4ff8319a'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Bacterial test', ProductShortName=N'PLT Pool PAS-B LUKOPR BacTest ', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.623' WHERE BloodProductGuid=N'3466afb9-c4c2-4b19-a519-f5b193024f2a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'347463af-edfe-4039-8c33-51db59a1ed63'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'347a244d-4397-437b-bec6-10e86be6efc5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3491c4bd-027c-499b-b9a5-900a544e38a5'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced|4th container', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'34b348a1-2501-45d6-80ea-a344e704f022'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'34cb1e09-ffba-43b7-bdc9-717a960f46bc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'351ee22a-9457-459d-b8dc-d1fc9dbc94d0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'356af699-3b07-4bc9-bcbd-18fcf94aac8f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'356cf127-7eb2-4130-a173-587b75deca08'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3588499e-907b-4e91-9112-b9a388bfafcb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'35c91307-faf8-4a74-87a6-6a2840a075f9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'35ce5d0f-d5ed-43a8-9be6-6641f4ec302d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'35d2b26c-cb13-4984-84fb-0f7828afedf0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'35e749ef-c166-4764-936a-2cefaabafc0e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'35e768fc-17d1-46ba-ad6e-430e5241c8d5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3615970e-29ea-4e5d-9d92-c2bd3caca143'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'361cbfbf-d3ad-4a10-8f36-3cd9f760a666'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'362c7e13-6f0a-4f2a-a9d1-e019674f7fe7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3637a9cf-e394-4440-9241-553fb31ce21b'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@17 C2 D2', LastUpdateDate='20101201 15:05:41.483' WHERE BloodProductGuid=N'36420e85-cfe3-41b1-aef5-8c2fd83db504'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3643828a-7742-4898-b9ba-62a4939567c4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3650dda4-7299-4119-836a-3a01ba3bb1bc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3664e118-47e6-492e-ac94-f99daac20f8c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'36672499-9a8a-4834-8a63-f99cb959bf2e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'36a0ff79-8cd6-4627-ac7b-aeba47e58906'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'36bb2f47-75be-41dd-87cf-0b4334f16f1b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'36df9a97-a579-473f-a475-ec840dd8b666'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'37094f89-050a-4045-9cd0-5cfac57341fd'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'370ee9ab-e1d5-4a13-85fe-adef3b4fa249'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3724a81a-1d75-47bd-850f-7698d535e326'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3726f905-daf6-4783-8b0d-8479ef948f3d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3764a09a-26c3-46f0-8c4c-e88879929308'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 B2 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'37770885-8336-4111-bdb7-8693e53e46b8'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'37d1dc6e-1129-4528-ad84-8027570ce513'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'37ef2b91-0e6b-4517-9fc0-2b9d84a0b870'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'37f2d02a-3b42-44a6-b934-0d795489061f'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 B2 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'380acc8f-ea42-4e97-87c8-fc79408b3c29'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'382bbceb-ef0c-487d-9fdb-5b9fe701720c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'38414e6d-17e8-48a7-a6ee-ca1f682d1012'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'38525c45-600b-4384-a730-9fbb3e200a50'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'386f2344-eab9-4d8d-b879-f3d4da67013d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'38e31278-5a2d-41da-8242-d669f0456158'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'39092792-ad98-4009-b9fb-610e47bfe1e8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'390f0eb8-c212-4c35-bc2a-5327926c98e2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'39230702-bf45-4cee-a378-9cfe8c588fcd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'39255f80-5186-47a8-a7fd-f07dc0a81c2d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'393a3721-4cd4-4650-89a3-aaba7965d481'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'393affe8-7cf5-4941-844b-7ae1222fa490'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'394dc7cd-f758-4f5f-bf9a-a4b720eeed16'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'39507886-95fc-4d99-ad5f-f688fe8f8f4f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'395a8eed-5db7-4293-b5ea-bc58d8289e81'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'39b1349e-7bb5-4fca-8e3c-987fe97059c7'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced|1st container', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'39b7f594-e39f-475c-980a-77fd793fc0d0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'39bd5148-c581-4434-9698-1edfbb4bf215'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'39c4f586-597e-46e1-bece-5acc5e3d44a2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'39e4e9f2-4b8c-4fa8-8032-d21bdc943f64'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'39fbefa4-8974-4a75-8131-3cf0da3236c0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3a008efd-75e2-40fa-a0da-9555393c3299'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3a15ec67-42de-4aec-9bc1-a0c901bb980f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3a655f1a-15b2-4a30-88a7-26fe763860bf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3a6da786-07cf-42d7-8e47-fdd79a32c224'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3a7f63da-9095-46d0-9811-f8884f0ad0d2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3a9c4936-4c8b-43c3-a216-e4776e51ca13'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3a9d0a1c-a857-4ab3-b452-6817ad963419'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3aa74b6f-2324-4598-ba6b-e6fa96f118bc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3afc5dc6-47d1-43f6-b4eb-045c381e0334'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3b01d3e0-c69a-474c-8f2d-8beedd1b8832'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3b05248d-c73b-4700-b35e-ba4590bf36c3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3b20eb7c-b810-480a-8009-6380d1cfe506'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3b5488f4-f037-45e5-8561-66e85354efb5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3b6d5d75-cda2-44e4-a7fb-82da72b09f52'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3b71e033-71c2-4658-8e50-efd8a6ca1ddf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3b8effc7-3631-4863-bb7d-14180c84e29b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3b910da2-85f1-42ba-a184-4fde336d7800'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3baab6d8-f846-4268-a758-9aea14d30d7f'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Approx 240 log9 plts', ProductShortName=N'PLT AFR Wash PAS-B IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'3bda9e16-aa99-4fd2-96f4-50068b051cf4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3bec6317-1aa3-48e9-824a-0c3c6b5ee681'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3bf00516-3514-4541-b31a-200afe4f28f0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3c07350c-8773-4ed4-8664-1995677ded5e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3c0cec08-697a-4631-8abc-3d01aa55ee74'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3c326cea-b961-401a-917e-c5dcdafa09a2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3c42dc86-915c-4a39-aeef-473d8c038fe6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3c645895-f94b-43e5-9d61-d6a60783f97e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3c87563b-07ce-49a3-ba26-78266510d001'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3ca31adf-080b-4f8a-97de-2b9594263df6'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@33 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'3cb6ff05-3f4c-4917-bab8-967a73a3adb5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3d11d61d-2340-4216-9fc5-005717f2418a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3d183c31-6e4a-4312-9db6-108fe33c5de3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3d39bc60-6766-4dbf-b4a5-fa36b888dd5f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3d8f9df8-dd8a-43b6-bd21-181cbc5390c1'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|1st container', ProductShortName=N'PLT AFR PAS-C IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'3d9a084d-b77d-4648-924d-acfe242e5728'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3da147b5-8957-4b92-af41-98084084382e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3dec1afd-7f29-4b21-80f8-2a692b8f7987'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3df4b734-c97a-4234-8242-46fa596e9dd8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3e2d54dd-0f29-4e23-a7a6-733925a20879'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3e49a2f2-c04f-4871-a87a-408b6a8d5b86'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3e57428d-e789-41c9-8990-8efca23bae51'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3e7f25a1-139d-4795-83c6-a86b89a2e9a1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3eb5a15f-26ba-44e7-8cc5-5054f2009257'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3ebc6bf0-db71-4eee-af61-c6953859c3f7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3eeb70ff-85c7-402c-9f4a-3c3ab27f0f15'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@33 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'3f0f654a-195e-4838-8053-9cb4773b07ca'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Supernat reduced|Buffy coat plts prep|From 3 donors', ProductShortName=N'PLT Pool PAS-B LUKOPR SUPRed BUFCT 3DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'3f30f65e-f588-4c91-9210-43f9f2137b4e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3f944d58-2202-45d8-88ad-d0527cf08b7d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3f9881ec-dc14-452a-a447-77e0682df6e2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3fa08980-1faf-491e-9cd7-fc5d33bea369'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3fac80b4-2816-4b95-844e-811494b8c92e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3fb2121b-7d19-484e-9521-49b50bf0739a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3ffb0a1b-f382-4a82-9ab6-1f2e789e21c3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'3ffe8d6e-0879-4242-8c34-5b4befdc53fa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'401a17e6-b346-452c-b2d6-a3a7c9a28c09'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 C2 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'402384a6-c219-4609-b7d4-a1732c5701bc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'402e54d6-d110-408a-b84c-2b9834e11012'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'40301dd3-2858-4005-b3ba-17614552107f'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 C2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'40528d7b-7d6e-4c4a-946b-90fea38dffb9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4054a915-7151-431f-aecb-ab304b163d30'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'406ef830-0fd9-4f8f-ba01-5345ac1b253b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'40815711-9857-4779-a88c-3fd1742ce619'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4098c277-3994-42e6-9750-b75b6c568002'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS C2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'40a045ef-11e8-45fd-9b0a-06c7fdb58f2d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'40b0937a-557c-4115-99cb-61702159e816'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'40e2bceb-be3f-40ee-96a0-b79bc2dc0cd3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4108f7ed-af4c-412f-afaf-c78f77943dba'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'411128a1-1f7a-48dc-a426-eb35de30931b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'41285d4e-7c14-40ee-b242-e97a1f989293'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'414a7e92-430c-4aaf-a5e0-f3e036dec2ee'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Bacterial test', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BacTest ', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.640' WHERE BloodProductGuid=N'417381e3-02f1-40e0-8e00-44a9fc4ec88c'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Bacterial monitoring', ProductShortName=N'PLT Pool PAS-B LUKOPR BacMtr ', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.607' WHERE BloodProductGuid=N'417bb220-edda-4ced-963e-d0ad314c3ff6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'417cf047-87c2-4c02-a6cf-0ef0b1648327'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'418485ab-9ff9-487c-9469-8ffb15b54abb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4187b787-7881-4f32-8b5f-35255a09a7db'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@71 B2 D2 JI', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'41888622-6f2f-4996-8e8a-c51bc75071bf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'41a55963-5aa5-4dc0-924b-fef97e8711ed'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'41afc28f-1a9b-4ed6-90ee-255fb695492e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'41b968d2-b0cc-4193-a09f-adafd9507bfc'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Buffy coat plts prep|From 2 donors', ProductShortName=N'PLT Pool PAS-B LUKOPR BUFCT 2DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'41bd4a9a-681f-4cfe-88c2-79d9ef984223'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'41cde27e-195d-4ba6-a022-d8ce335a29af'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'41d57687-dded-41bc-933f-ee257ac0f989'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'41e33aeb-d1e4-4c6f-9f58-376c95f10966'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|Approx 240 log9 plts', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR SR/PA BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'41e35e26-773b-44e0-9a7a-d276636abcf1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'41ed1fde-15cd-4e3c-9c4d-30b6c34cadb3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'42549e7b-1ec9-48fa-873d-c40eae7f0ebb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4282bfba-de31-4240-9210-b7c6838393fb'
UPDATE dbo.BloodProduct SET ProductName=N'Washed POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Buffy coat plts prep|From 3 donors', ProductShortName=N'PLT Pool Wash PAS-B IRD LUKOPR BUFCT 3DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'42920d66-4654-4566-9881-8dbbf033eec2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4296af12-7915-4a08-93b5-03d48f57d791'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4299de09-4d45-4718-9da7-6b31dc8e101b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'42a8d362-8fa3-466a-b808-4e480bc0e309'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'42ad3044-8e8b-4944-847d-a685b0d249c1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'42c318a5-94e4-4c58-bffc-b129f3373f04'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@17 D2', LastUpdateDate='20101201 15:05:41.483' WHERE BloodProductGuid=N'42d2d69c-6841-4716-a874-80a98cfbbc33'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'42e4c7ee-f86b-41e3-b9fd-e40185b55739'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'42f53b8c-5df7-4450-8177-5d52a95e5d10'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4317fe01-af0d-4b8b-88d9-c702ab17e9cb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'431bbd61-cf7c-44c2-beff-81595cb7d148'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4327948c-7ede-48b3-9e5f-9c9cb7d07d91'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'436779e0-5f74-41be-9d00-50ea8a5cc5bb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'436c3da3-9f68-4e60-b6b7-a7fc63caedc7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'439762aa-133e-46a0-9fa5-39e13a5d40e6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'439d2e02-1626-464c-8a28-9a88d8248efe'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84 D2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'43c7d099-4427-4e2f-ae18-0e8da78575c3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'43e3728b-f86e-4da0-9eeb-8aede3c0c594'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'43fe8568-c2f9-45cb-b243-5876f9db4807'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'44516a3c-6f0b-449a-be0b-394eca59f154'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'44549de0-6f50-4449-a81a-484977906277'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'44bdba38-7c5f-4149-a74d-462b6a17ce2c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'44f39195-b4f1-4a8f-ba16-01390a61e42b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'450f7c05-1d88-4eda-9982-d8cc9401e4d4'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Approx 120 log9 plts', ProductShortName=N'PLT AFR PAS-B LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'45145b93-8eab-4b60-aeb0-025604039a24'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4561e0f1-1106-45cd-a6bd-0d30c239e816'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'456a36fa-be34-4eeb-bcc6-9448313d770c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'457ce649-b6d3-424d-aac0-4fafef37c4b2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'457d67fc-3204-4f1c-a842-9bbde0aa2536'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4582a2d6-a073-4dc2-aa28-b0557b4399f3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4583965f-2763-4941-b36a-18072c56ef13'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Buffy coat plts prep|From 4 donors', ProductShortName=N'PLT Pool PAS-B LUKOPR BUFCT 4DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4584b764-82f0-4268-9ebe-49eb4acfc87d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'45986b58-4056-4b98-aa84-98da00a7f643'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'459cb6fa-349b-46f6-91d5-0a6115123f02'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'45ca564f-98bd-4b30-993b-415d4abf5082'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'45d643f8-ec29-4ecf-8eb3-0e5241f9fecb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'460dff6a-bc84-47c8-bec3-9d193bc33941'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 D2 JI', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'461b1167-b838-4a36-bfba-a47aad362520'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'462fe04e-f9b7-4e5d-b6c5-de7671d5fffa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4638f8bc-5694-4ee6-8eae-67145a065b7a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'463e5fc5-e4a6-44bc-9653-820121a85ca7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'464b189a-a928-4895-952b-33b1c6c9b0a0'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@66' WHERE BloodProductGuid=N'466bbb57-8ebe-418f-90d4-67230680201e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'46980d02-02f3-48e7-89cf-12594850ba51'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'46a94c0b-fb3a-411d-bbe3-54603098f524'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'46beff0d-9af2-49fb-bcbf-3dd56594e10f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'46d40072-8b6b-46fd-b53e-426ab055e8db'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'46d6fd0a-7fbc-4983-9f99-7fdbb3b9565f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'471139e8-6c08-4f7d-a053-f9b5a53ce529'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'475694f1-c0a6-475d-9801-3e2e4473b362'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'478cd783-aa91-49c3-a07e-67ad4269d3e7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'478d7db8-f205-4fab-acf3-94c7a8cc0aba'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'47a37d36-63a2-40b7-aa55-ea8d7b75c90e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS B2 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'47cbe092-3780-4a9c-9f8f-e9edabc687f6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'47d6d1ae-664a-4c2a-8fac-516e0f08ab4f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'47e4d46c-522f-451f-bfff-aef02629281b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'48110eaf-6267-46c7-a91c-b2d2c26ee3c2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'481dd0a3-7430-40f9-a2a7-1e7426ac5481'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS B2 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4825f31f-73b0-497a-8d3d-d47d80b1b011'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@33 C2 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'485e75f1-06ba-4e5b-ac24-ad10a3e7aa11'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4886dadc-d438-41aa-9e90-c9e433a7d06d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'48be3170-83c6-4caa-bcc6-7a17a57ba722'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'49151a2d-3676-4d33-8081-18506e6b3e3b'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@71 A3 B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'495357ac-8120-41dc-9287-616fd5920d1d'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Approx 120 log9 plts', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'495709b4-3316-4c5d-9844-799e20ab8ce2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'496bf7f3-df40-4eee-ab1c-7e2161e27582'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 D2 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'49820f7e-a772-41ab-9c2e-83ef6d188436'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'49962165-a383-4135-87b3-e408ed7219ca'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'49a703bd-d3db-4264-a0aa-c1874ff54d0d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'49ac7763-975e-4ab3-bbe0-7bc173f0cde5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'49b43135-1c11-4351-8843-3ae6d5965193'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4a0d6dcc-39dd-4983-b72d-cb7f4adbe2df'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4a477929-0d66-4df8-aa55-696f95c17c3a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4a5b9f3f-60c2-4189-97c7-7f78d0d1c55e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4a8e3a48-ffc7-412d-96c4-87595f283ba5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4ac4ccf2-6063-423b-80a1-613440b25fd1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4acd6f8c-7989-4a8c-84e4-887a8d7333c6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4aed736d-272f-4e8b-8727-7ea994f6e851'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4af44482-ce0a-49d0-9d44-0b92e740c3cf'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Buffy coat plts prep', ProductShortName=N'PLT Pool PAS-B BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4b333983-0ac5-489b-b962-e1772ddd6b86'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4b4484be-3e51-4d59-836f-b0dbd028ca27'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4b4f840a-783a-448f-ab7d-471868933f69'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4b6bcd04-0a4a-48c3-ad9c-605548ecc7b1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4b820582-5979-4757-9b03-584387d7b2b2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4b883bdc-3827-4f26-8954-3970b155d9cf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4b98ad24-268a-4e4c-aa14-e774d3bb6c42'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4b991fe0-5647-47d1-ae50-4bf5c596664a'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 B2 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4bb4c8aa-67eb-490d-bfd2-bf94c778ff7f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4c026fd8-93f6-45e2-bf4d-1efb62472aa6'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 B2 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4c04ca70-08a2-4aa1-9601-b4b33de61198'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4c174702-8043-4822-bfa7-f0ea78b7e785'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4c192ac6-f21e-45e4-b81f-062baeead28f'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 C2 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4c1de5c1-32dc-412b-8b89-c2e145d2169f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4c354ad5-09bd-48b0-864a-6273b7ef3db3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4c56cd1c-0c4f-434b-8b4c-b306db578dfe'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4c5eb64d-99f7-4a46-b18f-b6ec026f6f3c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4c62108d-b311-4f04-9e05-733cf8f52976'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4c6d4502-f1f4-4d6e-a318-f10d6c6d5006'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4c89b253-3db4-441d-9211-c673297ae8f2'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.640' WHERE BloodProductGuid=N'4c9c0ae4-5c7b-40ae-b5b6-5f721fba20df'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4ca4fe3c-ad68-4953-a074-7cab31929df7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4cb497c4-9ded-42e2-8f54-5092b7f11cb1'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|2nd container|Bacterial test', ProductShortName=N'PLT AFR PAS-B LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4cbc5013-f0bd-48f1-85d9-a8cffcbb7c03'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@71 B2 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4cbfce73-6bf2-4dfc-8237-5f2361e7bac6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4cd1594f-c1cb-4a21-8c39-32102ad052f3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4cfe9a37-ef7c-49a2-9e28-27a8edf17c1c'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84 B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4d1c2265-464b-4832-a9b7-91393f9a1605'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4d2421ac-161b-4477-a0a9-ade0c053a0e4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4d503c75-86e8-4c68-8226-f36638c2058b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4d53a334-667b-475c-a334-c9c961ed9edb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4db99aed-5555-42bf-a595-485ebf6c1b1e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4df436c6-1484-4765-b5b1-0cd578aece94'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4e090380-0909-4490-8d01-691e7de4cebc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4e4beef6-3882-4275-94db-50519ab737ea'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4e588d2b-9564-44a8-9aea-4338600f4040'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4e614a0d-a045-4676-9037-9b05f295a889'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4ebf8b74-b207-44b5-b03c-96d6115dd084'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4edf00bd-8267-4ab1-ba6b-ab983649d249'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4ef86afc-4c8d-4f9b-840d-c839fc319152'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4efee2bf-52e9-4cbd-baf8-f6666ede326f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4f0cadb6-b070-4192-8fbe-29f6f0887c3f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4f1a32f0-b76d-4636-9a5f-7cafd8bdb4aa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4f1f5d42-65ea-4427-b409-6605e7f7f401'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4f512fe7-f423-460b-a2c4-4a532fb93f2b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4f514ea7-9325-44a9-8076-a7ca9fc045d5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4f5e011f-ba52-4aaf-9e70-f9973686fd30'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|3rd container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-D LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'4f609bce-1994-43c0-abc5-00797f5ecb87'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4f67d392-2962-45af-a4ad-24c6331c41a7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4f7f276b-65a6-4614-ac13-2bac4c8ea5f5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4f97c474-2fef-41a3-b154-a26ea1e79756'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4fcbcd10-9811-48ae-9685-726c78542128'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4fd22a2d-9df4-4bc6-a9dc-e19a95b34e71'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4fda635f-eddc-4415-878f-7d19df846388'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4fe0416f-30b1-49fa-a9ff-a0272f7448d6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'4ffe5ba0-9c14-4dad-81a8-9157e2f2ea44'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'504988fe-2ed9-4a3f-90b1-d5c3c110497a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'504f9c4d-ae38-4030-916e-156016ab5a93'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'50530d6a-6ef7-4e96-9f04-208ae2a6ab84'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'50548ece-396e-4a1e-beec-28c7f0f7d180'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 B2 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'50565f6e-6fc3-4513-8633-1a00b8491c35'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'507c50b7-f513-4ce3-84ca-d074f924d2d8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'508bb37b-8e36-4af4-aae0-f58407f76182'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', ProductShortName=N'PLT Pool PAS-D LUKOPR BUFCT 5DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'50c028fb-26e6-4c43-99fd-612721a4bcc7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'50e6fb03-c842-46c1-b91c-928eb6e2b183'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'50f39e24-2da1-4b53-beb0-b2937b6c8563'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'51245be0-d1bf-4cfb-baa7-f46b788051f6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'51307382-8880-4505-9882-398bb961bdb6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'514a9616-db2a-4048-a05d-44e0cc5a0496'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'51576c28-1c5b-4cd9-a0ca-05321922ad1b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5159e325-fcd2-442e-9ae9-46bd8f4119d9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'515eda3b-78bb-4a6c-98c1-0ae2069f0d1b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'516eb5de-e885-44ae-92e4-22d9b7ca4b84'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|Supernat reduced|4th container|Psoralen-treated', ProductShortName=N'PLT AFR PAS-C LUKOPR SUPRed Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'51726ffd-6477-4764-bb44-c1c2e2e61992'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'51a58867-8a0b-454c-a0e9-44994f06f94f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'51c6be48-eb73-4e3c-82df-5d06122f7559'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'51c83ccf-ad59-4c94-a616-fd3c4c032e4d'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 A4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'51fd2a37-e15b-4a43-861a-ceb93b3f8f71'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5259610e-b12f-496e-afe3-b64197117319'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'52786332-5eb6-4a3d-8813-4a718a1f8dff'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'528d87d7-d109-4bc1-a45d-34c71ad9bd09'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'528f22f7-995d-454e-ba60-42d842581278'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'529170f0-93ca-4545-a7af-2670226ae260'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'52b71650-fc98-4d3f-b8e7-1008eff40988'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'52c26247-f433-411a-bb1d-f17b56e3eb2f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'52c9e9b3-c7ff-4dae-8806-41a9e72d69d6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'52cef603-d3d6-424e-b57d-d951ca83bf66'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'52ec8622-7271-46d5-9967-01d6732ea52c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5332a014-c81e-45eb-a1e4-7fa35d2c9359'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'534c0bfd-3d69-447d-95b3-55ede3a2e4f4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'535ab155-e2ac-4736-ad65-3660b599234d'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 D2 E5', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'536da362-8e5d-4fe0-bb41-de38fcb5eebd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5389278b-1105-45ce-bb0c-2f0df1aed236'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'53a4a1df-c04e-4f0c-8fd1-838cfa34519c'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.577' WHERE BloodProductGuid=N'53aa4a86-d561-49aa-80cd-a131d269c136'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'53b732df-a1bd-409c-8f70-87814851b168'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'53c6d384-c7ce-437f-addb-c8e0ec9baab1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'53ce0a4e-49f0-441b-b486-b02e438ad2a0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'53d6d1e6-fc1e-4e17-aa43-d12e7f991281'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@B1', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'53e02c32-dc97-45e9-a4fd-07546948e9d0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'53ff38d9-969f-43f6-9e8d-48df7c609b8d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'541a5793-8fb0-4836-9ab4-984c8ebea224'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'544cba9e-d136-4fa6-81a8-2f46081c46e7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'546b7617-2038-47f5-882f-e26fb9db98a9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5473e61b-7b8f-4acc-90cc-77cac2508f7a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'54a02163-73d9-4a8e-8b41-955f40dadc66'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|4th container', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'54c8c9ce-c67e-420b-b3eb-12ce8674bb4c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'54d7a051-0cac-4f01-9308-84f05051700f'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@71 B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'54e3d5ba-8351-4927-9dbf-036bb1282a4e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5512895e-a9ae-4d38-8120-cce553d5d91d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'555c9685-f73d-48d9-be7b-628f6cda4570'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'555d6e09-f74e-4085-b83b-86f5c915714e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'55826e89-b3f4-4ea8-a614-b00aeb034981'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'55896b79-c984-4013-ae43-675c0052f68b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'558ac514-a74d-4c39-9dea-5c66c680b41e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5593c188-6515-445e-a48e-1624e035f596'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5594c550-e72a-4943-9399-edd87a4810c3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'559657e3-155e-4d50-91aa-43c4013aea74'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.687' WHERE BloodProductGuid=N'55c94a59-553f-4876-a811-30f161c6802a'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|1st container|Bacterial test', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'55db4ba0-e37e-4aa7-a89d-fb4a724bff51'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 E5', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'55dc3591-7b96-4acc-a337-7e193c3c1365'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'55e12681-1108-4796-a43f-26f68a8dd88b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'55fbdb82-88ee-4855-bf44-270d06ca8766'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'560873a4-937b-4694-ba67-20831edbd2e7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'56161580-fe11-4081-be8d-51b82a246444'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'561fcab9-074d-43f2-930d-7176c557a57a'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 B2 C2 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'5622b9fb-3ca1-432f-9c00-54ef0271e30e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'565cd2d1-a2ac-4d31-916a-1c4015573b2c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'567b34d0-fabe-43fb-a9f2-f7cd352966e5'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.640' WHERE BloodProductGuid=N'56a0aeb0-7099-465c-9613-92d2d50cb788'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|Supernat reduced|1st container|Psoralen-treated', ProductShortName=N'PLT AFR PAS-C LUKOPR SUPRed Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'56b9cef5-3969-4d5b-b39d-4cb81e7a8aab'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'56d83e45-230b-4d6b-85f7-f452808ce7ad'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'56d84a25-e497-43cb-ad57-04fcf629b636'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'57233239-f820-4d2a-b1fc-9485f3019e11'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'57280ed4-feb2-4c01-a713-304d28229162'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'57327824-abd7-4137-aed9-770e16068951'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|2nd container', ProductShortName=N'PLT AFR PAS-D LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'573c55d3-c780-43ab-90d7-810ed9f48511'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'575dbf9a-e775-44eb-99f0-62f83f06d8dc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5769e438-8fa0-436e-bed2-ede658d633ab'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@30 C2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'57b78cc0-0cb6-481f-aeb4-7dd6b09bb637'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'57bb5731-e9eb-4ba2-a013-2005286ebb70'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@30 D2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'57e0a8f6-8a8a-407a-9286-135f861b352c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'58067761-ad64-4d2b-966b-277224bb019d'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|For mnf:injectable|ResLeu:<1log6|2nd container|Approx 240 log9 plts', ProductShortName=N'PLT AFR PAS-C MNFI LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'58204fb7-7edb-41be-bb36-c4dc22f28490'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'58327b27-c617-4c39-abdc-321ef4e7c192'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@CE A2 GB', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'585aa77f-7e10-499a-88e0-ec54d9ec4cc2'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|CPD>PAS-C/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Approx 240 log9 plts|Psoralen-treated', ProductShortName=N'PLT Pool CPD>PAS-C LUKOPR BUFCT Psoraln', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'5882aa53-6eec-4eac-8b3b-2d0d7a5a5a98'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5887a7d8-5019-4af8-800f-968938e4cfb9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'589a23c0-d4c1-441b-b3ea-cf9ae2c7f587'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'58b56027-6a77-458b-8f72-9a11e435827b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'58e3ddd5-ec0a-49bc-8de5-07174db40efe'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'590bae47-7079-433e-a64f-327d2958f9c5'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84 K4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'59132905-77a6-4db1-85bd-940e15d0c7a3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'592ddc97-1af5-4a73-b1cd-ce7b14ef9c69'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'59455cf3-fb54-49b9-8a7c-6befe2dfff53'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'596e9e2c-8ace-44ed-a572-939329d941d6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'59a60483-dbca-4747-ac4b-7419be544f1a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'59ac0695-dffd-49a0-b0da-395692721940'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.590' WHERE BloodProductGuid=N'59ce083c-88a7-453a-8037-7f5142a05d58'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'59de2b1b-b96e-4075-87f5-164dbe0410d3'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|2nd container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-B LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'59df50b5-bed8-448a-807d-15a7445d7dc9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'59fac7ee-45dc-4d9e-a4bc-aeb644789cf9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5a03a62d-fcbc-43df-89b1-270772b7c8bb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5a0de9e7-0b98-4e92-8a7c-d666b51ead84'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5a12bca5-ea77-4509-9637-96bc49616a35'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5a214f1a-a062-4fd5-aa2f-84f836ef8b7d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5a2d397c-382e-4bb0-8a3f-77a98113d87e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5a31b9e2-e14f-424b-975a-e1ee4306f299'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5a7e31c1-0fd3-4479-8a76-443e88e280d7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5a9f38db-7f07-4fe9-b421-14ce3935d2fc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5ab45191-4725-483c-96fe-bd91df24a771'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5abdcbe8-6ed9-4e3a-8e03-45da32b3c209'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5ac37696-e6ca-46c0-aa2d-ba39bb26d9f6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5adc555b-8aa8-4aa9-bc77-17ab178b07dd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5aebcfbb-4f90-496e-a018-e60599b4cdb6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5b1e5238-ef0d-4bd2-b3c5-dad5e9b693db'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5b2379ab-2f74-4e24-a960-fa5c4d2ee381'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5b54c213-d405-4ef4-903f-e570239879bd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5b6599a7-b5ac-4578-91ba-8141d2f2e22e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5b75a177-f5cf-40cc-9263-c060e122f658'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5b959b0b-b44a-49f4-8237-90623d4af7dc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5ba43481-faa2-4295-aebc-9c32d3612356'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5ba47932-a0b1-49c9-aae6-befa6e13da2b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5bcbc8a5-6fa5-4554-9f80-52d8b5db8810'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5bff222d-90ee-4822-ba52-67bb2ffc8bcc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5c0cb10e-1a82-4978-abaa-d31035a5d406'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5c41d5cf-9af7-4552-9cc5-9a7795705c60'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5c441153-220d-4811-b259-e50c9a9768a9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5c492083-8afe-4b8c-ac80-e8b8f3c0c2b0'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELET-RICH BUFFY COAT|PAS-B/XX/20-24C|ResLeu:<1log6|Approx 300 log9 plts', ProductShortName=N'BUFFY PltRch Pool PAS-B LUKOPR ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'5c4ec475-bd19-4e92-9e1e-4bf097eb4fca'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5c5b2deb-1e49-4cdb-b3e3-0aa4019781a5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5c7983c6-50ea-4a2f-8a4d-7cb54c144f89'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5c7d3cc9-383e-4e4e-abd4-3fdc691fa069'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5c9fb45f-b3c5-4a4a-87de-a899d02566f8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5cbf1ae1-08c9-4ede-a0b1-7375338ccc45'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5cd4dc51-8652-4a3d-943f-e873cc7eafc7'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 C2 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'5ce3a337-0456-4b63-a074-9054eb6e2d99'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5d131dda-bcc9-479f-9744-c94cae231b89'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5d4a2520-28f2-4fa4-8e45-7ce8cde49f42'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5d9d3efe-edde-44b3-934d-5df329608808'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5da6e583-f677-4e0e-aec2-c120309b555f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5dbc9278-9a46-4474-91c3-d8b4597c497a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5dc5b8e9-7079-4b65-acd8-4a9af64ab4c8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5dfae1b0-0647-4ae6-b5d1-768fe4e6f777'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5e099c14-7d06-4952-9544-20aea10a0fa8'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|4th container', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'5e137f5f-f025-4ac1-a40b-80bebee709f5'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Supernat reduced|Buffy coat plts prep|From 2 donors', ProductShortName=N'PLT Pool PAS-B LUKOPR SUPRed BUFCT 2DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'5e202363-004e-4c06-a24c-32b388be6216'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5e3710cc-1d11-4fe8-ad71-833cc31f8790'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'5e72801f-e44b-4aac-85eb-ba4bee9d38b5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5ebafebc-bbff-4b0d-88c6-efbc7428a0ad'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5ed7b324-fe14-481b-aaf7-b84e5f3365c9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5efdb7aa-6377-468b-a03d-e6ba77c500e8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5f029b2b-96a0-4748-a084-a2a9f51a7b3f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5f14a5fc-f532-4d60-a7ee-09e9eb213010'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5f44d1cc-8696-4b4e-a0bc-8b6222a4e0ff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5f4ae902-82b3-4bbb-970c-edec6e801cf3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5f915961-5f3f-4374-b5aa-a4d940032382'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5fd9ff3f-ab44-478b-90d9-4676e4a2f18e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5ff9dd7c-9d19-4dac-8efd-bce83d4495a7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'5fff13bd-0723-4b43-bbc2-031d2b5dd83e'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|4th container|Psoralen-treated', ProductShortName=N'PLT AFR Wash PAS-C LUKOPR Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'60384ba0-27cd-486f-a69a-b0c80ed3c45f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'60a0a7ea-d335-4b8a-a6ff-d39cae8582d7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'60db1456-5107-4704-b48b-70f04da0d93f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'60ea3e4a-f025-40eb-b503-3562bddaa691'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'60f1d127-5097-4ab5-8bbb-272138bb66e7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'611ceabb-ecdf-47a6-8cff-7ab5c8fad6e9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'61466f1f-f84a-4efe-aebd-60b6ce5c79fc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'61638d77-ab0e-4966-a76e-63fe7c8fcb93'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS B2 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'61986408-1463-4ba4-9f9d-c4574a90bedc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'619f9cda-c631-43ef-9104-fa1671b03ea2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'61a0f2b5-9f5c-4449-9371-767b70042904'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'61aa55c6-d137-4d4e-922c-2302a76fe66a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'61ab2176-fb97-437c-b1a4-0171db4f0f86'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'61e557a2-2e66-4228-b3c0-62c7bd8d5c88'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'61ecb831-5487-4e7a-a93e-42ac8a3a0be3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'620b8e65-2fd0-41db-af37-35cc64efda91'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6213cb92-8a46-4a5c-9ca3-718041da8205'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'62211d5a-b45b-47b8-bd59-09af2d96ddb6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6242f94b-dd55-4a87-bb77-520eb573a230'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6251129e-6980-4b52-bbc0-423a535daf0e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'625e3a53-db79-43ab-80d6-2e79a2c9f90f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'627ac4cb-7be9-4edb-80d9-27d07ad80e48'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'627cf93b-30ec-4b45-b288-db1e052e35fd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'62a76363-c321-4a12-ac07-3574861ab99e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'62c86293-cf5a-482a-a01b-2291e8148335'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'62efa443-b480-4b71-8d44-79d1474393cb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'62fcc3f0-dfe5-4eaf-8323-83647f578c16'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS D2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'63069187-cc45-401f-834d-05952a4e7636'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'630ae365-1893-4513-957c-83a2b4fb1754'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6314ad52-b4f0-46a1-a5c0-1fcaeb3a2c83'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6341e3d4-1830-467c-bd72-4386c8d7e9eb'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@B7 B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6360b009-2092-418b-a1e9-b549ae89f601'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6374a9bb-6749-47ee-a1e6-9714491d02f0'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Approx 180log9 plts', ProductShortName=N'PLT Pool PAS-B LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6385e170-fd51-4bdd-babb-2f8643508d4e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@71 B2 C2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'63d4e7e8-8499-42fa-84da-5aa2e09d9815'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'63debe80-b956-4e66-9187-0ae0e356eea1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'64038777-6388-4c18-9e7d-380502ddd027'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6420ee62-f024-4e11-9123-f0a900b938ce'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'644d489c-4655-43ac-a4e8-da18fa45fd09'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'64ae2340-f61b-4912-98bf-1e0d95128c6a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'64c2f7b2-ab9f-421d-b932-9a74ba4b278f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'64ca9f23-dba8-4454-8c59-a61cd739847b'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|1st container', ProductShortName=N'PLT AFR Wash PAS-D LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'64d6f9cf-89d9-4e96-ab9a-d483f2f92e87'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'650075f6-d80a-449f-ac11-f8c9007c262e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'651c3c57-7724-46b1-aceb-c124c4c29f95'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'651dce1d-716a-4cb1-b026-04e0a018d069'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'65220a01-494b-4854-866b-7297f1331f40'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'653808de-331f-43a6-9e73-5cf53342c27e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6540a49d-8563-4eec-8778-fe035a06198d'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 D2 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'655e0c3a-c9f9-4c7a-a944-e7d8232f2489'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'656cbf17-173e-4ae6-b194-266f817fc91c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'656ec57d-e4ce-4906-9234-0e02e96bab46'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'65a1764f-88f2-4934-b497-1197f7582ebe'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'65a8e048-9729-460f-8dc7-68e09f5b609d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'65c0647a-530c-42b9-ad4a-ae5f93ec72b1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'65c747ae-02df-4c31-a8d7-730c2426fafd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'65ceecd9-3cd7-40af-8d34-ad84c275c672'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'65cf916c-7736-4de3-9963-3952c9145d21'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'65dad306-ebbc-48bf-9462-88e32c0f910b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'65eaf3e8-c0bb-42e6-81db-84d00492f6b9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'65f91bf0-bb48-41d3-8a7d-a9bf737f0dd4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'666d836a-e6f4-499b-9e99-f23120bb3b96'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|3rd container', ProductShortName=N'PLT AFR PAS-C LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'668b3674-ca6a-4b18-8428-2a3a41352bd7'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|3rd container|Psoralen-treated', ProductShortName=N'PLT AFR PAS-C LUKOPR Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'668c6063-787b-4de8-939d-06ddc66b3069'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Bacterial test', ProductShortName=N'PLT Pool PAS-D LUKOPR BUFCT BacTest', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.700' WHERE BloodProductGuid=N'66964d3f-a653-4ede-b700-3e53b5835b73'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'66966ff7-f512-497d-b4b4-23fefdaf67e2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'66e566f3-43b9-4c8d-a112-a17dedbe37cc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'66e89e78-464b-43fb-831c-3027d4f26cd9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'670e872f-a40d-45ac-95f9-7720af965744'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'671b70de-229e-4247-bcfe-aae738db7fc9'
UPDATE dbo.BloodProduct SET ProductVolume=225, RetireDate='20100201 00:00:00.000', LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'67740f78-1c72-4578-9454-80f6629a1aab'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Bacterial monitoring', ProductShortName=N'PLT Pool PAS-B LUKOPR BUFCT BacMtr', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.560' WHERE BloodProductGuid=N'679eac81-1cac-4716-8372-5434b0c093a3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'67bc2e6b-9fba-4308-8d95-21b6921a2f02'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'67d1309f-6c51-45f1-9826-d3e98bd2075d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'67defa5b-a280-4e61-81ec-acabde850946'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'67e4bd7a-1873-4d51-bdb1-8a63be9badad'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'67f1730f-a865-4b17-a671-280b4d6f480c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'67f5f872-62d6-4115-a31f-fc2efcde74f5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6809cb72-556e-40ca-9967-09a1995f87e4'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Approx 300 log9 plts', ProductShortName=N'PLT Pool PAS-D LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6829724d-fa8b-442a-9149-ca7f573935e9'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@B7 B2 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'683644e3-dae2-41d6-b113-35fcd29bd12b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6836a277-a82e-4eb3-9a72-7a8eda0e5d09'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS C2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6847d814-363f-43b8-bae4-6a599483a7ec'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'684f9451-8dd4-4445-a68e-ff310325000d'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|4th container', ProductShortName=N'PLT AFR PAS-D LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6857b774-7322-4d6f-89b2-a9bc44091762'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'685bd699-141c-45f0-aa63-bf602412278a'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS C2', LastUpdateDate='20101201 15:05:41.500' WHERE BloodProductGuid=N'685c66cb-2987-46f5-863c-2c0bf4568a45'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6865b584-ca5a-4cae-aab8-e2b14e425c9e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'688c2250-8071-4de3-921d-59fc0f8f23fa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'688f63ce-2078-4fab-b60d-c496b16b93e9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'68ad3d99-f87e-44e0-a306-df7cdf65311c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'68ba8b8f-8066-4a50-b97a-53f288d2ea4f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'68f0fde9-c954-4650-95b2-f67b8cb42bf7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'69142e32-e51b-4815-b104-a52312c8485d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'69354d0d-b975-4422-90a5-fe8be0a8ed87'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'69376e87-3d65-4688-b731-5f627268875d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6950a3a7-c6b6-4d88-ba0b-e67a06c7a041'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'69562e77-67de-4c16-9f76-b9e7a865a03e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6963921d-ceeb-4202-affa-9eb0fa607dfd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'69889167-81ed-4a91-963e-93031b0f9b76'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'699826c6-cdde-4610-8ec8-93b2988c11c0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'69a4eab3-6aa6-4191-9310-71962e2852b5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'69bb0e1e-6f62-4f59-b482-ea4c3226022b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'69ca0b35-a06b-4e09-8709-409cacc1a028'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6a6f345e-eacb-4c8e-827c-ec2c11229870'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6a8b2b36-2985-4c17-9aaf-80173a5061cd'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.733' WHERE BloodProductGuid=N'6ac6dead-197a-4340-8dee-9d3ae21d65c9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6ac83609-9fc3-4614-99e8-e1c376a02075'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|Plasma reduced|Buffy coat plts prep|Approx 120log9 plts', ProductShortName=N'PLT Pool PAS-B IRD PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6ae6b2e7-e03d-446f-abf4-5912618f8fe7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6b062169-7bea-4e65-97ee-e691a0726dc6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6b1cbe38-82f8-444a-a9b3-09f396253e9a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6b2a6998-246b-4e27-ab60-57aea7c3cf65'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6b2b774f-4e54-438e-96d1-f7a6b8dd84a7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6b480a2a-367f-410f-8c9b-eb2ba6525c3b'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6b78a6bd-f8da-45e0-9a4a-3e060f7dbbe0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6b917cae-efa2-4668-a50a-d468b20dd789'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@70 A3', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6b96df48-5037-4ada-bed7-28e4636e186f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6ba86f3c-83c5-4bcd-a30b-6eab0f907059'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6bb595d5-06ec-4746-9b47-8013cd2c5ddc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6bb979c1-3bed-409f-b965-7057cf41aa8e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6bc31c2b-9125-489b-989d-7286cfec1701'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6bc9cef4-e5e8-4495-9dea-014096a75b3a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6bd1003d-436f-42fb-9648-5a536edb0807'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6bd76b41-4adb-4da9-bad9-90e1cd67f97e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6be1d1c0-0be7-4ff8-8e61-9fea59047f78'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6c25c7e6-4a3d-4162-806f-c73349a7e75f'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Approx 300 log9 plts', ProductShortName=N'PLT Pool PAS-B LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6c966c1d-6c6e-41a7-9432-a31d78db733f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6cc9d730-ad39-4c4b-9505-161ef62fd21a'
UPDATE dbo.BloodProduct SET ProductName=N'Washed POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Buffy coat plts prep|From 4 donors', ProductShortName=N'PLT Pool Wash PAS-B IRD LUKOPR BUFCT 4DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6ccac928-e199-4097-bbe7-0faa5362d5c3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6cd00817-0ecf-4ab9-bf22-855006be6a78'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6cd56ce8-e13d-45b3-a386-9594b501db99'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6d17cd58-ed03-4c18-a764-a60c58df86ba'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6d1f1a6a-7ce6-4947-91fc-f1274c45f274'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6d293dc3-ec53-4998-86d8-99dc01c30101'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6d3b6241-bbf8-4dff-8d8d-5ed99b01f06b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6d545345-77a5-47fa-ac89-b341f955eb23'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6d824a40-1bfe-408b-9336-f186eedfc84c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6d9a44bd-4c5a-4ba0-95bc-e28885d4caa1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6da0ab06-7fd6-4469-881e-83f3acbc0bea'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6dbfb6cd-b101-4cc6-9dbd-42e8d691b563'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6e03cf02-e30b-439b-80cd-05e1ec8567c0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6e0bafae-6a20-4c09-a5d0-52f78983a8f9'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Riboflavin-treated', ProductShortName=N'PLT Pool PAS-D LUKOPR BUFCT Ribo', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6e166dc1-f8cd-4a3a-b7c5-947c86b646c9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6e43d868-5963-4a36-9276-01dd0a4aac03'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6e47eda2-d112-410a-9398-41afd8024b0b'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Approx 360 log9 plts', ProductShortName=N'PLT Pool PAS-B LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6e5b61e4-61fb-4461-bd11-c1a47ae68967'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6e8cbda2-2bfa-4a10-87ac-82a53b7ed1ee'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6ea81db1-3e89-4cb9-9250-c56f7adc3a02'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6ea95b68-ff5b-494b-a211-598b4b37f313'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6eb0b436-15c4-4567-8e72-1b2dc8229e86'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Approx 120 log9 plts', ProductShortName=N'PLT Pool PAS-C IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'6eb85deb-4149-4131-a895-7384ab937503'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6ebe62a0-3445-4080-bc9d-b22e98ec24e2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6ee8ec26-b742-4f6c-b004-d3e71005dd73'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6f035084-8197-4530-85c0-59f0104576cf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6f2aefe0-5bc8-4185-b3d3-512c8ce388be'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6f7d05e6-4d2c-4d26-bea9-121bf44753af'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6f9757f6-71e2-479c-9bc2-3a815f50de46'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'6ff468d6-747c-4587-9788-dcd355256cc2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'700e0623-7bfb-40a8-b4eb-7d38e9b14caf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7046f86d-46de-465f-89c3-c261930a1f18'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'705ebdc8-fe5d-4a1b-9767-c6910719077e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'70679e35-5c35-43e8-9281-9f5475fb6b49'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Approx 120 log9 plts', ProductShortName=N'PLT AFR PAS-B LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'707a1125-dbb5-41b1-be2a-6f449600def1'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Approx 120 log9 plts', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'70ab7927-914f-40e6-b897-6df302376826'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'70af08b8-399b-40d5-bd1b-508bf03082b7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'70c7750d-3056-44aa-8158-3e6233aef538'
UPDATE dbo.BloodProduct SET ProductName=N'Washed POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Supernat reduced|Buffy coat plts prep|From 4 donors', ProductShortName=N'PLT Pool Wash PAS-B LUKOPR SUPRed BUFCT 4DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'70cb9e4c-6386-4b0c-9064-4a9172088758'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'70daf241-82fa-485f-a834-1a1f53a28dd3'
UPDATE dbo.BloodProduct SET ProductVolume=225, RetireDate='20100201 00:00:00.000', LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'70f18bed-fa15-439c-aa79-efcd7290612f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'70fdb496-9f5c-4b26-8b8a-800e02efbc83'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'71152094-52a0-4037-b3ae-ae29fa8028f1'
UPDATE dbo.BloodProduct SET ProductName=N'Washed PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep', ProductShortName=N'PLT Wash PAS-B LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'7121572a-6f76-4942-a13f-904b80b10690'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7171257f-73e3-4ba0-be24-4ba8326f743e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7186758c-3043-4689-b9c3-1ccfedb7acdf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'71bd90a3-b315-4adf-b404-33ea58a61e58'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'71d64386-5792-49b4-9526-2bcba4f00905'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@30 C2 D2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'71d9158a-946f-4097-8a25-d3559abfabec'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'71f55004-9139-4ab6-81f5-25574a6e7372'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'71fb98f9-af79-4aa6-975a-b92ff2aba250'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 C2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'71fc669a-97f7-492f-8a6e-39b529f7327d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'71ff1c52-f8c7-4773-b908-732e3efdbde4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'71ffdaa7-3313-43fe-a3ca-d6abb16cb06f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7202d54a-6ee8-46db-b841-d9455e74bcaf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'72069650-b54f-40b8-9e13-2cb8ffc0ec37'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'72498521-10a6-44be-bd22-7ad5ed34baba'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'724b7db1-2fff-45bc-8f23-9ce2f9e4656a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'725f22e7-6cc7-4eae-81ee-f763067ec328'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7268c0e1-2c21-42e9-a245-0b8b51a77b79'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'727b275d-d30f-462a-a5d5-0dbee8a52b14'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'72961c02-24f4-4127-92d9-be5a193d539d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'72a87f90-c333-409e-b416-f0b331680f28'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'72d62690-0943-4eb8-9ebc-14349ced4d6e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'72e9d14f-e528-4ab3-8b2e-cfe1a3903ff0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7304e1db-b73d-488c-9fb8-3dc86da04828'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'732eaa31-bba5-41ef-bfdf-a26b3ad78db2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7334438b-a503-4906-8f14-bdaa9e3ea369'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'73450290-2f04-4d7a-9435-3e50e97692e5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'73513bd0-f3c7-43d7-b326-4863a6c7b63e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'73880700-b048-41d3-9399-97cca3e3a1ce'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7393267b-4b55-4e99-a648-bc31dcd78da4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'739a16d1-2a4d-4189-8d90-395bbf6f8921'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'73a832d5-8f96-4323-a374-1ebd73f7e3a3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'73c28ed8-774e-4251-99b7-7a137a16b7bb'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|4th container', ProductShortName=N'PLT AFR Wash PAS-D LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'73c2b2e3-ef44-47dc-8dd4-60339db9b8e0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'73d64176-4ebe-4041-babb-9ebd27127a90'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'73f7c0f3-ff21-4670-a7b7-7590aa1b3d89'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 C2 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'741b094f-e2e6-4c8f-9d23-c8ee471102a6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'741b6474-69f3-44f9-9df8-52a3e492de65'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'743cc1e8-4525-4288-ae9c-18e99127a73e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7456a940-331c-4c24-803d-e1de486c0de3'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@33 C2 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'747f9d11-c154-4892-8659-52f35ff5dc38'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'74816556-3fdc-4472-8b48-9a9e214da3fb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'74911a84-8437-44ee-971e-009d23280b03'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'74c6f844-a166-44d5-a99c-c0c9cfbb614f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'74ec4040-0ca4-47c7-b3fe-32a2b8f35994'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'75076e89-e24d-400d-9a68-3df34e7a510a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7511450e-2bec-4ed2-8423-dc3ea6ca8ad0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'751c8ec2-14ff-45b9-8ec5-dea2cd18dc3f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'754946aa-4828-495b-af71-bcd0416d3573'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'757b08ff-3f37-41d4-9f7d-fb43beb7520f'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.530' WHERE BloodProductGuid=N'75899255-732d-49ea-b9fa-bc8652092481'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'75989ddf-6d53-44c1-a490-088fe8ed7e46'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'75adf39d-213d-488f-86b0-33589eb71f09'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@76 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'75c92bf0-7529-4312-803b-235706e426cb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'75ceb639-7785-4a65-8805-ef45375c503a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'75d05816-2fe0-486a-96d1-57aa8367799e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'75d7a734-c8f4-4a2e-bfa8-b47d23a08308'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@C4' WHERE BloodProductGuid=N'75e6523f-b8bb-46c7-b2e7-e581a0b7daa3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'75f9e739-a716-4237-99f2-4957c67ee3d2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'760906d1-6e67-4a8d-8905-22ede22682cd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'762f0656-d64b-4eaa-bd6f-61da73afd353'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'763d29b0-a2d2-488e-972f-2fe5e2759a49'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7665bb5b-3a15-4f62-9052-27bade8681dd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'768b7537-3d74-4321-81f3-e8fd335bc134'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'768dfe1e-cb90-4f02-a538-ebfb43eced7a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7691ab2a-7fb3-4c3d-b861-bcb89c09106a'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'769ec36f-4d63-4f40-b3c8-a2d64c9b6757'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'76b8e306-744a-48f2-a5f6-1bd06428ccbf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'76bc9483-5c31-4043-9361-f05fbcd577f9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'76cec99f-5689-4e80-a60d-bbe9aab643e5'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@33 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'76e85690-1bd6-43bc-9202-8117dd41712a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7743486e-9802-479f-b5c3-7847eddc7d7a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'77445147-0ed1-4d61-8d10-baf92b119a33'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'77497f39-6d6b-4b4c-8872-d24d895a2465'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7752da29-269a-4c56-9ee4-219109e709aa'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.590' WHERE BloodProductGuid=N'775a9a13-73c2-4706-b363-fe602f5fd373'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7795d740-ed7f-45ad-9bd7-9aca65127bae'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7799d988-814c-4e4f-9648-bbb4ce0e9c03'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Plasma reduced|Buffy coat plts prep|Approx 180log9 plts', ProductShortName=N'PLT Pool PAS-B LUKOPR PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'779b2c48-b613-4f40-997c-832493fb9e0a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'779f6976-d71c-4d40-9c7e-5544273d9fc4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'77aa7e25-b4ba-4e8f-beaa-92fa92baaa33'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'77b2a2a3-a52a-451e-81af-9a54a02c15f5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'77ba6835-5e8d-46c5-93e3-e1a70023b89f'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS B2 D4', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'77e64265-cd73-4090-86ed-da611bd35d65'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7805a3ce-754c-4df4-a2cf-7de7fa5cc817'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7811cf22-0100-47b6-bd2e-e10e2e4e2859'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'78bd6274-255e-432e-8926-1dfe1b41277e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'78c956f7-e800-44bd-8b23-b1f96dfb1972'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'78cbde0d-422e-4df6-a907-9a89db992d6c'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|4th container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'793813f4-3b8a-495f-b9f4-1ea8e299ec20'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'795532d2-63ee-41e8-8ae4-2d33c0f1086a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'79bb078b-cf23-4b95-8dec-67df0062a940'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'79cb096d-f838-463f-9d81-a1545ea7398e'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|1st container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'79e068c7-4fb7-48b1-b6be-9bd7815b6799'
UPDATE dbo.BloodProduct SET ProductName=N'Washed POOLED PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep', ProductShortName=N'PLT Pool Wash PAS-D IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'79e13ba6-63f2-4e65-b38c-d26dfba056af'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7a0db706-329e-4041-bbef-536de5987039'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7a56d698-a37f-4562-aa4a-6a059e8ce2a9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7a75346f-19f9-4d10-8d84-7957ef642a99'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7a8015e8-625d-4945-bc38-a28450ffcea0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7a993cfc-93bf-4e07-a69b-bb234ad9ecee'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 A3', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'7b642e0f-81cd-4375-b66c-4ab8f6948905'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7b83cd72-b943-4aad-98e5-098ab60a1f3c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7ba41706-8a33-4906-9551-10b80e10e304'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7ba5dde8-4a59-4cc7-b924-9a5e2cdf0f03'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7bab8baa-3bf5-4895-ab4c-e12957143705'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7bd2b34c-99e5-49b8-b614-1203b1d16709'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7bd3c48c-ef5d-4ec4-9cab-b6fd62e7594c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7bd7bb5c-2e3e-472f-8f1e-0eadc4766775'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7bdb959d-771b-4bba-94e4-cad01a0a67d3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7bff0bf9-8066-4692-ae6d-eb87e05b0608'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7bff34a8-de16-4bee-9768-d8812ad81aed'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 A3', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'7c52838d-b54e-4b76-9e22-bd291e5450ad'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7c5bc92c-6c0b-4cd8-b786-c60a41f2a0b9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7c5c2e66-0c3f-4ef0-b380-d9a242ac7ce8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7c5d67cf-9ae9-4f10-83c8-92af72153368'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7c71e80c-01d0-44f6-bbcf-c54b72b9f3b7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7c73af5c-d8e7-487e-851a-90ea0115f74d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7c7a2eb8-0cfe-4430-8384-15df86cf1384'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@33', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'7c83d34e-16e3-461d-80c9-4a7600172d73'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7c920360-1b1f-45f6-bb4f-c2439d985a1e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7c9a8926-1b50-4dfb-b795-ce4748c2c43e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7ccc1d48-210d-4b0e-aa6c-daae5932c438'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7cdd3225-1170-44af-8fea-f0f7b59ead8f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7cdf1459-ea09-476f-a18a-8fef941b8d5e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@28 B2 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'7d171d30-0ac3-42c8-b2dc-504d3278015d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7d331be9-861a-4e50-89b9-394aecc7f97f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7d358fa4-18bc-4464-b0ef-03e6f77b1178'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@B3 C2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'7d44a879-0db6-4013-a054-d1603a84abcb'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Supernat reduced|4th container', ProductShortName=N'PLT AFR PAS-B LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'7d6b0a41-6f81-488b-af5d-cf20f8ba505e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7d70f7be-64cb-4468-ba65-4157d80af2a3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7d771c15-c2d0-45fb-97f5-a53f3d183284'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84 B2 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'7d84d017-23af-4edd-a422-cfb52ccceac2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7d8d6854-ffda-40a3-81b9-19caf7bf9d3b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7d9dd145-c7b5-46b0-b898-69d9deac34c6'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Buffy coat plts prep|From 4 donors', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BUFCT 4DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'7da81da6-b09f-4b27-811f-3dfebd43a3c7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7dc7d789-8702-4d53-b1ef-7baa983bf86b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7ddb3530-c198-433c-bf09-8b366d7edf10'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7ddefe5a-c809-4fda-840a-b4269c5a0509'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7df002f1-a969-476e-a95f-715dcd585144'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7e11075a-6dd4-4d96-a342-8c024d5295b0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7e347d7b-812a-4da2-bc83-419b802a098d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7e3ce649-090a-4fbc-9385-fee88818f263'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS A3', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'7e46d48d-8e27-43d0-9d9c-8a9c79ac707c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7e5037f8-110e-4045-b6c9-8d64339cf408'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7e62ca60-7160-4069-9463-a1fd5c8fecac'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7e978acd-561a-4c8a-bf8e-ed0c75626c16'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7eb3cec6-5886-4860-a981-462e96fc9528'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7ef4fdb0-242c-428d-8ce9-0e5a0a72cfe2'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Buffy coat plts prep|From 3 donors', ProductShortName=N'PLT Pool PAS-B LUKOPR BUFCT 3DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'7ef54aa6-2d76-4256-8759-1237c9eae808'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7efe9a6f-6080-456f-b85a-dfa9fcad53f2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7eff8dbd-e6ab-40cb-ad2b-cf98191a74fd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7f2b7ffb-c844-4000-9681-4243cd6231c8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7f3d21a1-ed5c-4809-aa12-c986662093e1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7f4cd791-a780-4c41-b011-1f2e63fb5048'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7f521cc4-f7fa-4950-b8ca-7ad787014e1f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7f5d959f-987a-4b8c-8122-92000a8c850a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7f5fb5ff-eab9-4348-9094-4be3c25f5854'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7f67b30d-f8bb-40cd-a93b-094ff999b207'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7f9273e4-32a6-49ea-afb9-f8cfa570512a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7f957e45-2c74-4d8d-949c-b189f13e83ee'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7fb80621-f86d-4d75-8565-9d21b6ef36a2'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@17 C2', LastUpdateDate='20101201 15:05:41.467' WHERE BloodProductGuid=N'7fb83d83-c0af-41fd-ad86-998c9eacaaa6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7fbb355a-77ec-4b1a-8662-af97086e42db'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'7fdcaba0-85be-4184-b8af-20bdd27d5a5a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8000efb0-bcfa-4bb4-8e51-d37a8d5c57d4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8018ed95-ab79-4885-8181-5a556f8c5701'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@A4' WHERE BloodProductGuid=N'80209022-7db1-40b0-b978-38068e2f1cc3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'808a01d1-9a98-46bd-88f0-82b2b86d263b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'808f052f-7a82-4744-ac99-8f73272d6c33'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'80ae1fea-f0d1-42e6-9110-a188db115217'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'80afc768-6153-4f80-8335-2eec319c94cc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'80b4abf0-6b5e-4c6a-a6f2-77decd40c25d'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container', ProductShortName=N'PLT AFR Wash PAS-D IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'80ffff2e-cba0-4835-8c73-1ae80bfa6efb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8137aabf-7449-48ca-967d-60149389e04f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'815bb490-6fe5-4895-97d3-15b06d3ae819'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'816129c6-a0bc-440d-bc31-7349ee3934da'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8170be5f-7c31-4ed6-9a15-9267f078d0dd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8182144f-f380-4bee-9a41-a9a971cb20b4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'81880b6f-6751-4c3e-8c8e-74b1ff22ed85'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'81922c8e-fc82-4f2a-8639-13d4c27f8b59'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'819b585d-12f2-4f6d-a96a-35f0fffacc29'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'819f0733-3170-4931-b63e-fdc97d77d850'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@91 B2 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'81a2c4ce-c27e-4aa3-8c58-faeefa03c8f9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'81b0bd04-9256-46ef-86b2-8268b3ea3ce4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'81b42184-f890-4402-b49b-0ea3297be7a0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'81c9afa9-4690-4239-8074-60b5ff913f9b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8235a351-61fb-4f62-8c72-3c3ded34f217'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'825a7420-327c-4b9b-88ed-2ded1b1f4f6f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8281afdb-d88b-49ad-b49e-ca8c97c6a336'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8287c8eb-2506-4870-93b4-7727db1f34ff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'828acee1-56f8-4b88-b931-266078d777b1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'82b617f4-a373-4215-b130-2885961c6a3f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'82da58b0-facb-49a8-8860-1af591f283df'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@13' WHERE BloodProductGuid=N'82e639b1-0baa-40d9-9e24-96f2241d4014'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'82fa38fe-380b-4440-8cf6-c8cbc83f2f1e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8337e7fc-37b0-429d-b4ff-b8359b23228f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'833887fe-1018-431b-b990-9200cfbad13f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'83666c53-ea17-478e-8512-b493d309a374'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'838a3ca8-6d4c-479d-8e32-a9bcb26895c7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8392d289-6207-4e05-914c-f0cc19968fe8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'83a9db89-df11-4f8c-a9c7-b02c15b436b0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'83ad6362-6b45-4711-b14a-46f7e71adabe'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'83ba6518-da29-4de6-9bd1-9d0a9459b15d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'83d754b6-f3d7-4841-9318-a57fc2957743'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'83db8c2a-1bf3-4e1c-8ca1-1fa943a4c1ed'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'83dcc1fe-9ac1-40c7-b6d4-0afc9e717e6b'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@C8' WHERE BloodProductGuid=N'840ea45f-e1bb-41cc-aabd-84a8e7b379e8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'841f597f-670b-4e34-aa9e-71adc4a08be9'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container|Bacterial test', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'841fcf4b-b592-4a65-aa4b-5b0ea8d84ab5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'844129e6-3196-4615-b509-0bff7d311265'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 B2 D2 E5 JI', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'84a04238-8eba-4651-bd08-c0392bd88b5f'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 D2 E5 JI', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'84d37d83-91bd-4697-8f0b-c0359d81adfe'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Approx 360 log9 plts', ProductShortName=N'PLT Pool PAS-D LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'84d9d5d4-f005-4d9a-9481-258a832f8cb8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8504b9cd-0efc-45a6-865b-59ef9ca1ced9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'850632dd-929e-4a98-8774-fa1ace637f10'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'850a7f78-377a-4323-ba1c-84e85eea0cf3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'850c89f1-8d50-4ed3-aabc-6d9637aa8b9b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8544b75f-8277-44a8-bee9-e0ff3baae24f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'854a4be3-1c13-46be-b8b5-fcaa4ac984b3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'85c6f4bd-8017-43ad-b20b-f3f4df41c370'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'860c6e12-5190-44c5-a20f-368893ce45a5'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Buffy coat plts prep|Approx 240log9 plts', ProductShortName=N'PLT Pool PAS-B BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'8610075e-0f19-4235-9d98-afd10fa3c728'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'862389da-4786-44e1-ac9b-0f130ce07086'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8657b94b-9356-4eff-804c-6c696edb79b3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'86813557-2713-4835-affb-c4fdf7f5c0fd'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 B2 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'86a7c326-804d-4e6a-851e-220595d566a8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'86ba713f-bc95-403c-bfd0-7c373011737e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'86d082c7-b813-4200-8f38-ef1545035570'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|Supernat reduced|3rd container|Psoralen-treated', ProductShortName=N'PLT AFR PAS-C LUKOPR SUPRed Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'86f6a460-c399-414f-83a3-97614a7ec990'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87124af4-88e1-4fbd-96b9-587958c91f43'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87170cda-af70-422a-8af4-0b17a114ef47'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87243b02-1e3c-475b-8093-084adcc3a540'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87352973-d1c1-4e78-b098-e9d6dc0de0c2'
UPDATE dbo.BloodProduct SET ProductName=N'Washed PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Plasma reduced|Buffy coat plts prep', ProductShortName=N'PLT Wash PAS-B LUKOPR PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'873a40f2-e2b9-4e64-9573-3c07cb636cb8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'874ab8b7-d78a-457d-a4cc-742a75ec2467'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'878db69e-205b-406e-b356-5a0698824c13'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87a00742-5d9c-4e4f-8301-f447f3529a72'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87a12af3-8c7b-4da8-a109-940ae8cabb7e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87ad4d5c-ef92-47d0-9132-ba2b69fc98ee'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87b662c9-9616-4171-bac7-4e666357289c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87bcc3ae-c6db-45df-86c8-383f4bbc6f43'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87c3fc89-3088-42a7-8adc-5c6efd2d3db2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87cec13c-d1de-49c7-983d-1662a6fd3910'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87cfb57b-4b22-4676-8d4a-3ba45a18a238'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87d6ab20-0f6a-40ce-9ab1-173140609062'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87e5fa33-0a13-4d88-bd12-6a3e68e390f3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87ec0c25-fd29-44d4-a66d-219a79c35f90'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'87f08eab-65d5-451c-9e04-6fb7ea4e5568'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'8807be58-c754-49ab-b614-6caa4a05ffc1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8808aff8-c3ae-4fc2-b443-db2a0bd8b0d4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'885ca279-cd94-4d74-9f86-995db84a2915'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'889b9daa-25a9-4f05-bc6c-58158b64d045'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'88af9231-a89c-4456-9e17-5bdf49df9866'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'88afd288-733e-469a-9e89-80ff8c089b77'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'88b917bf-38f5-44db-a460-f898a2cc3f29'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'88fbd6b7-7cb9-403b-9934-28e08f98e62f'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|1st container|Psoralen-treated', ProductShortName=N'PLT AFR PAS-C LUKOPR Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'8902fdb5-4d39-4d25-8170-1ea261dc4545'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'890547b0-e6d9-4a57-b668-94428990a571'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'89062129-409c-4eb4-8b48-e6fdecbcab28'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8920d585-060a-4952-8e8c-1024757c90be'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'892f13be-e1e3-4fee-ada3-9a484fb99563'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'89552161-f2c0-4e87-a2ec-e2ccab57aa9b'
UPDATE dbo.BloodProduct SET ProductName=N'Washed PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Plasma reduced|Buffy coat plts prep', ProductShortName=N'PLT Wash PAS-B IRD LUKOPR PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'896c0a7f-49ff-4d74-a502-54dacc1d258c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'897cbea0-e411-4bac-8329-46d32e2b31b2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'89d977bb-9cc4-4d76-9912-f2fd96e6aaec'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'89e92265-04b1-4a8c-9918-0f16e61f2437'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8a0b370a-08eb-4bb3-97ce-91a039b9225f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8a1983a1-c2d9-48b2-8692-2b485f9795fc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8a213698-6df8-41e5-a921-8d3395c7acf1'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@CA' WHERE BloodProductGuid=N'8a30be92-b636-44ff-ae23-bbd1d7720acc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8a3447e3-dbea-4259-a6b3-5252dbc5d55e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8a356896-f4a3-4e6e-bb79-0937b9b70add'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8a5b4a90-e5fa-4c70-920c-ee82b184f712'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8a62015f-60a0-4f8a-834f-803cfd7744ee'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8a7c8f68-5dad-458a-b878-1735de4900b3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8a7d766f-37ea-4702-b56b-fe23d9160442'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8a85e614-d20a-4cea-bb26-4697b77bd9fb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8ac6c354-1b49-4e1f-b158-02c6e60c62aa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8b020238-5520-471f-8fb8-7d35ab595a72'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8b164d20-190a-470a-a8a4-9b60f43f3f31'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8b7f7821-fdc8-4481-8fb8-6ccb8d87d844'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8b819f4b-0d64-4830-88a9-7d3ea3ed316f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8bab43de-8af9-408f-aa19-f1c4532bf51d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8bb93b9c-a8ac-4940-bee4-e68fa0c4b363'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8bba2af5-c912-4db0-b5d2-cb8ba86085bc'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS B2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'8bc25b97-b0c0-4f9f-b518-84554b3c6ef4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8be2aa26-c325-452a-b8ea-a79b6b1c580c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8bea2fb1-a179-4799-a5af-68be972e15b2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8bf0da49-b937-4ee9-bea0-3a2071e81b93'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.670' WHERE BloodProductGuid=N'8c28163b-e635-4251-9fcc-fe4dde79eaa5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8c35a106-93d3-40b2-9ed5-551e8614c7fa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8c55ca53-c1f2-4630-a151-f79e4737cadc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8c63cfea-8d09-4788-a4de-ef0d802e600b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8c697881-66cb-4010-bcaa-2e9181e13f7a'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|For mnf:injectable|ResLeu:<1log6|4th container', ProductShortName=N'PLT AFR PAS-C MNFI LUKOPR ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'8c84bc25-06c9-4bd4-81b6-00e3b8857610'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8cb01063-b5cf-4571-b1b3-c88be2e31743'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8cb84262-2a32-4ddd-9e2b-61b2ed7b232e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8cbbd037-6b91-4913-b769-2472d58ec292'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8cc2e322-18eb-400f-b1e7-5efd4af4d40a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8cd7347a-7508-4f33-b258-80ac61f9b3ea'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8cdfbde1-dc75-43a1-9dfe-d0c29ffb9d0a'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Approx 240 log9 plts', ProductShortName=N'PLT Pool PAS-D LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'8d098183-8f5f-4139-9764-79550bd9180c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8d12c082-6caf-494a-80be-e03b515f26bf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8d145ea1-64fe-4792-b1cd-5f0343315347'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8d324b96-3d63-4edd-994a-9b3d558dbd96'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8d3685d7-c935-4cac-9ae8-3a0b064a4873'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8d379270-9889-4896-b2ef-1e3403034dc8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8d3ed716-6e7e-4062-b8ed-b695b78eac97'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8d3fdf65-dc05-40cd-b181-46d206ca23b0'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@95 A3', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'8d5bcc96-9c8b-476d-91e5-54b7768d024e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8d672738-1803-4263-8f54-0a4a36752096'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8d75f90a-10f6-47fe-9461-32a2687e450b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8d87722c-0e76-4ae1-b7e8-43da3a51e460'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8d9aeb37-3955-40e8-8fc7-774bca4f460b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8da1e134-e0f7-461f-8b11-d97ed8b25e48'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8db9f46c-8b1a-4c08-b6d8-653f105922bd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8dbe3c2f-e640-42c4-90ce-d651bdcecd8c'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS C2 D2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8dd404b8-1254-4e34-8221-6d4d5d1bbcb1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8ddde28e-cacb-4823-900c-ebb7501bd2fb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8e050d4d-4df9-4e6f-8b96-eb9c6365f7a3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8e16a892-1285-484e-a4e8-989590e3abb0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8e1e7d56-cf86-4aca-bab0-2e51765530ae'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8e39f78a-1a43-4d8c-ba4f-d1923625c9f7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8e4ccc9b-f6a1-4499-8346-41a491fbfda4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8e570e47-b30a-4e32-8bf2-a8aa4e556e97'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8e57aba7-b879-4bc5-8434-7b8d0d57ac94'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8e8dad98-7341-4dc5-b1d9-ce0082eede50'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8e93b9aa-fcc0-4e9f-9f6d-abbc885461ad'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8ee982b3-7786-4378-a248-73b23f0362a5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8efed44d-2e44-4052-b147-bcdd7130863c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8f0209d3-021a-47ed-bfe1-d8cfe8d1c5b2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8f11992d-0771-4668-91b4-967ddf4ababa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8f3fff60-5736-405b-a55e-3582b7f42f06'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8f5057fa-d75a-48c1-846e-de0ac3422b62'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8f64c387-bc6c-4b9d-ae10-05cb45a39811'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@33 B2 E7', LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8f6f9db1-d6cb-4254-9c8b-1432608b2f70'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8f883679-5910-44ac-ac68-ff78a579722d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8f979d8e-acd1-4bd5-a551-ea69f3c34f82'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8fc4f1d4-72ad-4c86-b808-b9f263a91666'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8fc830c8-3f61-4967-99e9-8192195668a6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8fcc100d-48dc-4bbe-8a23-dee5464884a4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'8fe43139-884a-4f02-bbe8-659031eee575'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'90410b59-fe4f-4426-ae9f-d26bbbbf1e26'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'905685fb-a9b7-461a-a17f-6f47b0d8a34b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'906f9336-0d90-47c9-b649-5553f96b8b8c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'90bf97f7-db06-417a-8acd-5da19f3933e6'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Supernat reduced|1st container', ProductShortName=N'PLT AFR PAS-B LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'90d74415-9ff7-4bcc-859d-0552da159767'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'90edf8f5-3676-4a33-ab62-72c58da4550d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9103069a-7bab-4a61-bc3a-255b2fa271db'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container|Bacterial test', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'91305cd2-6405-490a-a6ad-43c999a7ad36'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9149b1ec-28c1-4ef2-b529-efda65792b1e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'915e71fd-048b-444c-b996-26448e2d7f04'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'91a310d3-b3e3-4e42-ba5c-c9ddf34b94cd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'91e2e159-a1ed-4842-bf26-03c675c282db'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'91f7566e-158c-4bb9-950d-bfe144e62a55'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9216d1b6-9168-4295-8ae8-53001042d772'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|4th container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-B LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9239761e-b195-4d88-a660-b2446e4464fa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9248b600-46eb-4895-a980-56d68a13f0ed'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'92be95a6-6954-428b-af6a-ae705c8d73f1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'92f7c668-a684-45f1-909a-520ad32310d7'
UPDATE dbo.BloodProduct SET ProductName=N'PLATELETS|PAS-B/XX/20-24C|Buffy coat plts prep', ProductShortName=N'PLT PAS-B BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9326e5c2-bbb4-44da-932c-1d1534b5aea5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'933240b8-98de-48ac-996c-21b4b78cb264'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'935ef03c-c766-4394-b798-5c18eafb4de0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'936529d7-ca77-4921-be9e-5395e9847aa1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'938d64d3-5722-49f4-947b-b8952fee632b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'939235aa-ca38-47bb-8b99-94f6cee68e7e'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Approx 120log9 plts', ProductShortName=N'PLT Pool PAS-B LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'939fe15c-00b0-4ba5-a379-04741dd760c8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'93b15a57-4965-4224-ac32-60fb5ac97ac2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'93cc2b45-eedf-45b4-bcb0-0a0e7755f359'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'93d34fb2-20b3-4426-926a-1e6c1fd9abc6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'93f65cfa-c3d9-4708-ade6-52ad26a0a54e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'93fd945a-9f55-4739-8b10-f0e872a382fb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'94180fbf-6a1c-4d54-9bbb-33a04a1e3c1d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9420c4c8-5157-43cf-acab-4703c917731a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9438cadc-c861-4c12-8ed3-6ab453f1892d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'943f3f56-12cf-4e84-8dfc-e31c8fec84d1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9460d0b8-6bad-4b35-b353-1bf144aa60cb'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS A3', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'948c0fbf-011b-42db-b7df-0c40a9ff626d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'949707f4-5aa5-4106-af24-797ff919bb57'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 C2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'949f7a41-c8d3-4b34-b8e9-7727a65e8c74'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'94aa1381-4c78-47b8-8c6b-817d5b776c19'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'94b4e32f-60a4-4790-82f2-f309848a0fcb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'94d0f304-ef26-409b-a54d-2d8eb001f112'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'94d9fad5-ed91-433f-8026-8fce7c56c6c7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'94e5017a-0eb6-4895-a63a-b45e25b0a12d'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS', LastUpdateDate='20101201 15:05:41.500' WHERE BloodProductGuid=N'95154028-f10c-4336-a367-378e956d12e3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'953c4b00-08a5-48ca-b4c7-947b6effb3a9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'954a8a91-11cd-4964-9d2f-8d0db8a0c281'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84 C2 D2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'959dced3-1ee3-4c24-ae44-426b332e984a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'95aed581-00f2-469b-93b0-02490867afe2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'95b159a0-9d49-427f-8a17-2b1174573d8f'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83 C2', LastUpdateDate='20101201 15:05:41.780' WHERE BloodProductGuid=N'95c42aa9-7e54-487b-b56b-f09ca836fa3e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'95d83ba1-0f6f-41ab-bdaf-5b730a38be62'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9631e952-bfdb-4c35-ab6e-e4bc33ecac78'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Plasma reduced|Buffy coat plts prep|Approx 180log9 plts', ProductShortName=N'PLT Pool PAS-B PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'964c4a54-0b0a-4417-8068-6ba7395fb720'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'964db408-c95d-4589-b021-ed68b2f2cf63'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 B2 C2 E5', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'964fe28d-33b1-4fa0-9e63-265dbc89a454'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'966c7134-76a0-4ca6-a9f6-915d6ff68712'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'967aba28-cfd9-4665-989a-6d1554b45cb8'
UPDATE dbo.BloodProduct SET ProductName=N'Washed POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Buffy coat plts prep|From 4 donors', ProductShortName=N'PLT Pool Wash PAS-B LUKOPR BUFCT 4DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'96ab4ad0-4074-4f8f-bc1f-c1fd4a10b5e9'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 B2 C2 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'96f36be2-7b2b-4a89-838e-b8951eddadd3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'97114df0-ed1a-4af3-b84e-54d81c4639e9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'977cf507-1d25-49c0-8a00-42fb7b63dde6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'979214f5-4021-4500-9876-d7caa1951d17'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'97b4c180-5631-4bd9-abf7-1b7069283c4f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'97d6b1f4-13d4-4e0d-bcc9-71bb000f9d3f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'97dc12df-d8f9-4801-bdd2-b88605b9dc3f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'97dcc505-7eac-4952-9b93-b912b028409d'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'97de0a0f-987a-4fc3-8ab1-9a4868255637'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'97f4d9dc-af46-4a28-9aa5-6b23c737a8c8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'97f7ea03-14f6-4284-8170-07b7e403b39c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'98074582-7f6e-4d6e-8651-6f0808b7f19e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'981350bd-20d9-46ed-ad43-e2d13d57e0d2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9823b601-4b21-45d1-9778-480252d64a6f'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9833ceb5-f93b-4ff1-aacb-6a242b347c8a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9857f2fe-a82f-43e9-8dab-770bc9fa6807'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9881ce9f-deea-4230-9a50-d5421f30bd81'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'988f5d19-9653-4441-ad5f-88b2ddfcfea1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'989850f1-1e80-4dda-962d-28bf10991765'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'98c5b914-6c23-4c51-b6bd-2ce47a9866bf'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container|Bacterial test', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'98e06cc5-c90d-4a01-b47a-355080fec083'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'99479587-569f-433a-bb69-e05e45b2a84d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'994b5a64-d58c-414b-8685-e0bdbbb20bc8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'995e9d39-8ecf-40fd-b58f-af62d2d62439'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'99845bcc-fbca-4b2a-86e6-e793174c3538'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'99852735-1003-4b6a-bdae-7a4ec3329171'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@B8' WHERE BloodProductGuid=N'99859dfd-3bfe-44d0-bf9f-ed711412a5fb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'99a97941-6827-486d-ad7f-103d9ab49c82'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'99cfe380-15f2-4a58-90bc-50fa69aa594e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'99da7903-9969-48e1-96da-4a5cd735d07e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9a4a60d9-07c6-46d3-bde7-445e7d8b6dd4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9a4ed130-eabf-4465-a52a-63a9035dcba2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9a6076f5-0864-4c9f-83d9-e624d92c86e7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9a68603f-c40e-4bd9-a441-bbde9236dea8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9a930be0-1e0f-4784-bc70-ca8bf9b18560'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 B2 E5', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9aa9e1f1-9059-4e85-beda-d9c8981b4682'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9aadc3f7-9470-43d7-8008-01e276d269ad'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ab6fc5c-196f-412a-b2c4-36b95b0d648c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ad82e85-4693-424a-80b1-bbfcb1723489'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ae0955f-c932-4bda-bda3-67b2c4a2ff4f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ae472d5-7c43-4f2b-b786-e3d42f2d3e57'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9af92869-3c6e-4c9c-9ce8-2d8194a2a642'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9b1a2408-ecae-44ff-b478-4a7861e835d0'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|4th container', ProductShortName=N'PLT AFR Wash PAS-D IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9b1af311-7ac2-4444-919c-1d782c0a619a'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83', LastUpdateDate='20101201 15:05:41.827' WHERE BloodProductGuid=N'9b3d535d-58ca-40ca-b959-6e461d8d8a40'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9b5bbc23-71df-4736-bb9a-256bc1a4c27e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9b5c0f3a-1682-4466-81eb-03f4faeb65ff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9b713d38-7da8-4761-b975-4e81b47336f7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9b77e22d-dd82-4fe3-a836-511ff8c91805'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9b7980f2-db9f-4519-b928-25a5d00f810e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9b79f44f-f6b8-4ab2-b5ca-070a40e28bda'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9bbfade5-2343-46fa-8c6d-a5d634c850fe'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9bd2b6b3-53f5-4892-97ac-e35bd7cbe6a0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9be15b29-5e61-4734-9bf6-ab7955ea0a6b'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|Buffy coat plts prep|Approx 120log9 plts', ProductShortName=N'PLT Pool PAS-B IRD BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9be922d9-cbe6-4b12-bd11-49ade02ddc1e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9bf47bdd-fe65-4479-a1b1-88af27ec9dc7'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|1st container|Bacterial test', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9bf54aac-4c32-4750-83c5-adede76a988d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9c071df0-3f34-49b8-a53a-9ea001cc7e84'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9c0b9411-27c7-4133-8746-d66f9238a1ed'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|1st container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-B LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9c5f1fc5-81b0-45f8-a2a0-834fe8e6e1f7'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 B2 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9c617d04-097c-42fe-a76f-3cb79bd90820'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ca011d7-a07f-4b13-9002-cf28f9ebfb39'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Buffy coat plts prep|From 4 donors', ProductShortName=N'PLT Pool PAS-B BUFCT 4DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9ca10d52-9cf1-4feb-bba4-d34b63957f8f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9cb0284e-c372-41a7-b593-2486dbe485b8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9cb6a610-9867-467e-a573-50f6e55f3f0c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ccfbfb6-3dc4-4370-b2eb-50ab7224756a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9cddd0b8-64a4-450e-9ee0-2f77e29361e2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9d1275a9-38ba-4a3b-b79a-789513e6e24e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@CC' WHERE BloodProductGuid=N'9d39e20d-54fd-4cf7-8470-57e71a2b2322'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9d3bf5c1-d4d6-44a8-ac83-9d998c8a07c8'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Approx 240log9 plts', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9d51b0f6-d308-4e13-8c2e-7c87c5f82ce7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9d539fb4-2219-469c-b3cd-808e7858c30c'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|2nd container|Psoralen-treated', ProductShortName=N'PLT AFR Wash PAS-C LUKOPR Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9d54fd17-df5e-49d0-a9d3-6d2bd5f9f464'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9d6ecc1c-7070-4667-858b-ac99fa3287d3'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|4th container|Bacterial test', ProductShortName=N'PLT AFR PAS-B LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9d7544a4-430b-4d85-aeaa-d183633ed839'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9d79fbfd-71ce-4035-ba6e-b063cbbdf694'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9d802b6c-d731-45b7-8c48-7ebcd845c955'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9dad85e8-8ba9-4f04-97e3-bfa6ce00e9ec'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Bacterial test', ProductShortName=N'PLT Pool PAS-D IRD LUKOPR BUFCT BacTest', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.717' WHERE BloodProductGuid=N'9dbf11dc-58f2-4f9c-97ac-18a1de774d49'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9dc632dd-5e31-43b0-8d3b-00dbaee14a9c'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|2nd container|Approx 240 log9 plts|Psoralen-treated', ProductShortName=N'PLT AFR PAS-C LUKOPR Psoraln', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9ddcd7c6-812b-4616-af27-85d0f28b2c2f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9dddd102-15f8-476d-b8cb-08ec2553eb9a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9de862c0-f0f0-45d3-b863-ec969edfee11'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9df5c84c-31c1-4722-aba9-7198f3fdb009'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9dfa676b-25df-4bd5-b67c-0d24c5c7a0b9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9dfedf29-c325-44f9-b862-97b847d75045'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9e0339f4-2c0e-4b6d-a0ad-9ee1ce965d7c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9e33b9b5-d90f-4397-bf0d-c4990e04b97b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9e4371a9-2be4-404c-8bb8-35382de0df34'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9e5fc2e0-d191-42b6-9937-844891e75bcf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9e668c35-b765-4018-9c0b-d5361a55746a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9e837dac-a0cc-4e93-a5ab-bb2d6780ed41'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9eadd300-400a-41cf-9b2c-08192148a177'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9edca461-c907-44c8-bf6e-b72d2f2d4318'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ee201d2-76b5-4c78-bf23-7d65ee73c586'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ee39617-b86d-41f5-a223-e725d341cd90'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ef3d17f-689b-4378-9e5c-ca8c93d78570'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ef74e2b-d608-4c41-b6c5-fafeaa43cd4a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9f0cbe2e-c588-4773-b92f-cb10ee3af8f4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9f3582b0-e6d2-4d6a-9e1e-01fdd0254240'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9f41e79b-760d-440d-b1a6-6b0f2dc155b6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9f434205-a023-466f-ae9a-f90b26b72dcd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9f92e55c-ab26-4d02-93a0-6e22fa90bee0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9fa9dc40-de0e-4cb2-be24-feabe304bbe6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9fcd2123-36d9-48bf-8b7c-896a8f47fc86'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9fd14dec-b64b-46a2-a32a-482c63e16c05'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9fe241f6-deb8-4a1f-bcfa-c09c3f8967eb'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@B3', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'9fe9ef18-e02f-4e2f-8cf0-4ccf9fa627c6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ff79d07-ef3d-411f-a2f4-2601f7e09cae'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'9ffab403-70c0-48f9-a16a-8be5be4c8b0f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a00acc09-60a8-477e-94c0-b100b84ee11c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a01e4621-deb6-4fbc-8818-f3dabd252324'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a04b365c-3839-4d58-848b-27bdcdf052ed'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a04b7569-3b2b-4ce9-89a3-56865842f376'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a06e3d0d-d709-49f3-b606-99c8b3c8ebfc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a087a7cb-d251-4f2c-98e9-17a52456f946'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a09b8886-2998-4d26-ae48-e4751a12991d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a100aeb0-3148-432e-bb0e-cdbbef8e62bd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a1163a24-e68c-49aa-a98f-5c4ab7c89f81'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a12c7783-c328-4e86-907b-35698d68f7a9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a14c23f5-351a-4c22-9100-585719f0fa25'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@91 B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a1608ad9-52d6-4a11-9196-4a88c6b00ceb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a16fac51-4cec-4375-82a5-8440c3fc53a3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a17015c4-2722-4284-a9d6-b6e4a328feb4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a1b2feaa-701a-48e5-8457-bf5e6a1d4d41'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a1bb087c-de1c-4b7d-aac5-d2bbcc965c7b'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|For mnf:injectable|ResLeu:<1log6|3rd container|Approx 240 log9 plts', ProductShortName=N'PLT AFR PAS-C MNFI LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a21ef749-721e-40f4-803d-c099e6ef5c71'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a2235eac-7e29-49eb-956a-1005b1fdf1d1'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|2nd container', ProductShortName=N'PLT AFR PAS-C LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a238e591-cbcc-4266-99e0-52d7157593f2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a23c273d-8d93-4d34-be1b-2ed40203f2f2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a2674b63-47b7-40f6-bbc8-0002da6666d3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a2b8db14-1332-4b20-aebf-5bd744328863'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a2d8f3f7-2b2f-4c13-a4d2-561b08382e77'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a3292720-b3ae-433f-9313-e0620c4b3fc5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a349ce5d-fe75-45c1-b0f5-114900edc81f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a34c97dc-13d1-43c6-b9bd-4f6f0c70a67e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a3647f4f-4e1f-446e-997d-640d82b423d7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a36e9f5e-b3a7-4d65-85e6-6c997146c2c6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a370385d-8c46-45f5-b126-60ebce0d8070'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a38a928d-6334-4b74-a471-928c033e7c59'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a3cb5855-38be-4788-85b2-bbb1813c8e88'
UPDATE dbo.BloodProduct SET ProductName=N'Washed POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Supernat reduced|Buffy coat plts prep|From 3 donors', ProductShortName=N'PLT Pool Wash PAS-B LUKOPR SUPRed BUFCT 3DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a3d6f2b4-675f-4e28-ba6f-b03f431364b2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a3d85968-3e29-4639-b8c9-3aeab481e94a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a3d8e580-f20d-4ac9-9612-1742b75cb68a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a3e3182d-a92a-465d-9a08-dbf6360c7776'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a403547f-b794-4d28-9550-a44ddf0a2181'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a408e2cd-0680-409e-a7d0-6c13218cbe3b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a40de53f-aa71-4beb-ae4c-4a9b8cf93e66'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 B2 D2 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a40fdc81-1195-40ad-9fea-e008f23137cf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a42517e5-472b-433e-ac76-93f2e8e51fad'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a42df799-cf2d-4259-b4fb-bd271f2469c4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a42f4033-3ec4-4689-8fb1-b070c15fcb63'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a440d79d-0b18-42bd-a1ae-c15fec6851f6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a47c8e52-836b-4fce-94eb-b67be1c1dcbf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a47ea0f9-9b73-4b02-9784-6ad15084e88c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a48acd4e-49bb-4294-98a1-9676ee64c106'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a4943235-b365-4ff4-93af-17df9bb4ac21'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELET-RICH BUFFY COAT|PAS-D/XX/20-24C|ResLeu:<1log6|Approx 300 log9 plts', ProductShortName=N'BUFFY PltRch Pool PAS-D LUKOPR ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a49ff3f1-09f7-467d-95ff-b918ecfebc3e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a4d8d215-1a39-4a34-b837-b9ed59118b9d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a4e08046-7849-49e8-aaff-837787537b83'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS C2 D2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a4e67db1-17ab-4908-babf-1ab1460820d8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a500aa10-b727-4167-a0c9-f5d825f6b5f9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a53d9aa1-a9ba-4f94-9e5a-6abf388c7c17'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a5467b61-30df-41e6-a1cd-346c476b506c'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92' WHERE BloodProductGuid=N'a55003ca-bb23-4fc4-a1a0-ba68236e00d9'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 B2 C2 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a5614e6f-41ff-4709-aca5-a86cbba899a4'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|1st container', ProductShortName=N'PLT AFR PAS-C LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a57eefbd-d6b4-4c87-a0d3-b7523038c81f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a58538d4-75b3-4ea8-85cc-7faa7252c54a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a594a1f4-5539-402b-88a9-1a04c7f3e0a9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a5f9f5ac-6fd8-439e-a18d-8cd22fbe3671'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a64ba3d6-35be-4308-92b6-65fbd8e2c449'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a663ce07-07f5-49e6-be7c-fca61c273b01'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a669b3ed-d55e-41e4-80f9-468d6d65e499'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|4th container|Bacterial test', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a6aee923-c778-4571-b201-f0feeffc2e35'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|3rd container', ProductShortName=N'PLT AFR PAS-D LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a6b8489f-dea1-42e2-87ec-6d09d1b01133'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a6c61e01-6d38-40c5-ab26-5e7c7a1674c7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a6db0b48-eacc-4278-9256-cfb6a22debf9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a6ea87ca-8ac7-4405-90bd-6478468e30b2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a6f23d0d-085b-4489-a37e-8c7799ea38e4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a6f28004-4088-4f75-b754-98539779b170'
UPDATE dbo.BloodProduct SET ProductName=N'Washed POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Buffy coat plts prep|From 2 donors', ProductShortName=N'PLT Pool Wash PAS-B LUKOPR BUFCT 2DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a6fd35fa-93ed-44e7-9f92-7ce1106ce98b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a70f4723-127c-4f7f-be82-e476d0716a5b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a70f6c27-2463-4ac3-a6a7-f5db97a21626'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a739e42f-fb25-4f46-b126-87212de92df1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a74790e1-f212-4253-b61d-5165a8437f12'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a7511d52-8ca4-44a3-ba23-d0901971bee9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a759992c-9eee-44c8-813b-3d34487dd86d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a75c6fe4-d6bf-4322-9011-d562b4ffb020'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a769c1a0-38b8-4449-a7fc-05a4cebf69e5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a78b56f5-06a8-4b5a-9c14-e5de0d8309bf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a7b9668e-28de-48e6-b439-f0af63f4661c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a7b9be7c-e284-428b-9bfe-19e7db69ed2f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a7f8a12d-2750-4a81-a244-fa19d2e9131a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a7fa67b9-028a-4092-a23e-27f91b7efe19'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a804056d-fbc1-49ca-9323-537f6449bb96'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a8094b24-edbc-4ec5-92bc-cca92965bf47'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a80dc102-93c0-4384-a54b-caaa495d6807'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a81e3021-2615-4624-860e-47c064c28fd0'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced|Buffy coat plts prep', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR SUPRed BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a84ded86-7721-42ad-af29-007d475e4896'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.547' WHERE BloodProductGuid=N'a87c80ee-493a-42ce-9109-17a9bb73f65f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a87e4962-ede5-48a3-af9d-bc36bcca2a8f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a8883ce8-8c1e-41d1-9a07-d52a8cf07ab7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a89694b4-7b1f-472f-9820-b094d5fd37ec'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a8e07af5-e945-4172-97c9-14af3a4da37b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a909ce04-7677-4f85-96bb-e611e67a845a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a914944d-5bbb-42f3-a5e7-9d3ac6e22f57'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a915b863-7606-4e1e-b692-efad7f73dd8f'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@C6' WHERE BloodProductGuid=N'a91e9e5b-b4a8-4b48-88f4-b1d554b87011'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a927183f-7510-4fc1-9c0b-22c8e58b4a0b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a95127fb-cb35-49cd-af69-88fcd3743a62'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a95c7ef5-ccdf-4c65-9476-51c27c37ad82'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a98084b2-7565-45c7-993b-4bdf057d997f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a9989c07-f190-4cc5-9aca-097da3b13f2e'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-C/XX/20-24C|For mnf:injectable|ResLeu:<1log6|Buffy coat plts prep', ProductShortName=N'PLT Pool PAS-C MNFI LUKOPR BUFCT ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'a99ca001-3fb8-4753-bfb0-8801584e626b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a9bf7aab-a275-4810-840c-be5534f00592'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a9c255c8-ee9c-49e8-95ad-3918895b4b2d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'a9c363a7-b814-4cd6-b41b-1fd70860c7ed'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'aa0be3e6-71c7-4069-aa6d-2674a517af89'
UPDATE dbo.BloodProduct SET RetireDate='20100201 00:00:00.000', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'aa16983b-4f01-4b68-8ce5-81dc60fb69f5'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container|Bacterial test', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'aa2b73ad-c83e-4777-ae28-90b9c09a9541'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'aa513a64-48f2-4202-b441-6ec31f9dfe91'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'aa975df1-b998-46a0-a6a6-e96e44f4839b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'aa9c94bb-5caf-4c40-92b8-f998442a50f1'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 C2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'aacf51a7-0ff3-475a-a670-a716dd86bf19'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ab10af18-fa18-4fbd-969c-78c7c90d2007'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ab2b6622-b500-46b0-92e4-b031fc595927'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ab2deef9-6c4b-40a5-bca6-878738401ebd'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS D2', LastUpdateDate='20101201 15:05:41.513' WHERE BloodProductGuid=N'ab74ca22-ebdb-41bb-8d83-16582fb61981'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'aba6a992-60f5-45fb-b043-909d5b270e24'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@95 A3', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'abb473d9-b9c9-4730-93d1-e0d134a71389'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'abdafdcc-521a-4519-8b08-791d29df6f3b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'abdd0322-5759-4373-8050-f4c15cf3a41d'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Supernat reduced|Buffy coat plts prep|From 4 donors', ProductShortName=N'PLT Pool PAS-B LUKOPR SUPRed BUFCT 4DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'ac05d880-ed01-475b-a755-566669ac4b6a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ac1681ea-ceda-4b1a-b899-de64a6d4e66b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ac33e10c-685c-4e9a-9564-660c43801cb5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ac34fd39-7b6d-45ca-8d0d-7ecfc8dd7781'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ac48c4da-bf05-40b5-86cf-1d2780b72cf2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ac826d0f-f966-447d-bf5b-2d7caff74d32'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ac854129-d2b6-4d28-af16-a921fb86014e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ac8f3c1c-f173-4485-880e-9795a8338a49'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'acbf11e7-7d78-4970-b342-dcc826881510'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ad0700d9-b1ae-4419-b8f2-359729a92144'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ad3a0500-a50e-434c-8be2-df27b20e9bea'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ad76fe7a-dc1a-4f6c-ae19-288dce3d8415'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ad9fc5a8-aa0a-4d6c-8594-d914de76476d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ade47b22-f427-4e16-800c-0e9203b5e91b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ade49671-430e-41b8-985e-6dc65da94cb5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'adfec4d7-3ff2-4a52-ab00-98a1b058db86'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ae227866-0ce4-4ce8-b2af-5436cf831f9d'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 D4', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ae3121f2-70a8-449f-87e0-47ecd3f0def4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ae457ca7-fcfc-4386-935e-f929dea40af0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ae66ddf4-e272-4df7-89f6-8163ec1864ff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ae74547d-9dce-479b-9c13-cfdce20bb3b9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ae8c98c0-4252-42f4-9e7f-221919473fac'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ae8f9cf9-4629-4ada-927d-859d77ab19c3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'aeaf2261-351c-4b8c-8fba-84d41ef4dbb4'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|Buffy coat plts prep|Approx 180log9 plts', ProductShortName=N'PLT Pool PAS-B IRD BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'aebdfc4c-04f4-4528-aa85-f3104d471817'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'aed31213-c44e-4c3a-bdf1-fde279faef09'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|1st container', ProductShortName=N'PLT AFR PAS-B LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'aee3b2a0-e968-4042-a9c0-675bf05628be'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84 C2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'af0bcdad-36c1-4ae9-87c7-422826325303'
UPDATE dbo.BloodProduct SET ProductName=N'Washed POOLED PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Psoralen-treated', ProductShortName=N'PLT Pool Wash PAS-C LUKOPR BUFCT Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'af31795e-795f-4ff5-808f-e262d528c68e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'af39ddd1-2c38-4e69-928a-8e8457f2e580'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|Supernat reduced|Buffy coat plts prep|Psoralen-treated', ProductShortName=N'PLT Pool PAS-C LUKOPR SUPRed BUFCT Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'af51996a-c66f-41e4-8793-8a6f6f710a2e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'af52811b-736f-4b02-bfaf-8bc3e293db33'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'af579068-9d61-49e2-9f0e-3267bcc133a5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'af5ea588-0de5-474d-b00f-72f6f0336047'
UPDATE dbo.BloodProduct SET RetireDate='20100201 00:00:00.000', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'af6955e3-f26e-471f-b899-a41addf89088'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'af9c2eb2-5264-47d0-b5f3-fc9353d83471'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'afb8da63-4e1a-4cda-8501-7468c4887207'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'afc7de67-96b9-4104-b1c2-5fb941974b3c'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@C5' WHERE BloodProductGuid=N'afca9749-29bf-4dff-998c-013967b97ea1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'affab2ac-cb0c-4e17-87eb-46c931209efa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b00f3891-2e78-400b-aa59-2f70303e615b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b02908d3-456f-464e-844d-81888e3d4b31'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b0355af3-f0cd-4118-aafa-aa02c73bc1d9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b056c52f-205f-4822-bc65-1cd51dfb4c1b'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b07edf47-b339-47e2-8dd9-f156fd20ad19'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b08c5a34-c12b-47f2-88bd-18c9cddbc6c5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b0ddf3e6-98d5-4559-9b64-e69ef3dd0a38'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b0f50092-f58f-4969-ac3d-79f221f46bb5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b10d982f-8784-4079-ad81-01841e9aee32'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b1269a74-e949-4007-97ed-f4fa29806886'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b13f2da8-9ef1-47c2-aaf1-b10de698b107'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b1420b61-6f19-4d0f-8eef-4e54cb97d38c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b1462fb5-16bc-486f-a3d9-92159bc8968e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b1649f17-f670-4c3d-8f62-4579daae014c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b166cd8c-bc24-4b29-93f2-15321f744cd7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b16fae06-a2e1-4ad1-9e90-afebe05eaef9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b18db563-04cc-4d00-a6fb-e7b001b4f85f'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<2log5|Approx 240 log9 plts', ProductShortName=N'PLT AFR PAS-B LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'b190a1ce-2a07-48f8-9dcc-cf92bd8e8553'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b1d2afed-9bd2-43d7-9398-1e92ad7d91a9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b1f26a42-a8d3-4d6b-9a17-6c53291fafdc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b1f9f326-0ad2-45ba-927f-ca33fd678d1b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b22dba62-3c78-494a-aba7-dfed5e3f3f35'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b2609bd1-c5f6-472f-9ed9-af20f89275d3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b281420e-c0df-4ee6-9157-e776e093c073'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b2936ade-293b-4512-9bb5-0166b80bcc91'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b2c2890a-bc58-4340-a811-878955c4e1e0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b2c4b691-af23-4f08-94b3-c510b401663e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b2c8891d-0603-4310-a9d7-70de83441d6c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b32cf506-a0a0-4799-9257-2cc1ed707390'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b3351a3d-a6b9-4331-857f-963e0f3ba2e9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b342097b-a12c-43c7-8bfa-e951991486bb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b35aad53-6695-41cd-bdb2-77a3855b0d62'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b35f4d04-95ad-4228-a1f7-37337cf6b80b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b3878283-a225-4d82-87a6-3d195aa5fcdc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b39235ad-9e32-4382-b959-c9c630d2f46d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b3a3868e-a796-43e7-930d-8da975dc68b6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b3afe139-989f-4720-a837-92417b059170'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b3b6cff8-bf3c-41f0-9dbd-a9b9ef75d484'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b3bc9249-10e5-4d9c-93a0-d240ea529b6d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b424211e-4558-451c-b2df-b2b2f7d8bfa6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b43a6ea6-8334-4fb1-8151-61333325544a'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83', LastUpdateDate='20101201 15:05:41.797' WHERE BloodProductGuid=N'b46072c3-c9d0-4238-a765-4a458b6a8e21'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b470b179-07ac-479e-8b14-0e5b677f46b5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b48282a1-8c43-4072-847f-46c7cc145a9e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b4844c4b-2186-4649-946f-908a99cf3a81'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b48a957a-0d06-4a64-816d-ccb862a92aa8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b48fd89c-f402-4ab9-ab04-75ab5b02a345'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b492d2b5-9826-4af5-ba7d-7e09c527c43b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b4ca1bcf-61ae-4ac1-aab0-b3750feee769'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b4caf7b1-98d6-49de-a53a-facac6c66218'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b4cf3e2c-c0f8-4631-9167-a9f044fa82e0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b4e3ae0a-aac4-4870-a60f-d06bc20d5fbe'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b4e63a87-7842-4920-b4b4-408e1f7fac6e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b5148ba4-330c-450e-bdac-f71e43653665'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b53d63e2-b964-4b6a-973e-e97d0f4665b6'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-C/XX/20-24C|For mnf:injectable|ResLeu:<1log6|Buffy coat plts prep|Approx 240 log9 plts', ProductShortName=N'PLT Pool PAS-C MNFI LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'b5634580-7c8c-4e7c-9d2d-b34d42173689'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b5bbfd24-4b33-46e4-b140-34c4e4e4d46e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b609b2b9-580a-4c77-8a9b-52ff13140e00'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b619330d-a128-425c-8711-555e5c0e10b5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b6517583-e492-487c-a9fb-cab33b25161c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b6519942-6400-45df-8c8a-df1003e287d5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b652e23e-7d8e-4947-9f2b-69b5a05b50fd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b656dc06-4024-4330-a73b-cd95dd832a85'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b6656b35-dc40-4053-96b0-0b6d2f04ff37'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b6735782-bfdc-493c-8a6f-af24dd4c3881'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b67461fc-f8cb-4f49-b890-1260e72795a7'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83 C2', LastUpdateDate='20101201 15:05:41.810' WHERE BloodProductGuid=N'b68a9477-2e38-4314-a07c-f17a4b1cbf30'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b6988f3b-e28e-4aa6-b9ac-42d38615db79'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b6c3558e-94d3-4694-9cf1-9d7d56f73a3b'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'b6c7dcd8-8548-4b2b-916c-305be5741cb0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b6d4afb6-ebab-4b7f-808c-24a2b1a8957d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b6dd8e34-3f9f-4e3b-b9b5-2782bb2f03aa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b6e250d4-6c9c-4763-98e4-1161c2fd929c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b6e9c554-3f2d-4000-9552-c2bc1430185b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b6f00165-1c25-4338-a6e7-f080908196b8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b7104bdf-c779-4574-b543-5ed3244507b5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b718f791-5c94-4743-b657-385e06a267d0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b72efb2d-3011-4581-be13-d52219456ae9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b739ccc4-55a3-4d5d-a056-7192ec13975c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b7726a4c-fb89-4432-a3b2-976344552898'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 B2 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'b77ab35d-cf9e-4d99-85d7-bf478bc2ddfa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b7c9f1fa-b746-487c-a34b-1c63162b91cf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b7cc7958-06a5-4a16-9416-da3599228e09'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b7e67167-395e-4a24-94c3-2af5fa7dcf37'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b8410a4d-13ed-4871-a082-dd86771f8322'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b8449908-fd68-4876-b0ef-57fd864e8977'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b84d1341-79a0-42b0-b951-d4255fd58661'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b87ef841-6098-4834-8dc6-039337506dcf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b8910215-bba0-41f1-b279-2808e9b6c9f3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b8929f1a-ece8-4014-9440-f41992a44698'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b8ccb9f1-b56b-4eb2-ba02-57df39efff8c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b8e04eff-4a9e-4fac-b6ab-f45fc89b7168'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b8f8a95f-a77a-47ff-ae7e-99be594be743'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b8fcff41-55f3-4f54-b8dc-3a205ef7a2b5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b940be40-0295-4e02-8064-ecbfb92c4088'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b94334d4-2847-40f1-8e13-09658199dc78'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b9825717-ca09-4408-b9a8-045f0e4de9bd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b9a72106-992c-4ff6-834e-4745960ed5b4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b9dfcb17-ed4f-4e5d-8b03-a58c18759b76'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b9f73a88-1c89-4176-9bdf-4128c8cce58e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b9f90562-9707-4d75-8370-b3f2485243d0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'b9fbf7fa-add4-4b37-9184-0771dcaf0be0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ba0b299a-e838-4cfd-b63c-159607b4a0b7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ba11e29c-0776-4b98-b54b-07dc01431bcc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ba314d06-39c9-48d1-887d-8008d8d5c70e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ba37dc62-3dcd-4631-b28f-76fe6c0aea53'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ba465f2c-c434-4365-9a5c-75aa9ce5a15e'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Bacterial monitoring', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BacMtr ', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.623' WHERE BloodProductGuid=N'ba53ca22-c00c-4ec9-a00e-359e1244d8b6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ba57ee8c-5e12-4ba9-a601-b518fde6aac0'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@BW' WHERE BloodProductGuid=N'ba7b909e-d350-4926-ab26-7b78a54f2654'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ba7dd334-d7f2-446e-ac3f-604a465bf710'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'baed917d-5f98-48bf-b2d1-61767d9fba83'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bb1569e3-8984-4a80-b0ae-c06a0c75773c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bb32e6f4-be25-42f4-b015-c3aa4d81a8b8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bb4cf0ea-a066-4d53-8fab-421acc4d2049'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bb72a5e0-d8fd-4a0e-8e24-3a28b2f4b39c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bb76c632-f7eb-4552-9b82-413100f08e85'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bb7b28b2-7c1d-43bb-a7f8-23144e291ddf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bb92d448-406b-4ab4-97af-1603fd9994ec'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bba91000-ba4e-4208-8061-e9fafd3d2fe0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bba9543c-334a-4fbe-a74c-fb2450374848'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bbb0ea95-b393-4a05-89af-791fb66ccde9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bbb56cbd-09f2-4b08-a6bc-591a2a6509aa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bbfa6e73-c436-4b97-b109-527c4dc3d0d4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bc0c05db-de5b-49f9-9169-538d93bfcb40'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bc1218ea-38a8-4069-b292-fcb265827dc7'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Approx 240 log9 plts', ProductShortName=N'PLT Pool PAS-C LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'bc5ce8f2-2351-4de2-8e70-859e769d5dbd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bc707dcb-98be-4bd5-84a6-e375681d87e0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bcac14f5-7883-430f-a047-a969d69f908c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bccf2864-7743-4181-b592-1ccf18bf6576'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bcd35da3-a961-473c-92b5-dcda4ddd5766'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bce4a9c1-7957-43c6-b92b-7f4058ce5962'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|3rd container', ProductShortName=N'PLT AFR PAS-B LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'bcf42a26-f01f-489f-a43a-5e007696c4f0'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|3rd container|Psoralen-treated', ProductShortName=N'PLT AFR Wash PAS-C LUKOPR Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'bd22de49-8255-4607-b5ad-7bc3f74bc15a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bd28eeeb-0d5a-4bf7-9075-6d816b7238ae'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bd28fb18-9824-4d17-a020-2cea534dcfd6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bd38cf4c-6054-4b14-a74b-11b595c08cbd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bd3ecf9a-9455-4a7d-aa7f-6c6667ef943d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bd6a5039-354f-4729-aeea-c3a1e1de4c6b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bdb3c0ec-6b3c-49dc-8ec5-b8b0c123afd9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bdb6bb76-3c46-444d-90a5-4198573990b9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'be4a988e-f8e3-4f46-bc31-6224edee493e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'be6fffaf-3c97-41dd-92ff-31c390b50e73'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'be77b065-fd88-4ddb-900f-ddc25d72dd6b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'be852f17-0020-4afc-a800-5c00c4a52d20'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bea89edd-0793-4441-bd02-96032626a77a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bebfa1b6-fd1e-49b7-b35b-9a25c3489212'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bec50adb-cf53-4e32-a26f-b020a1e3aa66'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bec7abc7-62e1-4636-8d0a-043055b94e66'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bed09427-51ab-4748-b937-2607a3113f9d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bed18717-b5b6-4e65-aa9e-83824cbe2694'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bf023cd6-4749-4683-bb76-7886d8a638f2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bf269664-4231-4dc0-81f1-9713d8c5ce9f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bf3976fe-b4cc-4ca5-b4ce-5d292ea8209d'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bf6fcac5-bb15-416c-b009-82cd1ba5f25a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bf7ccf8d-5afd-40e7-a987-a9c3336eeca6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bfa9fa91-7405-4569-82b0-9b40796432ad'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bfb2bf86-883d-4fa1-8f0d-ad1fddd5c645'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bfc01f75-9bbd-4e7f-bbd8-bb46a51f3c2e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bfd703f7-95ac-4f64-a074-3e7a5c4b9510'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|4th container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'bfe834db-94d8-41b3-bc32-413d1c2dfbf7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'bffad210-f917-43e9-bb44-cf66fb7484ed'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c0464b9a-bd4d-40f3-bedc-e1b8441a53f9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c087ddfb-4ed2-44ae-8a23-f74891e3b11f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c08fb7f1-c2e0-4f6c-a502-ea5260db4f15'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c095b482-33af-4a4b-8f82-95d1cfa51b7b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c0960a9a-d99a-4cec-ad63-c614061583f2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c09a7f8a-ca87-4194-9258-e04ec05cf77b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c09ba01f-7969-4409-9906-f718d498ed9a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c0a2d0a7-0071-4671-b861-30f113c93c4d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c0b3430f-6b5d-431e-bd5a-51cfd6d9d1f6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c120959c-7154-45a9-ac6d-35be798bb0e9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c12909f2-f5a6-4327-b341-eac3baec91dc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c15de35c-0958-4058-931a-0ec014c1faab'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@17', LastUpdateDate='20101201 15:05:41.467' WHERE BloodProductGuid=N'c1842fbc-7350-4da7-a70e-8070be2803e0'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|Supernat reduced|2nd container|Psoralen-treated', ProductShortName=N'PLT AFR PAS-C LUKOPR SUPRed Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'c1cefe64-2957-46c7-acf4-2eaf1bf9a37c'
UPDATE dbo.BloodProduct SET RetireDate='20100201 00:00:00.000', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'c1d11744-9590-4cba-bccd-f64ed6bd5889'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c1e09f6b-6d2f-41dc-a34b-125581015017'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'c221fa07-3715-4bb6-bc8d-18d66dea7871'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c2320e10-18ea-4d95-b954-12bcd5d1c93f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c250ecb1-33ae-4258-b04c-a561efe90a51'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83', LastUpdateDate='20101201 15:05:41.763' WHERE BloodProductGuid=N'c2692b42-a2af-442f-a42a-2ee326874415'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c276c0c4-ed69-4da4-ae36-1e41950bcac0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c2d625b7-0713-47bc-9430-cb438cf60925'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c2dc29c2-1da3-444f-abdd-0c09b8a31e3b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c2ebf0e6-1fec-4e27-96d3-26a1ca727efc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c3013efd-5eeb-4718-a9d7-ba404c150778'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c31298f5-047e-4d7c-826e-619336266529'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c32f7129-1c37-48c9-8220-348f13d08c94'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c3a419bb-24f9-4628-8811-c8737472159f'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Approx 240 log9 plts', ProductShortName=N'PLT Pool PAS-D IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'c3a86b64-5b76-497f-8834-a2f3077627ae'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c3c495cf-39ad-46d7-a9de-4bf33d8ac657'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c3f7cf41-c038-480e-969c-6828eeef3873'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c425dd42-3ceb-4755-82fc-6f0c7c5ad48c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c42c7004-bf08-4e73-b682-636734387dab'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c45662a7-b59d-48ef-8905-33bc68bad6c8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c456d412-cf41-415c-8862-097ac61285a2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c457e0c0-7e0e-4240-a392-e389eb671ae1'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep', ProductShortName=N'PLT Pool PAS-D IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'c46c2732-9961-407a-977b-c00fb55a2d5f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c492f894-e86b-4f95-a1fa-a097934567ff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c4dd7803-70fd-4d7a-bbdb-597630ec7e9d'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 C2 D2 JI', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'c4e01215-48e3-4867-8fcd-3e931e482e9b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c4e1a309-b2d3-44b7-bdd9-000dfd4cd380'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c4fa00a4-a5e7-48f9-ba48-f6384ebbed23'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c52e2717-0a4a-4b68-959b-8bbfa9be3fb4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c535dd3d-e014-4d5f-ae42-1ec6195c305f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c5625a07-c3b1-4c6f-aeea-f0b1a6846fe3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c5a5a96d-10d1-4b58-8040-e06772404c25'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c5ba5382-9919-47d6-bc8d-1008f070c0e4'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@CB' WHERE BloodProductGuid=N'c5c14fe4-cb37-4454-9ccb-92b7c4a10fd0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c5ce0513-f12b-4bb3-9c17-78ed9803b186'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c603c5f3-2150-4429-a57e-5f56d5d5bc85'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c61f7624-af4c-472e-93d7-8bfe5d441297'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c644117a-2922-4e76-b021-bda4a5f30aa7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c644c0f1-335f-4bc7-bb84-e1369c98c100'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c69f42fb-c296-42a7-b404-0740519711f3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c6a1e396-fba3-493c-86ad-2b9629e58c7e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c70b616c-d668-4017-b448-7c152560a747'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c71f6908-1031-4ed9-9119-2effd0a305f8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c7bda07d-b8a3-4b85-9fb8-877e65300cba'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c7c6b068-a1d4-412d-a653-73eb49e41835'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c7e6246d-fef7-4855-ad28-1aa11c1dea5e'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep', ProductShortName=N'PLT Pool PAS-D LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'c7f9fa32-185d-40d7-b108-84fa4d22f195'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c8011fd5-225c-4328-8c62-ae891ad9679e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c8247654-c8d0-4885-9140-78a417e6a514'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c828fa0e-7991-44b0-84c8-4f17745989a9'
UPDATE dbo.BloodProduct SET ProductName=N'Washed PLATELETS|PAS-B/XX/20-24C|Irradiated|Buffy coat plts prep', ProductShortName=N'PLT Wash PAS-B IRD BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'c8484ee6-e0ce-4a38-b7b8-c151715ad17a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c8aa935a-dd57-4bed-bc95-917c08730244'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c8dbb9e3-542f-4e88-8bc7-7d6370f98bb3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c8e75aed-f9c1-4ae4-9303-9c77f68b7e53'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c920cc2a-c939-4f77-ad69-97a6e37cfe13'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'c98829ed-965c-43e9-ab4b-8d3bb5cdfc10'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c9a74e0a-2dc8-4d51-a4a4-59ae429bf110'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c9a9aedd-12cd-496f-9c1f-6db3be6b4998'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c9bc1aff-a356-46c7-b283-f0cb28c63147'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c9c8dd60-6d53-441e-a2cf-66d4c3b0bf9a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'c9f25694-9e44-4beb-8c91-53aa45606ac7'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|2nd container|Psoralen-treated', ProductShortName=N'PLT AFR PAS-C LUKOPR Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'ca063853-bafd-4b55-9df0-d32f1b7fb71c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ca1ad963-2eca-4767-97a2-5ca9a4986b06'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ca259a4c-b8e1-4fdb-aedb-26e14e53c693'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'ca2cb37c-969c-44f9-b45f-fb54d4a39535'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ca53e8fd-ddfb-4139-86f0-c14fcbf0f5ab'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ca5c0f64-8551-45f1-bfc6-c53ad3ca1d74'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ca6368b7-abbb-4515-a5ec-e590ab4f266f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ca6a9425-4a25-4f19-9d65-529dc9ffc92c'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|Buffy coat plts prep|Approx 240log9 plts', ProductShortName=N'PLT Pool PAS-B IRD BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'ca6fa683-55bd-4415-8b62-04607205973a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ca778ce5-ebbc-4203-8290-7ce5fa5731ca'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ca84a53d-9b34-4ceb-bc80-387ce875f729'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ca936df5-a9ab-4a3a-b089-cb221f6a4672'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cad80b39-2728-40ae-93e3-5152f7b95509'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cadd9151-a70a-4b85-8208-a8fba349a6a3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cae6da5f-99a7-4823-8d5f-b21d52e32f4f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cae6f0f3-bd36-41ec-8a0f-0f49d7ce5a65'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cb4f840f-e12a-437b-b507-3c63e86a1638'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cb72c9de-a488-4351-bd6b-efe1e2cb6f20'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|For mnf:injectable|ResLeu:<1log6|1st container|Approx 240 log9 plts', ProductShortName=N'PLT AFR PAS-C MNFI LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'cb79c708-3c1c-49d1-b6fa-b423a9f0a1dc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cb80211e-e808-4e4e-b54f-be5ae0a23924'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cb9256cf-267f-405a-8769-a9c55300d696'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cb9b99ea-9be4-4b47-b90f-47ce874f7006'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cbb1f262-d57d-4478-af51-d450574739f6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cbbb8b00-ea6b-4f52-a462-b97a5d258fbb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cbc53053-9c3f-4424-9ac7-c763ca98c619'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cbd27016-2fb9-463f-a52d-5b4b0a6a141e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cbe9f5d5-a8d3-4546-b488-4c37890ea842'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.670' WHERE BloodProductGuid=N'cc6c8a93-4fba-406d-ace8-3339e9137cb0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cc6cd527-d9e8-4d43-b8b3-d85dd6a286d5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cc747738-9bbc-40fa-ba5b-ab4de172ad04'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cc97afc9-39ca-449f-a283-0da363e6dfde'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Approx 240 log9 plts', ProductShortName=N'PLT Pool PAS-C IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'cca6e8ee-583d-4802-9802-8999e84b9ec3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ccd56fc8-0c86-4b3c-be2b-60b489c823fd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ccdb854e-2bc5-4339-8ba4-8fd9d4a93dda'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ccdbc5e4-bc4c-423f-9be3-182591d7bfe7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cce0edc2-f557-4a2a-a3e4-5844d6f9218e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ccfbb195-853c-4867-8acb-6dfb5e1dd499'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cd1078b9-13d6-4639-b809-6f50beb7f1b3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cd1374d6-31e7-4dbc-9e39-571fe1902ddf'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Supernat reduced|Approx 120 log9 plts', ProductShortName=N'PLT AFR PAS-B IRD LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'cd186bd3-d434-4108-9655-6d9f92c70011'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cd204ff8-9908-44f0-8128-f8edfff8a15f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cd2249ac-8b7e-46e2-97ff-56f6a649c0a0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cd3164b6-1dae-4a71-9071-b512ea5f23f3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cd62a120-913e-4704-8626-f2edd8213bc9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cd9e4f68-ee32-4c60-a8bd-54f1d884ff0e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cda1ed3a-2340-49ac-9ba0-ffe1e54c7a73'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cdac81da-cd77-4e13-8c47-4c03ab1a12ad'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ce02cd11-6bbe-4271-89c1-fb008acad45d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ce077d8f-848f-472e-87e7-b81ce8d8ce36'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ce2d908f-ae37-4484-9d46-869c5ebbc4a1'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Bacterial monitoring', ProductShortName=N'PLT Pool PAS-D LUKOPR BUFCT BacMtr', MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.700' WHERE BloodProductGuid=N'ce4f25a6-e77a-4755-87e7-0bf2ba0b2b1f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ce69e0f5-6ab5-4722-ad2c-2ec5ca45c14f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ce9c7b0c-7d17-4f71-8540-9064715c3ea1'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|4th container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-D LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'cebd8c49-d484-4a04-9212-efa2db066be2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cf25a787-1e0b-4371-9012-ee6bf6348e12'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cf2df929-621d-4cd6-90a1-8c875534515a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cf4a6589-b8ab-44a0-9018-18224d746080'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cf5c593b-64cb-40a0-864a-3061d2ec6325'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cfb16d87-3cc2-417c-87bb-3474ac792e5e'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Approx 360 log9 plts', ProductShortName=N'PLT Pool PAS-D IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'cfdee4f0-0c7f-461c-8c41-8fa52d6c251e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@98 B2 D2 E7', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'cfea00c2-94cf-4361-a850-380aedfb1449'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cfeb4953-c1f5-4241-9db4-e2843f13b0da'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cffba452-e80b-4671-ac70-605b4340fe5c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'cffc1e40-9642-4443-89cc-e06b9e924fa0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd02249b9-316b-4d24-98eb-559367927989'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd04c5d63-1489-4e6f-ba85-1d025625c5a4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd088c387-167f-4dcf-9a40-e6278c4fee8d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd0899f56-044b-4f52-a889-d4a1645a32e3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd09a741c-9f6b-43d4-895c-be0f7a813181'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd09cd63d-e7a0-40f0-930e-07e76c5fb09c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd0a050b5-4e20-4632-bc24-285c8c34655f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd0a3e4e2-c9e1-4fde-9894-4b38d6125215'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd0bf5288-be1c-4e04-abff-8a7eda12c7b1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd0c17c9a-8dfa-429b-82fa-69dd38b1558d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd0e6308c-8d8f-41ea-98e9-facd18517605'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 B2 D4', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd0fdfca6-fe42-42ef-b16c-852704245304'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd10dd248-309c-43bd-b33e-6ad9bc389cd7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd1379c92-4a00-4f1c-9b58-69fdb8ad7a2d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd1723e73-1dfa-4b35-a11a-8ef516a9754b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd175366f-9775-477b-ba70-dcf7d1a84922'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd17f2dce-cb99-4ae9-a588-227b589b5ba3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd1864320-514f-4ea9-a013-e9eb13fbea9a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd1a4d6c0-7c22-4187-ba2b-337227a0ddd4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd1b68519-7b8d-4944-b2e6-ab778c0714f0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd1b7c9dc-5632-4aa6-a2b3-51dbad08f6f9'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@CM' WHERE BloodProductGuid=N'd1f1de75-7f53-4037-96c1-b882bf1dd380'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd1f4d7a8-d672-435e-b2c1-8836d847e0c6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd1f83bd9-914e-4845-a6af-3b80aecc7c68'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd1fa5bce-1755-477d-9668-6d17cbc44db0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd20fed82-b718-481b-a7c7-fd9f912c77fe'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd25d3d54-f03a-4d7d-b98f-832144895755'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd28b1c61-3c53-4eb8-a7b5-6aab9c21d5d0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd2ac8a5d-d618-44fb-b0a7-a19e9ea2fd09'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@71 B2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd2c4f232-092f-437a-91a8-ca75a1dccdab'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd2cfea7f-3cf2-41bb-85fe-67e4fe388337'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd2fb7ee3-4e5a-4053-9405-afd9433d3771'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd3195c69-7902-4e38-9339-4b36c997d6b0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd32e6da9-16b3-4802-a5f8-2cff20c0af20'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd3665e80-be74-4f97-ab71-1d4dc169abe3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd36dcf5d-aff6-4bfd-8c66-a6d3e4b7378e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd377a97a-a8c3-4c35-bb29-994273258945'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd38252e5-32eb-4b28-9e71-7020d1079d41'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|4th container|Bacterial test', ProductShortName=N'PLT AFR PAS-D LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd38a0e08-3dcc-4754-85e3-44a5b16ec011'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd3a76b31-7960-46c1-a6ac-f83904585e68'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS B2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd3d6710d-eaac-4e4a-b3b9-359a02721b6d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd3fdaec2-7a8f-42d4-b25c-b7dc3a849e86'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd3fe2198-c8c5-4e16-ac93-d79849213e11'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd424daa5-a3f2-44fb-92eb-ca28a5c7f267'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd42eeb5c-429b-47af-bc4a-49184db1ec0c'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@30', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd455558e-1472-415e-bc36-cbd09a474421'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Approx 360 log9 plts', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd45b10c0-3aab-4849-b1c1-9e5e52019390'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|3rd container|Bacterial test', ProductShortName=N'PLT AFR PAS-B LUKOPR BacTest', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd460b64b-48c9-485d-9b53-d4eb8ed2d8ce'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd46783fd-3e08-43dc-a348-5a9ff4fcfad7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd4770e87-c046-4d2a-8555-8bf54d1cce32'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd4911cea-7b3a-48e3-ba30-92fb922ae067'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Approx 120 log9 plts', ProductShortName=N'PLT AFR Wash PAS-B IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd4a4e557-0bbb-489b-a981-3e84e5d3cb94'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd4b639e8-3637-45af-8f65-c0d882f95d76'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd4c77626-98c8-499d-8e1e-886a24246c61'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd4d0e1de-5807-4f13-9541-0854f43f3dab'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83', LastUpdateDate='20101201 15:05:41.780' WHERE BloodProductGuid=N'd4de56d5-e9b1-4e11-9460-431b31431a0f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd4e8fcb4-b7af-4db4-87e7-1ee598a17d6d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd51b58ad-62ac-4cf0-876f-d8ab209c9f16'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd51d3b01-d860-4d85-8439-0a626e789d95'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Plasma reduced|Buffy coat plts prep|Approx 120log9 plts', ProductShortName=N'PLT Pool PAS-B LUKOPR PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd5240773-bb02-4f57-a48d-2c91c150844c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd52cb57c-4946-4864-a68b-4c71a5535721'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd5311abc-5c31-43cf-901e-c747ca5db5ec'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd542eb2f-2d7d-48a8-9d33-8f0a1aed53cb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd56c18c1-e3bf-4435-ac29-8ee8bdb13a87'
UPDATE dbo.BloodProduct SET ProductName=N'PLATELETS|PAS-B/XX/20-24C|Supernat reduced|Buffy coat plts prep', ProductShortName=N'PLT PAS-B SUPRed BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd58432c9-0cd4-4879-a9b7-c50ccb91817d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd58b12f2-3ff7-41b4-9dfb-8d74d5bd4f0d'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@77 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd5aab6ac-e491-45f6-9a81-21832c1e6816'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd5abc95f-a03e-4e6d-810b-934be5b5b5ff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd5b77853-2c05-4d98-9480-0f2e9a961f6f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd5ccb6eb-43ac-47b0-850e-91ef2f165f1f'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Buffy coat plts prep|Approx 180log9 plts', ProductShortName=N'PLT Pool PAS-B BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd5cfc547-94bb-4778-b2ef-33f64a4402c1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd5ed8188-86f9-4a8d-bfd8-ff816b14ac9a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd612334b-a3b2-4e27-8184-755780d4e666'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd613e45d-aa4b-4c37-9591-031e8d89c299'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd62e38e0-83be-4bef-bafb-5f33c64debca'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd67be5c8-106d-4d73-9b47-29e1ad82be3d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd69b90ff-c0c2-4798-8665-0a3246231c4f'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', ProductShortName=N'PLT Pool PAS-D LUKOPR BUFCT 4DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd6dc3dbf-8230-437c-944b-f09f93a06c7d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd71d60e4-6596-4b06-921a-6bcdc5d1e3c5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd73b2d28-7aab-4f18-b7d4-86fb06a891b9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd758ce49-e018-4e45-8391-ee836044f8dd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd7692d54-0dea-403e-979f-f4e9a6afbe15'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd797beca-ed4f-45fb-b8bc-d55828936ac2'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Approx 240log9 plts', ProductShortName=N'PLT Pool PAS-B LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd7dc133c-3cc3-447d-b47f-c94fce904b84'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd80110c2-1cb9-4b17-9f2e-0b1a317e6a82'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd834acea-59ae-407a-85ae-a70d225255d5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd842d277-9ab2-49fd-ac02-732277ce2e63'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd8466f14-a8cc-4d64-9dc3-b21aa3ec8ede'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS D2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd8481200-657c-4f97-9b08-ad756967ba4e'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@91 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd861379c-4e10-47bb-a28d-0f6fbe3a64a8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd88cfc4d-9d62-45ce-9829-ba61c405010c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd899707b-5a6e-4e82-ac11-683a294b8f13'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd8add11c-bff5-4937-98e1-a5e3cb5334e1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd8c3a988-fb49-416f-8636-5aeb7edc9ca4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd8d5ffe0-7a84-4e25-afb7-459e1616d1da'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd8dac708-19ab-44d1-a2c7-17833e9b6110'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd930f2ee-6ba2-4e47-b186-838a2243d37a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd956ada1-cce6-4c0b-b8e6-4f00af351901'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd97357f1-be40-4de2-a05b-27bc83cd6927'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd97e55f2-d384-4792-b648-442edae3e7bb'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@B7 B2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd9898cd5-faa1-464c-931c-f2ba094d7246'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd989ecbc-5a1c-4f72-baf0-d7da867e3778'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd9c592a9-e4b2-4bbf-b293-b9db9a1debac'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd9f2e137-353b-43ca-8215-cd469e713ce2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'd9f74eb1-79a4-4949-acf6-48bb62223524'
UPDATE dbo.BloodProduct SET RetireDate='20100503 00:00:00.000', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'd9fc98ca-2bc0-4dec-a196-d4bd028360d5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'da121927-2345-4f29-98e3-5571bf002062'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'da228981-31a0-45f5-9e2c-b92ccf1b22c8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'da35ea21-af67-4b5a-95da-a0f28ea59481'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'da69b792-fd2f-465a-a6a0-2c0322ce5578'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'da6c6863-7ec3-4ff2-9488-3308bfc1aa28'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'da7639f6-f867-435f-a447-43a66555a610'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'da961b4b-e2e1-40d7-8c84-25e439460d81'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'da981cfc-5fea-4885-bd35-c8172952d93a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dad0cfaa-9d93-4696-8ca6-808349d345af'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container', ProductShortName=N'PLT AFR Wash PAS-D IRD LUKOPR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'dad4f466-4835-4838-b851-2560504a7548'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'daf8097c-cf60-47e9-acd3-934fba595e88'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'db20c658-b3c1-4b03-ab1d-17abf2c32017'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'db2595a6-a9a5-481b-89e0-8a61019f697d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'db40e0a8-d692-4646-8ac0-5cea77135072'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'db484828-26df-4555-af08-f04a39f93a9e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'db55f237-74bd-4387-8588-37d70ccf9adb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'db5bb915-b4d2-44fe-8eca-6d97a9148a1d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'db660c58-c4b3-4be4-9d57-e60fb62a41dd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'db7ac81d-6088-4eff-89d6-e61eed9294d6'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@28 B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'db81f2c2-4c35-4feb-8c47-9cf8fc329d5d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'db8ff156-1f55-42fc-b5eb-9bf61729eb2a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'db957a22-b61b-40f5-8d67-95dc2aa9c796'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dba5d15e-c220-44ae-b782-9995d97541bc'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@B3 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'dba665f6-bd72-43a8-873c-db0b0feaf7a6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dbe88b1a-42e4-4b2e-becd-57f61b43bb87'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dbea63ce-b550-4a7d-977e-19b261322aaa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dbfbf38e-5641-47d8-b467-24b74cef7d7d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dc3799b0-70dc-423d-a9a0-eb444851d7cb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dc8af030-2119-4b89-94b3-92eaeb1f710f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dcc8b607-c2df-4911-89c4-554541e0af57'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dcff10d9-a82c-4b48-8433-8e1aa3a7589b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dd278194-e3d2-4eb7-9fe2-8bbabe78951e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dd337ae6-2e19-4bc4-adf7-3e5a696d6fa2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dd40bca0-a29d-461f-8b85-80904e93355b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dd43bc3f-f976-476a-a763-f319e33860ac'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dd4e4dea-14e8-49b5-b17b-621ce283d972'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dd4f4e6f-1ec0-499f-8bdf-cccf0c8622df'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dd52cb0d-9c51-4a8b-b915-3869ddf55ec2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dd5ca20a-9614-4627-9fca-440f2111701b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dd5fbccb-564d-4d35-9a27-7786d8112f2b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dd82c5e7-239d-4a1a-a8a1-59c3510fc69b'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@CB' WHERE BloodProductGuid=N'ddcd3ff9-9988-40a3-b3ee-78b8825cc66e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dddcc285-e8ae-4659-9414-9fc8f74c9030'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ddfc16fc-c621-456f-bb28-1cd50022b8b3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'de071807-9640-4357-81ee-53a01a0aab38'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'de1c4d0c-2eaf-45a0-9f75-61823f2af174'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83 C2 D2', LastUpdateDate='20101201 15:05:41.810' WHERE BloodProductGuid=N'de2f2307-20db-4d91-b53f-998741e666f7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'de4ddbe8-4800-42da-9aae-ab0a2e21f0c0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'de5a87ad-38de-4237-92be-b1f2dab0b575'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'de714c35-6912-4153-9d72-81951b43d28a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'de8121d2-2ca3-40c6-ab23-4fa78fc78b16'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|1st container|Approx 240 log9 plts|Psoralen-treated', ProductShortName=N'PLT AFR PAS-C LUKOPR Psoraln', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'dec26bdb-4c0c-42f6-823d-ac28c80576f3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dec2fab4-2618-46b0-8f25-61b6a54ce0bd'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.653' WHERE BloodProductGuid=N'decb38ba-e006-4d17-8c3e-187062d0024b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dedf0595-81ce-44b9-914a-560a6dab7ece'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'def7d3c1-ac1e-4b81-8e29-ce3efbe7cdb8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'defdb9d4-5190-4dd0-9f7b-0acf23f0edc6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'df0d253d-3aaf-43a1-bdc7-56465ea402a4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'df2402dd-ee86-4a15-a64a-d632f0fdde5d'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@AS C2 D2', LastUpdateDate='20101201 15:05:41.513' WHERE BloodProductGuid=N'df3b7754-9c07-4abc-9458-a49d5ba2e942'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'df47e307-3be6-4ee6-9e5a-fecd5f3fafb6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'df4ef575-3037-4e2c-8a63-1b796e704827'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'df7ecd17-cf79-40df-b8df-7d0a763bcb35'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'dfc9b6f8-a259-4181-9396-708f6b168f27'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e00d34b0-529a-4c23-aacb-b109d9a7cb46'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e03a84cf-28df-4d28-beb9-65e5ea203efd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e044d8d5-4754-468c-92a4-2a0af0652cb2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e05826ca-9fce-459f-a3b0-22eec8e372ea'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e0669ed9-a4dd-45fc-990f-4238d4e6b015'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@CA' WHERE BloodProductGuid=N'e07b30ca-250f-468a-97e3-48046ee32534'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e0a2b2d2-8d04-4e0a-8323-83403a2cb09b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e0d0770f-c880-4323-8c2f-1f3c53554e70'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e0e4ab2b-5e1c-4181-85dd-77c83b07944c'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|3rd container|Approx 240 log9 plts|Psoralen-treated', ProductShortName=N'PLT AFR PAS-C LUKOPR Psoraln', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'e11a6e1e-53ff-442f-84cf-5e9234c462fb'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e13044e2-1ed9-4a36-ae7f-8d414dfecbe6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e13aa1cf-eee0-4e96-be23-6ca40b739918'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e13c5f57-ad7e-43fc-8f77-c9feb9372d51'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e189ac7a-fe45-4951-ab70-1719b53a1aab'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e18ab3c2-d22c-421c-b3cc-83175343938a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e18e0722-31a1-455c-a072-8cffcd2f4c98'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@10 B2 C2 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'e1914df8-4e9c-4a57-9f28-2301f8cbc378'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e192450c-0eeb-4e25-b411-5fed544b5093'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e1a3ee16-520c-49bd-834e-e77957ecc2ee'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e1aeab5a-0dd5-4417-a630-e83bd86c5f06'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e1e98ee8-fc76-4011-89f6-a4b4e37f4a6c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e206b99d-a844-4fff-9d29-c81836c176c5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e20af921-bddb-4536-ba36-887b52123f33'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e20e860c-888b-4dd5-be55-7d2d67d1769d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e21dec58-bb24-494e-a89a-32d11a481e31'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e24312a8-4521-434e-a484-e3c74f2c8bc8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e24d11e3-4412-4aef-aace-dfef671144f4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e296b6df-330e-4249-9ce3-31934418a568'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e2a31a85-bab0-4c09-819e-32fd44ed829f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e2b3d742-2be5-421b-bd10-405c856d1311'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e2eedaa1-3aff-4fca-b802-15a8be8d9608'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e31bd70e-47ca-49df-823f-a3f543b21fbd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e3505c79-cf2d-4bfe-9066-5ef2b8c449cf'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e3823771-6101-4d64-834f-e0171236f5e5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e38b463c-3f0f-4fff-8735-54fa7786087f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e39e2736-6869-49e5-98e4-453c991f51cf'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@BX' WHERE BloodProductGuid=N'e3d1139c-510f-4071-956d-178feb7132e3'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@13', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e3dfa37c-ef4e-46db-a876-d407a5472d98'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e3e0d245-95a3-4ac4-8596-78bb48b0d965'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e42485dc-6c26-4f95-b61a-8ce49ec1fb82'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e42dac78-7a13-49fd-b2ff-ca57167c3043'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e44a944b-58d4-46a0-bebc-e03991f073a1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e45953e0-50ab-4504-93bf-f714575667c3'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.607' WHERE BloodProductGuid=N'e4793320-91ab-4ba1-8739-31c67d31f6f1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e48a1ae4-b15b-4c0c-bf5e-cf453f09cd0f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e496e8b1-dca2-49d0-b811-ea1726ab3e38'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e49ed73e-e6be-4a4d-a7b3-b613596e84e0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e4a89a72-5693-4216-aa97-4fe7862d2342'
UPDATE dbo.BloodProduct SET RetireDate='20100201 00:00:00.000', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'e4b5b516-28eb-4d2d-912e-42852db09641'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e4ceb0e4-a6bd-45f1-83ee-eb8609981c0b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e50df5a4-7664-40ef-8138-97037cb91d67'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e53060ec-51d1-4366-bf8a-1c333a9d7564'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e5548a42-5f75-41a1-82d7-849d2df31725'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|1st container|Psoralen-treated', ProductShortName=N'PLT AFR Wash PAS-C LUKOPR Psoraln ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'e55c2a15-b9a0-4bce-9fb9-ece501459f32'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e5613c48-d296-4cf2-8c85-92465d484a29'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e5f1f80e-d209-4d0f-8d52-1ce2e379718d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e652a42d-6cd4-4e61-aa7d-45e6e5394607'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e6612747-475d-4cb9-adf8-3252fcf02489'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e69af77e-63c7-486a-958d-1a8e55824a7b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e6aeb811-b64f-42d7-8674-835ecd5bcee0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e6fdf87e-3c4e-4186-a2d4-7a12c2f87886'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e72d380c-94e2-40e8-b43a-8a187f222d48'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e75a3097-654a-4ed6-bbfb-f557e5d55953'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e78ca5fc-e692-4419-bf56-f361fed72c0d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e78fd8be-2d1b-46af-b729-20da09d23674'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e7c4404d-cb97-4781-b7b2-af0f6ecc353e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e7df202f-4f39-4711-be4b-c574553c6dea'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e7e16d9e-a8c1-4b2a-89e2-8aa3580a5669'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e7f7a44e-be91-4bff-9fc0-9796a7328f22'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Plasma reduced|Buffy coat plts prep|Approx 240log9 plts', ProductShortName=N'PLT Pool PAS-B PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'e814a484-db45-4599-9acd-7a2fd26c4db0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e81710d8-7947-4680-9f3d-5eebd3bfea35'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e824c684-451b-40ef-961d-b703fc672e80'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e83b2f9f-389b-4b07-bbee-aba2c8924431'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e84fd7f2-44b5-4c49-8afb-9c86a5f19265'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e85c7837-85c5-44d5-a5ba-523ed9937908'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@92 C2', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e860e4f6-b735-438a-ac2b-eef1a4d7603c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e874028e-e2ce-43d4-bd40-73ad01f51d51'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84 C2 K4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'e8859cad-3e9c-4e9b-8845-7ebdbc2dd27e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e8c5f842-5440-4c8d-92b8-c64de6a63e27'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Buffy coat plts prep|From 3 donors', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BUFCT 3DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'e8cb9c45-c321-4edf-892d-e78fcf9a1cef'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e8cdfde8-aea5-43c6-af81-d09a4d8d578a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e93d9376-7a92-4f17-be70-03af610cbc95'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e93e553a-99d2-447f-8756-b4380c06b687'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e93f8b57-8592-491a-a44d-8f92037d37d2'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep', ProductShortName=N'PLT Pool PAS-B LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'e94734da-a054-4bc0-a0e0-579918a9943a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e9553cbe-9098-4a30-ae9d-30c7579baf58'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e97d88c6-5c11-4d4d-a44d-f3e42a4456c0'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83 C2 D2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'e9862f4b-2248-4c37-bf88-3e93948710b3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e98e847e-13c6-48c3-b494-a2ab457d1222'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e98ff397-d7c6-4da5-86e0-5df2ccc07eae'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e9a25268-365d-43af-b920-ae8970a8504f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e9abe521-6bd7-4654-83e7-49540c294aa1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e9e27e7b-26fc-46c3-a025-2c85eba556b2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e9e2adf9-f41a-467c-826a-f70e6e71d705'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'e9ec4370-dacd-49f7-9e85-d6dbb7175ada'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ea00554d-5abe-4fad-86a0-37405c6e9fa0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ea14b146-9fc1-413a-91c3-089f4a33f07f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ea14c650-3d07-4a27-b54d-ee790ce4fcba'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ea74e646-aa1e-442a-b5e1-134433c735fd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eac3a424-c92d-4cae-b85e-637faead8f26'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eac40ecc-42c3-41f3-8a2c-ea059f1541d5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ead89291-548d-4a22-b25f-8a76053120b9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eae0e7ef-58b3-4806-9bfe-3458d8fa2d57'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eae1b739-697d-419c-98cc-8e39b18edd23'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eae4eab4-2661-46db-821d-1ab0d937d4e1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eafb904c-c816-4878-b5f4-97057e813794'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eb255f9c-6ac3-4ed3-86eb-fe5b8828aecc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eb396b76-1de2-4b5c-b472-9d6498623ca9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eb520a2a-8781-4859-96d3-c4a0f2bd4a21'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Approx 120log9 plts', ProductShortName=N'PLT Pool PAS-B IRD LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'eb685ab1-ded5-485a-a838-6c8537d98906'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83 C2', LastUpdateDate='20101201 15:05:41.840' WHERE BloodProductGuid=N'eb9429c4-9898-41ad-bc14-bf65c1515788'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eb987ac7-5192-476e-9204-515ef1229f9d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eba6d93c-d7c7-4123-8248-d68e0e8e4720'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|1st container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-D LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'ebe2d846-9f38-4495-a61e-87ed3cd95cd3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ebf909ff-52c6-4707-9740-a106df183cc6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ec4e956f-4462-476d-8d9b-e3127e57b86a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ec6026d5-a25e-492a-82b2-dd251d9d3db8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ec7498c6-6e38-41e4-a627-8c388f3a8a65'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ec918aa4-248d-4772-8fe2-e98e1da2f608'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eca0caa4-b5ca-4173-a9ae-27f112cd0a6f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ecbe7581-4178-4d26-b153-fdb43039161c'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|ResLeu:<1log6|Plasma reduced|Buffy coat plts prep|Approx 240log9 plts', ProductShortName=N'PLT Pool PAS-B LUKOPR PLS Red BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'ecf68c75-1224-4004-8f6b-56cb3bb8943d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ed32f500-4b84-490c-949f-e5febc09ad36'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ed37cba0-413a-42ab-8207-324d68ea0ff0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ed543ebf-3448-4679-be59-e600926739e0'
UPDATE dbo.BloodProduct SET RetireDate='20100201 00:00:00.000', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'ed578b25-7d3b-41d4-a24c-a91ad8b46b72'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ed5c4a9d-c7bd-4c0a-b4bb-81bc045972ce'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ed6af9c1-fbbd-4565-82b8-a8711185070d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ed7ddc67-e130-45d7-8a78-31d556d52923'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ed8d52d0-919a-4bb7-80b9-92a96e08831c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eda86712-d37b-4b8f-bb20-c9f95671d48f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'edcd7f05-93ed-4ed2-9831-464d91afc40b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eddd5ac4-78fb-48f1-9161-b2a7f40d4954'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ee318c38-dda3-470d-b674-799ab598cd55'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ee410618-49f9-4ab1-a0b9-419055f0db83'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ee464842-8ac1-4554-a6c7-218070018fe7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ee5639af-c049-493f-bc22-401a4ce22f5d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ee6361f3-d522-4a73-b479-de28c564c173'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ee848648-0a4f-4108-b506-16d3a9fe9865'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ee8c755f-90af-495a-abee-2d7c270a2412'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ee8e4769-0ef7-43cb-9955-993cd92263e3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eedebb1e-ffcc-4bb6-a711-2420e9ba77c2'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84', ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eeee8882-f32a-4cea-bd11-5f5e1339e388'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ef00b131-e55a-4a4f-9397-ee83bb2a5a70'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ef13923b-c5a0-4354-a26c-d0c76f7012a4'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ef18e9fe-25b1-4053-8450-4ed6834c95ca'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ef197bb0-d99c-4c5f-a7c4-64d1fb345d73'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ef1faa87-bcbd-48ba-aef9-460a7457f577'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ef30e1c8-7553-4347-b0d7-0df5d5b6ce9a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ef7e03c8-df92-4d32-988d-7f4c9e986349'
UPDATE dbo.BloodProduct SET ProductName=N'Washed POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep', ProductShortName=N'PLT Pool Wash PAS-D LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'ef8d03cd-28d5-420c-8c06-269a8190a463'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'efad28b2-27b2-4d96-abb2-20080ee69178'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'efd47c86-5a3f-46cf-8679-6475c31745ea'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'efe72339-3fbe-4d75-9e51-c8ed8c304700'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'eff659cf-f445-46fc-b4ba-6fc6df267065'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@C7' WHERE BloodProductGuid=N'eff93f9f-9dc0-4119-99f6-339aebdf216d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f02d0796-2248-42cf-9dcf-b1fbd596acd7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f0337d5d-e140-4991-bbd0-125ebb1be241'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f054993f-9279-40ca-8d44-6ba8b395f36a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f06ffee9-63ad-49dd-8227-4b947065ac8a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f08b62f9-6651-44d6-aafb-b7d0930dd652'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f0af45f4-0dea-4b99-8c6a-61582f295529'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f0bcdf57-1d17-4cfb-baa4-3c8799946ca6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f0bd0f62-80de-4012-b45f-23c48cc20334'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f0d9d2c9-cbb9-46e3-b89c-a0ffb3e241de'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f0e9f395-577f-45bd-9828-83bf29b38717'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f10cdae2-c426-41e5-8a3b-b20c5cc55497'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-B/XX/20-24C|Buffy coat plts prep|From 3 donors', ProductShortName=N'PLT Pool PAS-B BUFCT 3DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'f10f2b14-8cea-424f-9e3f-48a2489badf8'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f125f84a-97dc-4ab1-8877-6c38d6bf087e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f13ab787-505e-4fdf-ac3e-6bec74cff1ae'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@70 A3', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'f174576c-8603-46be-a515-7c45adaa7a19'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f187265e-6a86-4887-8a17-1ec96893c38e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f19e2e3a-4b28-44b2-869d-459f2840f803'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@33 B2 C2 D4', LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f1d7f00c-a1f1-40a7-8168-2484f5a34ebd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f1f83226-f4f5-459b-b615-a71b7b2fcd7f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f206b1f1-5538-4faa-898a-ae971072da79'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f2293ceb-c145-4612-8c45-4eea04ec29a1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f2927735-1a64-4c39-a3c3-9a028e5220b2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f29a0418-1c9d-4b24-8bf2-bdacc3a37701'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f29f50ad-0484-4cf9-bef3-abf6c26dee28'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f2a3032b-d882-4258-a6bf-3c5983b9dd01'
UPDATE dbo.BloodProduct SET ProductName=N'Washed Apheresis PLATELETS|PAS-B/XX/20-24C|Irradiated|ResLeu:<2log5|Supernat reduced|Approx 120 log9 plts', ProductShortName=N'PLT AFR Wash PAS-B IRD LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'f2b1f431-c674-4aa5-8402-e1f7f1b34ae4'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|ResLeu:<1log6|2nd container|Psoralen-treated', ProductShortName=N'PLT AFR ACD-A>PAS-C LUKOPR Psoraln', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'f2b8df38-730b-4511-a516-c882e44ef6f3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f2c81cc0-6c28-461c-bd67-3b01fd587367'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f2f12a12-ab5e-4d23-8f2e-124a6ce5802c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f3404584-2d8b-44da-b4cd-9d2e117de066'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f3408a57-4ae5-451b-9bab-ffc1e8eaeeaa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f3a83f90-6889-452a-a077-be0610061745'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f3b6c470-97f4-49c7-a5cd-e89f354db81b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f3bc53ec-8647-494d-9c27-c894cf6e8c6b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f41df4a1-46fb-4134-8bea-69aebc58b32a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f42b1342-8546-48ee-9134-3f260ee46096'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f447f73e-0e12-44b9-9ca8-3d64991745d1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f4867fd0-0ad5-40df-908f-e17abb0319fc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f493b51a-555d-4c24-bdec-65c70b492b07'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@74' WHERE BloodProductGuid=N'f4afa9c7-120a-4477-8c7a-32b2cf8e878b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f4b4d1c0-32b3-4689-bc0b-e79d1b185a1c'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 B2 D2 JI', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'f4b61132-de34-4bde-9a65-543a65aded77'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f4e784c8-c255-4b4c-9fec-05769510397a'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@33 B2 D4', LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f4ee9e96-d244-42d8-bd70-c02cb758871f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f5226ac1-af22-4761-9a3c-daedcb423d3c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f526acd0-db6f-45c3-8050-6ff376d5c534'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f5404bc8-793c-456d-93ec-f87393888b11'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f5413783-f932-40b6-9e4c-13e35723f00b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f54f6718-5698-438d-a595-dcdc96cfdf26'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f553563f-5016-40a6-bd46-23a8cc7c58f0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f5570a22-f198-4b6d-a0bc-dc4f3244386f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f57687e9-8814-4414-afd3-0b137be32249'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f5809f5e-abe1-4a6b-93f8-54b48db19113'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f5aa481e-2c49-4853-a1df-f23bd4fd1508'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f5d660e9-eeb7-4370-8b9b-78e491af985c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f61b3709-0c93-403e-b375-5bd4d0a334ff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f63c0a87-1b8d-4b6c-8970-c7235520c1fa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f646c562-eaa8-47cf-b6d4-d08ca81a0164'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f64ce0ca-9c5a-4bc8-873c-e6ee65c9bd94'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f652542f-bf73-4959-8287-e45025a7727f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f657edbf-c9fe-4fb7-8620-7d1e79d52c16'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f6605c2d-8178-4240-8edf-5682e0f0da69'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f6659bd5-51f3-4762-bf0e-457ccef4bbfd'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f66ca02c-2324-4164-8efe-0f4ba451a257'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f67b84dc-359e-4259-a1bb-d2ec4a29fa15'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f6983478-4420-44d5-86a4-8b2515538069'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f6a19b16-3190-400c-936d-6844c364fdb3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f6bb58c6-fac3-4450-8085-5856f630b93b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f6bc5d13-b869-4b08-99e0-75581d126b08'
UPDATE dbo.BloodProduct SET MaximumStorageTime=120, LastUpdateDate='20101201 15:05:41.530' WHERE BloodProductGuid=N'f6e57ee5-2faf-46a3-9efb-c49b3095039c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f711519f-a879-4076-af92-9b068c6814ff'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f7511bf5-3136-46c6-8048-0002d234403c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f75133e6-b14a-4cf1-ab22-b2d8e13e18fe'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f7593345-ddfb-419d-9e81-0e9ed7c71778'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f790e7cc-7f34-4cca-97ff-a7486d0345f3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f7a80419-ef33-49ef-b579-a875ec731f95'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f7c83c50-ccd7-4adb-8069-0c130d148964'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Approx 240 log9 plts|Psoralen-treated', ProductShortName=N'PLT Pool PAS-C LUKOPR BUFCT Psoraln', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'f7d5d1ec-867f-4d6d-b8e5-f5417ef2139b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f858dbb0-2e84-41f2-9cbe-e9e41d464a1b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f8688c31-68a0-4d87-9acf-b01ae776dbc8'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR SUPRed', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'f8744c2d-3bb1-4c13-a22b-4551ac41d9f7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f87cbf4d-bde8-499c-b2c6-4984479da587'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-C/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Approx 120 log9 plts', ProductShortName=N'PLT Pool PAS-C LUKOPR BUFCT', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'f897d7ea-af7c-4f6f-b631-7efca6645949'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f8a3caff-a7f9-4fa8-94ff-fd2d5b46f15e'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f8a536fe-da76-43aa-858e-4b0f912fc0a5'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-C/XX/20-24C|For mnf:injectable|ResLeu:<1log6|2nd container', ProductShortName=N'PLT AFR PAS-C MNFI LUKOPR ', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'f8afe733-e4c6-4384-a500-4dd073375da2'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f8b674e6-8b16-45e9-9088-92ca4a125373'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f8f091b8-7b05-4328-9b4a-e723c6e2f716'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f95b4108-713c-4779-8323-e7e67e18119d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f973833a-c69b-4694-ba64-6783d2d5c484'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83', LastUpdateDate='20101201 15:05:41.763' WHERE BloodProductGuid=N'f9b27c0d-3496-4b72-8dc1-47ebe0d9d7c6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f9b46ae4-4e73-4acf-95c0-0a8a34b98a28'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'f9d4b0a9-b926-4b9e-aa48-b7f845bff2d1'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@84 B2 C2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'fa131407-dd3c-4979-8416-72bb74f712a6'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fa1b5395-fac9-44b6-b4f5-0d70daacc0db'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fa1bf434-90d6-43a6-a03e-4dfa0e2d0dc7'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fa60610e-db80-46e5-a1d5-34ead12a1b61'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fa77341b-fbe5-484c-99b2-c6dc4b6e7bb8'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@71', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'fa7c2325-a8a8-4ae4-a8ee-71873ad7b465'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fa84735e-264a-46c9-9406-5c754639f646'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@83 D2', LastUpdateDate='20101201 15:05:41.797' WHERE BloodProductGuid=N'fac35294-1b99-4531-9494-1ef2e8898c38'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'facb493f-28eb-49f5-b5ad-d6377a32ea28'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'facdb859-c0fb-4dfa-b644-d5791ac050f0'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fad319c7-764d-4a60-b226-2861614be8d5'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'faeb0a92-0f18-43c8-8f29-1a7eec3c230c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fb176624-4ac0-4022-9f5c-ab2204ceb6b9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fb28a957-18fa-4939-992e-240b17a6dbf3'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fb3b622a-e271-4c2a-a8b1-48577ef6b570'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fb658605-d43b-42b2-8339-99e8373f7457'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fb881282-460c-4802-99a5-90b0036dfb4b'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fb89f11d-778f-409e-8f59-0aa6ec8ae395'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fbbfcc10-b2c8-41f6-b146-4a266e14bb43'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fbd91ed7-87c9-4af6-9d26-d0b9c3915f5f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fbfa6ef2-f58b-40c1-8711-92acf3247836'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fc2588c0-8fe9-429c-b82d-76d1401a0c35'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fcc0ecc2-dad5-42d4-98e1-34fd50e6c7e9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fce442ec-f56d-4b35-bd83-a1fe6c3aa6eb'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 B2', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'fcf59eb5-8ed3-46eb-badd-87cb0a80556d'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fdaba984-836a-49b6-b0d3-2bdb05698f74'
UPDATE dbo.BloodProduct SET ProductName=N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container|Bacterial monitoring', ProductShortName=N'PLT AFR PAS-D IRD LUKOPR BacMtr', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'fe0aa3c5-35a2-40a6-ba03-0971a9eb3547'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fe127ab0-da69-443a-9b56-127d0580c12a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fe28fd73-e34a-4715-b4c0-7578c9e7f781'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fe2a075e-22b8-43e3-8180-98cb926d530c'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fe3de465-bde8-4ac1-81f6-b8d987dd2561'
UPDATE dbo.BloodProduct SET ProductName=N'POOLED PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', ProductShortName=N'PLT Pool PAS-D IRD LUKOPR BUFCT 5DNR', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'fe3e5451-03d8-4097-ad5e-ef9923d4fdc1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fe402cce-ffc2-4bd7-ba22-24855d10b9b9'
UPDATE dbo.BloodProduct SET ProductAttributeForm=N'@72 C2 D4', LastUpdateDate='20100806 13:34:26.000' WHERE BloodProductGuid=N'fe476eb6-1739-46fb-9f84-643e4bc5c00f'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fe4c1af0-2430-4858-9bfc-046c6cc8c51a'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fe670c21-bf0f-49fa-ac3b-0bd82985e908'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fe6c6068-f555-449b-9db9-929dfc818d14'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'feb7c4ef-b23b-4eda-81cc-b28e9c4cd956'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fec854a5-54d5-43d1-b5fc-39ba1c0f72aa'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fedbf2c2-81d8-479f-8a76-fb432959acf9'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'fee6c847-8509-4cbd-a05a-7fabb34e6282'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ff1fc422-d917-4654-a15a-642556ae70cc'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ff211bda-3afb-4193-9e66-bf7dc1d15eef'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ff30828d-0378-4c5e-80b8-ca5a194c60d1'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ff889b81-2ef9-411d-82d1-9ba2474b3b22'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ff9d0325-1068-43af-ba88-d928cd6ed695'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ffc04628-4562-405b-bfe6-26bd0d3ad026'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ffe1f1cc-7979-450c-8350-9c1aecd79b62'
UPDATE dbo.BloodProduct SET ProductVolume=225, LastUpdateDate='20100819 08:27:00.000' WHERE BloodProductGuid=N'ffe3fb1e-db75-4067-8e49-6cffd76aad23'

-- Add 565 rows to dbo.BloodProduct
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'005481d6-569a-4efe-b7b5-ac49e7bb2a17', N'RED BLOOD CELLS|CPD>AS1/450mL/refg|Irradiated|ResLeu:<1log6|Supernat reduced', N'RBC CPD>AS1 IRD LUKOPR SUPRed', N'E002', N'@18 C2 D5 EA', 350, N'E6658', 672, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'005e3774-4499-4cfd-abb6-7a7cb7a45897', N'POOLED PLATELETS|CPD/XX/20-24C|Buffy coat plts prep|From 5 donors', N'PLT Pool CPD BUFCT 5DNR', N'E022', N'@A7 G2 J5', 200, N'E6884', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'012b6852-6de3-441a-a0cb-a247e2a4ccbe', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool Thaw DMSO OPN LUKOPR BUFCT 4DNR', N'E026', N'@77 B2 D5 G2 J4', 200, N'E7086', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0150b4bd-bba2-4be5-be94-bdb3ead37ff9', N'Frozen Apheresis PLATELETS|DMSO/XX/<-65C|Open|Irradiated|ResLeu:<5log6|3rd container', N'PLT AFR Froz DMSO OPN IRD LUKOPR', N'E025', N'@76 B2 C2 D4 H5', 200, N'E7016', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'01543a95-ca4c-46b6-a3f0-1d8a25e9abed', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool Froz DMSO OPN LUKOPR BUFCT 5DNR', N'E025', N'@CP B2 D5 G2 J5', 200, N'E7081', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0191846a-380a-4f95-a8e0-d08a8195fec5', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED (for third bag of triple plateletpheresis)(variable contents: < 3X 1011)', N'PLT AFR Wash None IRD LUKOPR', N'E027', N'@71 B2 C2 D2 H5 JI', 200, N'12961', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'01e29fa4-286b-4b6d-b397-97582bc91eda', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|Irradiated|Buffy coat plts prep|From 3 donors', N'PLT Pool Wash None OPN IRD BUFCT 3DNR', N'E023', N'@71 B2 C2 G2 J3', 200, N'E6828', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'022eb88f-6109-48c0-9e46-667adb9cf524', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 3 donors', N'PLT Pool CPD OPN IRD PLS Red/PA BUFCT 3DNR', N'E022', N'@A7 B2 C2 EE G2 J3', 200, N'E6903', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'025f2f5b-70d3-4a9b-9b86-cf81b5f25096', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-E IRD LUKOPR SR/PA BUFCT 3DNR', N'E022', N'@D5 C2 D5 E8 G2 J3', 200, N'E6959', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'02cfd55d-993b-4285-a2eb-1bf2493c4fad', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY (by pheresis)', N'PLASMA AFR None', N'E012', N'@84', 225, N'18641', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'02d1c398-93da-403b-b014-8104a1b2f171', N'RED BLOOD CELLS|CPD>AS1/500mL/refg|Irradiated|ResLeu:<5log6|Plasma added', N'RBC CPD>AS1 IRD LUKOPR PA', N'E002', N'@19 C2 D4 E4', 350, N'E7033', 672, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'03543bbb-6570-4de4-ad77-a3503515bc70', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced|3rd container', N'PLT AFR PAS-E IRD LUKOPR SUPRed', N'E024', N'@D5 C2 D5 EA H5', 200, N'E6766', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'03934de6-43d0-48e6-a345-4a3ab90d3562', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool Thaw DMSO LUKOPR BUFCT 4DNR', N'E026', N'@77 D5 G2 J4', 200, N'E7057', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'03958448-a664-4a2a-b30d-19b8b7c1d2fe', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part D or 4)', N'PLASMA AFR Thaw None', N'E013', N'@AS H6', 225, N'49214', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'03d22527-1af4-401e-a0a0-b46e0957f15b', N'Thawed Apheresis PLASMA|CP2D/XX/<37C|For mnf:noninjectable', N'PLASMA AFR Thaw CP2D MNFNI', N'E013', N'@63 A3', 225, N'E6734', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'03dea0fa-5fbb-4127-8bbd-78753296f181', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|4th container|Bacterial test', N'PLT AFR PAS-E LUKOPR BacTest', N'E024', N'@D5 D5 H6 N3', 200, N'E6755', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'042dd3f1-310a-4c4e-bc30-28ac84c9a359', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated', N'PLT AFR PAS-E IRD', N'E024', N'@D5 C2', 200, N'E6873', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'045e7881-3e31-46da-803b-8764a04997be', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 8 donors', N'PLT Pool CPD OPN IRD PLS Red/PA BUFCT 8DNR', N'E022', N'@A7 B2 C2 EE G2 J8', 200, N'E6908', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'047d53b0-1445-4d43-8c08-d8b0b6aa64dc', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 2 donors', N'PLT Pool CPD OPN IRD PLS Red/PA BUFCT 2DNR', N'E022', N'@A7 B2 C2 EE G2 J2', 200, N'E6902', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'04a74933-9a79-4da3-af46-e314b7fb5f5a', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|4th container|Psoralen-treated', N'PLT AFR PAS-E LUKOPR Psoraln', N'E024', N'@D5 D5 H6 K5', 200, N'E7020', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'05de88ef-6e8a-4922-97b0-6c3dc78371bb', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part E or 5)', N'PLASMA AFR Thaw None IRD ', N'E013', N'@AS C2 H9', 225, N'49615', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'060411cb-e09a-43d3-91c6-c6941c7640c3', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part C or 3)', N'PLASMA AFR Thaw None IRD ', N'E013', N'@AS C2 H5', 225, N'49613', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0664e43c-0d49-4426-9a4a-de70775ea57f', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|1st container', N'PLT AFR PAS-E LUKOPR', N'E024', N'@D5 D5 H3', 200, N'E6736', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'06ec20d3-f381-4919-b5cb-dedb03ee9e8d', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 3 donors', N'PLT Pool CPD OPN IRD LUKOPR BUFCT 3DNR', N'E022', N'@A7 B2 C2 D5 G2 J3', 200, N'E6917', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'070ff18c-3396-4dc2-bb5f-a98c982ab5da', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|2nd container|Bacterial test', N'PLT AFR PAS-E LUKOPR BacTest', N'E024', N'@D5 D5 H4 N3', 200, N'E6753', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0720c6de-c18f-4aed-baaf-65c4d6f07459', N'POOLED PLATELETS|PAS-E/XX/20-24C|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-E BUFCT 8DNR', N'E022', N'@D5 G2 J8', 200, N'E6857', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0737eccd-082b-45ca-b2c5-c6a7cd334a84', N'Washed POOLED PLATELETS|None/XX/20-24C|Irradiated|Buffy coat plts prep|From 5 donors', N'PLT Pool Wash None IRD BUFCT 5DNR', N'E023', N'@71 C2 G2 J5', 200, N'E6729', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0787f51f-bbce-4037-9eea-092a6608f1ac', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-E IRD SR/PA BUFCT 5DNR', N'E022', N'@D5 C2 E8 G2 J5', 200, N'E6933', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'086c3135-bd32-43b5-a886-c1e77526b1c1', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part D or 4)', N'PLASMA AFR None IRD ', N'E012', N'@84 C2 H6', 225, N'49614', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'09405256-2fee-4f13-b748-e890123c15e2', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-C IRD SR/PA BUFCT 2DNR', N'E022', N'@B5 C2 E8 G2 J2', 200, N'E6712', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'09d4bde6-4fba-4fce-9408-9f9d15dbc2fb', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-C IRD SR/PA BUFCT 3DNR', N'E022', N'@B5 C2 E8 G2 J3', 200, N'E6713', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'09f6571e-8ead-4696-9a11-d7692fe8df3c', N'FRESH FROZEN PLASMA|CPD/XX/<-25C|For mnf:injectable|Frozen <=24h|Quar:>=112d/retested', N'FFP CPD MNFI QUAR', N'E010', N'@A1 A2 G7 I4', 225, N'E6820', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0a6f44ac-9137-44bf-be46-92c25006a0e7', N'Frozen Apheresis PLATELETS|DMSO/XX/<-65C|Open|Irradiated|ResLeu:<5log6|1st container', N'PLT AFR Froz DMSO OPN IRD LUKOPR', N'E025', N'@76 B2 C2 D4 H3', 200, N'E7014', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0a7362b6-742c-41d0-aff2-3e16aed2e7b3', N'Liquid Apheresis PLASMA|ACD-A/XX/<37C|For mnf:noninjectable', N'PLASMA AFR Liq ACD-A MNFNI', N'E011', N'@59 A3', 225, N'E6869', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0ac90e4a-e917-4511-9f49-9ea38d3b7490', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Supernat rem/Plasma added|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-E OPN SR/PA BUFCT 8DNR', N'E022', N'@D5 B2 E8 G2 J8', 200, N'E6950', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0aeb19d2-0086-42c3-a4bc-343846361fa1', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-C IRD LUKOPR SR/PA BUFCT 4DNR', N'E022', N'@B5 C2 D5 E8 G2 J4', 200, N'E6721', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0b0c664d-df1b-4436-9a57-e89656b5bf86', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|For mnf:injectable|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool Froz DMSO MNFI LUKOPR BUFCT 6DNR', N'E025', N'@CP A2 D5 G2 J6', 200, N'E7053', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0cdc6f2f-73f1-486a-baa1-426bba659b1b', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-E OPN LUKOPR BUFCT 8DNR', N'E022', N'@D5 B2 D5 G2 J8', 200, N'E6978', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0cfa6ddf-59ae-4a94-9789-63647e6728ee', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-C IRD BUFCT 8DNR', N'E022', N'@B5 C2 G2 J8', 200, N'E6704', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0d5e2a34-4ea0-4fe9-9e81-990e117d5fb2', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced|2nd container', N'PLT AFR PAS-E IRD LUKOPR SUPRed', N'E024', N'@D5 C2 D5 EA H4', 200, N'E6765', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0e8c2036-3fe3-4664-8c80-9f4c10af5358', N'POOLED PLATELETS|CP2D/XX/20-24C|ResLeu:<5log6|From 4 donors|Bacterial monitoring', N'PLT Pool CP2D LUKOPR 4DNR BacMtr', N'E022', N'@B9 D4 J4 N2', 200, N'E7149', 120, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'0fdfd7fc-6dff-4f9b-9799-1d685459b2bb', N'Washed RED BLOOD CELLS|SAGM/XX/refg|ResLeu:<5log6', N'RBC Wash SAGM LUKOPR', N'E003', N'@A9 D4', 200, N'E6848', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'100b1f53-1bb4-4ac5-92f7-17a1d08fd7a8', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|4th container|Bacterial monitoring', N'PLT AFR PAS-E LUKOPR BacMtr', N'E024', N'@D5 D5 H6 N2', 200, N'E6747', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1231294f-f282-419d-be3a-b8dc2cb3f2e0', N'POOLED PLATELETS|CPD/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 3 donors|Riboflavin-treated', N'PLT Pool CPD LUKOPR BUFCT 3DNR Ribo', N'E022', N'@A7 D5 G2 J3 K6', 200, N'E6685', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'129dac10-5e62-45a8-b66b-54b14b89f451', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|4th container|Bacterial monitoring', N'PLT AFR PAS-E IRD LUKOPR BacMtr', N'E024', N'@D5 C2 D5 H6 N2', 200, N'E6751', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'12d83109-60de-4782-a2d1-4ebb0c48974f', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|4th container', N'PLT AFR PAS-E LUKOPR', N'E024', N'@D5 D5 H6', 200, N'E6739', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1423512e-993b-44b8-90e6-2f2db0f9f22f', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Supernat rem/Plasma added|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-E OPN SR/PA BUFCT 3DNR', N'E022', N'@D5 B2 E8 G2 J3', 200, N'E6945', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1530131f-28a6-4f6a-83d8-889347d645b8', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED (maybe used for second bag of double or triple plateletpheresis)(variable contents:<3 X log11 plts)', N'PLT AFR PAS-C IRD LUKOPR', N'E024', N'@B5 C2 D2 H4 JI', 200, N'12951', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'153749db-e2d2-45d8-b69f-ef2c0d1ff85b', N'Thawed CRYOPRECIPITATE|CPD/450mL/rt|ResLeu:<1log6,WB filtr', N'CRYO Thaw CPD LUKOPR', N'E029', N'@85 DA', 15, N'E6784', 6, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'15d92428-a035-43f5-b934-6031f6e1d94c', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|Irradiated|<3log11 plts', N'PLT AFR ACD-A>PAS-C IRD', N'E024', N'@BH C2 JI', 200, N'E7010', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'15efec38-f1f6-4a18-bd32-69be1b83bdf5', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 7 donors', N'PLT Pool CPD OPN IRD LUKOPR BUFCT 7DNR', N'E022', N'@A7 B2 C2 D5 G2 J7', 200, N'E6921', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'172be15d-423b-40e4-949f-52f5ce77e290', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|1st container|Bacterial test', N'PLT AFR PAS-E LUKOPR BacTest', N'E024', N'@D5 D5 H3 N3', 200, N'E6752', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'17b57ef9-9c29-40f2-87bc-3b6ec543898b', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 8 donors', N'PLT Pool CPD OPN LUKOPR BUFCT 8DNR', N'E022', N'@A7 B2 D5 G2 J8', 200, N'E6894', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'180279f3-cd82-4007-af97-282a07de3245', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool CPD OPN LUKOPR BUFCT 5DNR', N'E022', N'@A7 B2 D5 G2 J5', 200, N'E6891', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'18c6a4bb-c794-4c5c-8e9a-2281e177a525', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|From 6 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR 6DNR', N'E026', N'@77 B2 C2 D5 J6', 200, N'E7126', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'18eadc43-64a2-47c3-bcbf-c9ab8b2296fb', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|Buffy coat plts prep|From 2 donors', N'PLT Pool CPD OPN LUKOPR PLS Red/PA BUFCT 2DNR', N'E022', N'@A7 B2 D5 EE G2 J2', 200, N'E6909', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1918a394-9558-4332-92c8-baa055d20605', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Plasma reduced/Plasma added|From 5 donors', N'PLT Pool Thaw DMSO OPN PLS Red/PA 5DNR', N'E026', N'@77 B2 EE J5', 200, N'E7130', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'191bdf33-520c-41fa-8112-f75230b1010a', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part E or 5)', N'PLASMA AFR None IRD ', N'E012', N'@84 C2 H9', 225, N'49615', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1934d668-7562-4b01-ac02-dbe0ab1f562b', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-E OPN LUKOPR BUFCT 7DNR', N'E022', N'@D5 B2 D5 G2 J7', 200, N'E6977', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'19db2d42-b44b-4e56-9bf2-5e9b61631c7e', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-E OPN IRD LUKOPR BUFCT 2DNR', N'E022', N'@D5 B2 C2 D5 G2 J2', 200, N'E6979', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'19dc8bff-27b5-4307-9973-775379e443d8', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|Buffy coat plts prep|From 5 donors', N'PLT Pool Thaw DMSO OPN IRD BUFCT 5DNR', N'E026', N'@77 B2 C2 G2 J5', 200, N'E7085', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'19f5a7dc-eb59-4238-9cda-ed06618cc93a', N'Washed RED BLOOD CELLS|SAGM/XX/refg|Irradiated|ResLeu:<5log6', N'RBC Wash SAGM IRD LUKOPR', N'E003', N'@A9 C2 D4', 200, N'E6849', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1a11fa21-498c-4c89-b291-c98044bf52e1', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 5 donors|Riboflavin-treated', N'PLT Pool CPD IRD LUKOPR BUFCT 5DNR Ribo', N'E022', N'@A7 C2 D5 G2 J5 K6', 200, N'E6694', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1aa385d1-8bbf-4c32-847e-ce5d256d215c', N'Apheresis RED BLOOD CELLS|CP2D>AS3/XX/refg|Open|Irradiated|ResLeu:<5log6|Plasma added|2nd container', N'RBC AFR CP2D>AS3 OPN IRD LUKOPR PA', N'E009', N'@33 B2 C2 D4 E4 H4', 350, N'E6798', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1ad9a12a-cabb-46f0-8c49-c319b1d030a6', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 3 donors', N'PLT Pool CPD IRD PLS Red/PA BUFCT 3DNR', N'E022', N'@A7 C2 EE G2 J3', 200, N'E6671', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1b925a6c-7a71-4438-9a99-3b717d301651', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Plasma reduced/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool CPD OPN PLS Red/PA BUFCT 5DNR', N'E022', N'@A7 B2 EE G2 J5', 200, N'E6898', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1bb4a9c3-118f-4d48-a020-78191277ca01', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-C IRD LUKOPR SR/PA BUFCT 8DNR', N'E022', N'@B5 C2 D5 E8 G2 J8', 200, N'E6725', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1c46d305-2ece-4ab3-b195-9ae1a604d5a5', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|From 7 donors', N'PLT Pool Thaw DMSO OPN LUKOPR 7DNR', N'E026', N'@77 B2 D5 J7', 200, N'E7122', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1ca44259-a088-4be1-95e1-12794214065a', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|3rd container', N'PLT AFR Wash PAS-E LUKOPR', N'E027', N'@D5 D5 H5', 200, N'E6770', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1d37bd16-083c-4ec6-9089-7405be1b783b', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6', N'PLT AFR PAS-E IRD LUKOPR', N'E024', N'@D5 C2 D5', 200, N'E6875', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1d71ac20-8946-42f0-88ae-904b586b5b28', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED (maybe used for second bag of double or triple plateletpheresis)(variable contents:<3 X log11 plts)', N'PLT AFR Wash None IRD LUKOPR', N'E027', N'@71 B2 C2 D2 H4 JI', 200, N'12951', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1d875474-077f-441d-a262-0658820a6503', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|Irradiated|3rd container', N'PLT AFR ACD-A>PAS-C IRD', N'E024', N'@BH C2 H5', 200, N'E7000', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1dc60a8e-3b55-48a2-bbb6-c1062172c2bf', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-E OPN IRD LUKOPR BUFCT 5DNR', N'E022', N'@D5 B2 C2 D5 G2 J5', 200, N'E6982', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1f0107ba-0e15-4f32-8bda-f2a76a20514d', N'Washed Apheresis PLATELETS|None/XX/20-24C|ResLeu:<1log6', N'PLT AFR Wash None LUKOPR', N'E027', N'@71 D5', 200, N'E6666', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1f033121-3109-4d28-8af6-446536936a13', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 3 donors|Riboflavin-treated', N'PLT Pool CPD IRD LUKOPR BUFCT 3DNR Ribo', N'E022', N'@A7 C2 D5 G2 J3 K6', 200, N'E6692', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'1f315270-dc7e-42e5-9048-254872fe8a6c', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part A or 1)', N'PLASMA AFR None', N'E012', N'@84 H3', 225, N'49211', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2039ca59-7d53-4990-a1fa-386107017f19', N'Apheresis PLATELETS|ACD-A/XX/20-24C|Open|ResLeu:<5log6|1st container|<3log11 plts', N'PLT AFR ACD-A OPN LUKOPR', N'E024', N'@73 B2 D4 H3 JI', 200, N'E6843', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'204a1b13-d4a0-43ea-9390-7a403070c1c7', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part H or 8)', N'PLASMA AFR Thaw None IRD ', N'E013', N'@AS C2 HC', 225, N'49618', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'204f35f6-c692-4e6d-ba41-d06f4959b89c', N'Deglycerolized Apheresis RED BLOOD CELLS|AS3/XX/refg|Open|ResLeu:<5log6|1st container', N'RBC AFR Deg AS3 OPN LUKOPR', N'E051', N'@B6 B2 D4 H3', 200, N'E6653', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'20ba3260-c768-4fc5-8fd3-60e0b0e4b3ba', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Supernat reduced|3rd container', N'PLT AFR PAS-E LUKOPR SUPRed', N'E024', N'@D5 D5 EA H5', 200, N'E6762', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2127c349-7337-4c22-b8ac-908d9cd14b7a', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|For mnf:injectable|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool Froz DMSO MNFI LUKOPR BUFCT 4DNR', N'E025', N'@CP A2 D5 G2 J4', 200, N'E7051', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'224003c9-7010-49ee-ad57-ecd3dbdb6226', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container', N'PLT AFR PAS-E IRD LUKOPR', N'E024', N'@D5 C2 D5 H4', 200, N'E6741', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'224b7bf2-a80b-4a1b-854a-53d563972e66', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-E IRD SR/PA BUFCT 4DNR', N'E022', N'@D5 C2 E8 G2 J4', 200, N'E6932', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'22744629-c568-409c-ad5f-308f730aca75', N'Apheresis FRESH FROZEN PLASMA|ACD-A/XX/<=-18C|Open|>=200mL<400mL', N'FFP AFR ACD-A OPN LowVol', N'E012', N'@47 B2 F5', 225, N'E7050', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'227f2bcd-fc92-4915-9473-9262af7d262a', N'Frozen Apheresis PLATELETS|DMSO/XX/<-65C|Open|ResLeu:<5log6|3rd container', N'PLT AFR Froz DMSO OPN LUKOPR', N'E025', N'@76 B2 D4 H5', 200, N'E6868', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'237734ce-d3f6-4484-8e28-ea3b70ca8f95', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED (for third bag of triple plateletpheresis)', N'PLT AFR PAS-C IRD LUKOPR', N'E024', N'@B5 C2 D2 H5', 200, N'12960', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'252d239b-67cc-4dd6-afda-afa56b5be229', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool Froz DMSO LUKOPR BUFCT 5DNR', N'E025', N'@CP D5 G2 J5', 200, N'E7055', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'261a8b49-bb2a-4bf3-bdab-e483e0109c09', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|Irradiated|Buffy coat plts prep|From 5 donors', N'PLT Pool Wash None OPN IRD BUFCT 5DNR', N'E023', N'@71 B2 C2 G2 J5', 200, N'E6830', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'26cd846a-9384-4e0b-87fc-efb9f2486a43', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|Irradiated|Buffy coat plts prep|From 8 donors', N'PLT Pool Wash None OPN IRD BUFCT 8DNR', N'E023', N'@71 B2 C2 G2 J8', 200, N'E6833', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'289ea5dd-3623-430b-a819-b2a7658499dc', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part H or 8)', N'PLASMA AFR None IRD ', N'E012', N'@84 C2 HC', 225, N'49618', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'28c0b0a5-d1d4-4dcc-be39-c150389aa3b1', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|From 4 donors', N'PLT Pool Thaw DMSO OPN LUKOPR 4DNR', N'E026', N'@77 B2 D5 J4', 200, N'E7119', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'296af449-0b00-4d10-b53d-0fd4161e4e43', N'Thawed Apheresis PLASMA|ACD-A/XX/refg|Open|3rd container', N'PLASMA AFR Thaw ACD-A OPN', N'E013', N'@54 B2 H5', 225, N'E6801', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'29b1ac58-983b-4b9e-9c73-891d55d8caa5', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED (for third bag of triple plateletpheresis)(variable contents:<3 X log11 plts)', N'PLT AFR PAS-C LUKOPR', N'E024', N'@B5 D2 H5 JI', 200, N'12931', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2a56dae1-5353-476f-a250-f35bc176196f', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Buffy coat plts prep|From 2 donors|Riboflavin-treated', N'PLT Pool CPD IRD BUFCT 2DNR Ribo', N'E022', N'@A7 C2 G2 J2 K6', 200, N'E6677', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2ae03f2b-1d0e-4aee-bd6c-dba96235bd49', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Plasma reduced/Plasma added|From 6 donors', N'PLT Pool Thaw DMSO OPN PLS Red/PA 6DNR', N'E026', N'@77 B2 EE J6', 200, N'E7131', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2aef4a8b-86e6-4168-83e8-dc88f1fc9232', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-C IRD SR/PA BUFCT 6DNR', N'E022', N'@B5 C2 E8 G2 J6', 200, N'E6716', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2d8271ae-09c1-42ae-bb66-9486a7e3e205', N'POOLED PLATELETS|CPD/XX/20-24C|Buffy coat plts prep|From 6 donors', N'PLT Pool CPD BUFCT 6DNR', N'E022', N'@A7 G2 J6', 200, N'E6885', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2dc86982-aff8-47e5-930c-c4ff9cf7a6a1', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added', N'PLT AFR PAS-E IRD LUKOPR SR/PA', N'E024', N'@D5 C2 D5 E8', 200, N'E6880', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2e7e8ea5-7f36-40b3-831a-8046f353a4c5', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-C IRD LUKOPR BUFCT 3DNR', N'E022', N'@B5 C2 D5 G2 J3', 200, N'E6706', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2e950a12-921d-4f8d-8924-3c9a1dd0b0a3', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 8 donors|Riboflavin-treated', N'PLT Pool CPD IRD LUKOPR BUFCT 8DNR Ribo', N'E022', N'@A7 C2 D5 G2 J8 K6', 200, N'E6697', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2f960b81-bbda-4c96-9eb4-3701a0e368d8', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part G or 7)', N'PLASMA AFR Thaw None', N'E013', N'@AS HB', 225, N'49217', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2f985a58-6c1e-4972-8242-a24c7166dc67', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|ResLeu:<1log6|From 4 donors', N'PLT Pool Froz DMSO OPN LUKOPR 4DNR', N'E025', N'@CP B2 D5 J4', 200, N'E7104', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'2fae562d-cd45-4802-a244-a53949f0c5d4', N'POOLED PLATELETS|CPD/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 7 donors|Riboflavin-treated', N'PLT Pool CPD LUKOPR BUFCT 7DNR Ribo', N'E022', N'@A7 D5 G2 J7 K6', 200, N'E6689', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3035f499-0c70-4f19-81fb-8e128ae955aa', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|1st container', N'PLT AFR Wash PAS-E LUKOPR', N'E027', N'@D5 D5 H3', 200, N'E6768', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'30e04e9f-6d8f-4ec6-9dfd-b84e80d33e6a', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-E OPN LUKOPR SR/PA BUFCT 6DNR', N'E022', N'@D5 B2 D5 E8 G2 J6', 200, N'E6990', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'31045a45-e563-470a-86c3-f228cf569374', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|<3log11 plts', N'PLT AFR ACD-A>PAS-C', N'E024', N'@BH JI', 200, N'E7009', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'311fddd8-c742-403a-be0c-a21fd008a896', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part B or 2)', N'PLASMA AFR None IRD ', N'E012', N'@84 C2 H4', 225, N'49612', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3193e4ff-d96f-40ee-96b4-108e82921bf1', N'POOLED PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-D IRD LUKOPR BUFCT 6DNR', N'E022', N'@CI C2 D5 G2 J6', 200, N'E6669', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'32036c29-d854-4c6f-8b00-6064716fe9f6', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 2 donors', N'PLT Pool CPD OPN IRD LUKOPR BUFCT 2DNR', N'E022', N'@A7 B2 C2 D5 G2 J2', 200, N'E6916', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'324be73d-3d5b-4ce3-b056-83603c3e16f1', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|From 4 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR 4DNR', N'E026', N'@77 B2 C2 D5 J4', 200, N'E7124', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'32adf882-6ad3-414a-96e5-d0786978da98', N'Washed RED BLOOD CELLS|AS3/500mL/refg|Irradiated|ResLeu:<5log6', N'RBC Wash AS3 IRD LUKOPR', N'E003', N'@D7 C2 D4', 200, N'E6814', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'32e429e1-bd08-4f87-91a7-935d663e7fcc', N'RED BLOOD CELLS|CPD>AS5/450mL/refg|Open|ResLeu:<5log6|Supernat rem/Plasma added', N'RBC CPD>AS5 OPN LUKOPR SR/PA', N'E002', N'@22 B2 D4 E8', 350, N'E7034', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'32f49251-c8bf-4011-adbd-ca3455c4f7a8', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Buffy coat plts prep|From 7 donors|Riboflavin-treated', N'PLT Pool CPD IRD BUFCT 7DNR Ribo', N'E022', N'@A7 C2 G2 J7 K6', 200, N'E6682', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'347f3a92-97e2-4edb-844c-fd01a55fb17d', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|Irradiated|Buffy coat plts prep|From 7 donors', N'PLT Pool Wash None OPN IRD BUFCT 7DNR', N'E023', N'@71 B2 C2 G2 J7', 200, N'E6832', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'348943d8-9c9d-445e-9a7a-78205f466a4d', N'Deglycerolized Apheresis RED BLOOD CELLS|AS3/XX/refg|Open|Irradiated|ResLeu:<5log6|2nd container', N'RBC AFR Deg AS3 OPN IRD LUKOPR', N'E051', N'@B6 B2 C2 D4 H4', 200, N'E6656', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'35229bcf-f33a-493b-a311-e5364d223694', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 7 donors', N'PLT Pool Wash None OPN LUKOPR BUFCT 7DNR', N'E023', N'@71 B2 D5 G2 J7', 200, N'E6839', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3570cb0c-6600-4adf-92c9-638a6ca8ca45', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|From 6 donors', N'PLT Pool Thaw DMSO OPN LUKOPR PLS Red/PA 6DNR', N'E026', N'@77 B2 D5 EE J6', 200, N'E7141', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'35898cd8-4a3c-48a9-9db8-a370e7e355ab', N'Washed POOLED PLATELETS|None/XX/20-24C|Irradiated|Buffy coat plts prep|From 4 donors', N'PLT Pool Wash None IRD BUFCT 4DNR', N'E023', N'@71 C2 G2 J4', 200, N'E6728', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'364d22c6-9fb9-4a06-ab55-a9950566d86d', N'Washed Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool Thaw Wash DMSO OPN LUKOPR BUFCT 4DNR', N'E027', N'@77 B2 D5 G2 J4', 200, N'E7160', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'36a9651d-67ce-459c-830b-b4174373de5a', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool CPD OPN IRD LUKOPR BUFCT 5DNR', N'E022', N'@A7 B2 C2 D5 G2 J5', 200, N'E6919', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'36b1446a-eaef-46a2-99f1-bc0d884a3673', N'POOLED PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|Bacterial test', N'PLT Pool PAS-E LUKOPR BUFCT BacTest', N'E022', N'@D5 D5 G2 N3', 200, N'E6778', 120, 0, 0, 1, NULL, N'A', '20101201 15:05:41.733', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'377b6c97-4337-4913-83d3-667a861385ef', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|From 7 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR 7DNR', N'E026', N'@77 B2 C2 D5 J7', 200, N'E7127', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'37e6d83f-80d5-4c63-b96d-07489bbaeff3', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part A or 1)', N'PLASMA AFR None IRD ', N'E012', N'@84 C2 H3', 225, N'49611', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3918edad-c188-48c7-8e52-c41a4fdd7406', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR BUFCT 5DNR', N'E026', N'@77 B2 C2 D5 G2 J5', 200, N'E7089', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3a518b9d-abaa-4ac6-ba01-9c71e419e92e', N'Washed Apheresis PLATELETS|ACD-A/XX/20-24C|ResLeu:<1log6|Riboflavin-treated', N'PLT AFR Wash ACD-A LUKOPR Ribo', N'E027', N'@73 D5 K6', 200, N'E7061', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3aef9538-e817-45a8-94e0-29f5ef49c6bb', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-E OPN IRD LUKOPR BUFCT 4DNR', N'E022', N'@D5 B2 C2 D5 G2 J4', 200, N'E6981', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3b11a5a4-a165-4e34-9754-5b15ef02d54a', N'POOLED PLATELETS|CPD/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 8 donors|Riboflavin-treated', N'PLT Pool CPD LUKOPR BUFCT 8DNR Ribo', N'E022', N'@A7 D5 G2 J8 K6', 200, N'E6690', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3b1927c4-5b38-4a7f-b673-f5d25e7054b5', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|From 7 donors', N'PLT Pool Thaw DMSO OPN LUKOPR PLS Red/PA 7DNR', N'E026', N'@77 B2 D5 EE J7', 200, N'E7142', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3b99a627-e85f-4897-9c7f-419a91f82de5', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-E IRD LUKOPR SR/PA BUFCT 7DNR', N'E022', N'@D5 C2 D5 E8 G2 J7', 200, N'E6963', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3c82e58f-b79e-4178-b969-795080a98814', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 3 donors', N'PLT Pool CPD OPN LUKOPR BUFCT 3DNR', N'E022', N'@A7 B2 D5 G2 J3', 200, N'E6889', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3cf222e7-5f90-467b-9a9a-82208e240427', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|4th container|Psoralen-treated', N'PLT AFR Wash PAS-E LUKOPR Psoraln', N'E027', N'@D5 D5 H6 K5', 200, N'E7028', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3cf30980-9141-4eff-b83d-f8848e6d1475', N'PLASMA|CPD/450mL/<=-18C|Cryo reduced', N'PLASMA CPD CRYORed', N'E010', N'@AM E3', 225, N'E6659', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3e3acdd0-0c88-4431-91c9-87afe454bcd3', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-E IRD BUFCT 3DNR', N'E022', N'@D5 C2 G2 J3', 200, N'E6924', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3ed31488-17a2-4cc7-bdbb-14da3f3cdd30', N'POOLED PLATELETS|PAS-E/XX/20-24C|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-E BUFCT 3DNR', N'E022', N'@D5 G2 J3', 200, N'E6852', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3f6665f3-a044-41af-98f0-a4e3b5d836bd', N'RED BLOOD CELLS|SAGM/450mL/refg|Irradiated|ResLeu:<5log6|Plasma added', N'RBC SAGM IRD LUKOPR PA', N'E002', N'@24 C2 D4 E4', 350, N'E7044', 672, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'3f92d441-b864-4c3d-8829-861fbefec54b', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|2nd container', N'PLT AFR Wash PAS-E LUKOPR', N'E027', N'@D5 D5 H4', 200, N'E6769', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'400c3f4e-21cb-47ab-bddf-2affc2077ff5', N'Apheresis PLATELETS|ACD-A/XX/20-24C|Open|ResLeu:<5log6|3rd container|<3log11 plts', N'PLT AFR ACD-A OPN LUKOPR', N'E024', N'@73 B2 D4 H5 JI', 200, N'E6845', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'420d4302-47d1-484b-a4dc-00b409e20b9d', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|ResLeu:<5log6', N'PLT AFR ACD-A>PAS-C LUKOPR', N'E024', N'@BH D4', 200, N'E7001', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'422484ef-7769-4859-9807-6c6ceed2ea07', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-E OPN IRD SR/PA BUFCT 3DNR', N'E022', N'@D5 B2 C2 E8 G2 J3', 200, N'E6938', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'42aa818c-0698-401f-b13a-59bc6e0e318a', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|1st container', N'PLT AFR ACD-A>PAS-C', N'E024', N'@BH H3', 200, N'E6994', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'42b87355-32f0-474a-bc68-aae549f13f29', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-E OPN IRD SR/PA BUFCT 6DNR', N'E022', N'@D5 B2 C2 E8 G2 J6', 200, N'E6941', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'438e1883-7265-4752-a8c9-6529ba6c1c96', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED (for third bag of triple plateletpheresis)(variable contents:<3 X log11 plts)', N'PLT AFR Wash None OPN LUKOPR', N'E027', N'@71 B2 D2 H5 JI', 200, N'12931', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'43abb1e8-6a9b-4291-a9ec-b744f02f5223', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|Irradiated|Buffy coat plts prep|From 2 donors', N'PLT Pool Wash None OPN IRD BUFCT 2DNR', N'E023', N'@71 B2 C2 G2 J2', 200, N'E6827', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'43efdab1-708f-4227-96ce-eb9de824c1fd', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|4th container|Bacterial test', N'PLT AFR PAS-E IRD LUKOPR BacTest', N'E024', N'@D5 C2 D5 H6 N3', 200, N'E6759', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4447e7e3-5629-4657-90e7-c9bf8d22f330', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|ResLeu:<5log6|1st container', N'PLT AFR ACD-A>PAS-C LUKOPR', N'E024', N'@BH D4 H3', 200, N'E7002', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'44ee6709-367c-4f59-bf2d-a6cfe06e1ebb', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 7 donors', N'PLT Pool CPD OPN IRD PLS Red/PA BUFCT 7DNR', N'E022', N'@A7 B2 C2 EE G2 J7', 200, N'E6907', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'45a53429-3b01-41b1-b15c-0c165c6600de', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 5 donors|Riboflavin-treated', N'PLT Pool Thaw DMSO OPN LUKOPR BUFCT 5DNR Ribo', N'E026', N'@77 B2 D5 G2 J5 K6', 200, N'E7069', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'45fb993d-5fb5-4948-8f33-02808a5ff057', N'Apheresis FRESH FROZEN PLASMA|ACD-A/XX/<=-18C|Open|<200 mL', N'FFP AFR ACD-A OPN LowVol', N'E012', N'@47 B2 F4', 225, N'E7049', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'46d4fcb5-dc3f-4058-bcd4-45c8e920b89a', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|Irradiated|Buffy coat plts prep|From 6 donors', N'PLT Pool Wash None OPN IRD BUFCT 6DNR', N'E023', N'@71 B2 C2 G2 J6', 200, N'E6831', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'46face77-1e2b-4557-8319-bb9e246ce81b', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-E IRD BUFCT 8DNR', N'E022', N'@D5 C2 G2 J8', 200, N'E6929', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'474a90c8-5945-4317-9678-bea7f600e0ff', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container|Bacterial test', N'PLT AFR PAS-E IRD LUKOPR BacTest', N'E024', N'@D5 C2 D5 H4 N3', 200, N'E6757', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'47c51794-efd9-4539-b0c1-806e5747d26f', N'Thawed POOLED CRYOPRECIPITATE|CPD/XX/rt|Open|ResLeu:NS', N'CRYO Pool Thaw CPD OPN LUKOPR', N'E031', N'@BT B2 D2', 100, N'E6816', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'487a845f-dd24-4aae-817a-c95d006fcef7', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Plasma reduced/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool CPD OPN PLS Red/PA BUFCT 4DNR', N'E022', N'@A7 B2 EE G2 J4', 200, N'E6897', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'48e8f67d-8dc6-4ebc-8d23-cc6fc8d72d51', N'Washed Apheresis PLATELETS|ACD-A/XX/20-24C|Open|ResLeu:<1log6|Riboflavin-treated', N'PLT AFR Wash ACD-A OPN LUKOPR Ribo', N'E027', N'@73 B2 D5 K6', 200, N'E7062', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'49419d70-a25e-4d6c-8735-1590f7006a3c', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|Buffy coat plts prep|From 6 donors', N'PLT Pool CPD OPN LUKOPR PLS Red/PA BUFCT 6DNR', N'E022', N'@A7 B2 D5 EE G2 J6', 200, N'E6913', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'49635356-4200-4990-8ee4-cb90d78370d3', N'Apheresis FRESH FROZEN PLASMA|NaCitrate/XX/<-30C|ResLeu:<5log6|Riboflavin-treated', N'FFP AFR NaCit LUKOPR Ribo', N'E012', N'@CH D4 K6', 225, N'E7040', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4996696d-8ed8-4f93-9608-59520b09ad81', N'Apheresis FRESH FROZEN PLASMA|ACD-A/XX/<=-18C|Open|>=600mL', N'FFP AFR ACD-A OPN LowVol', N'E012', N'@47 B2 F7', 225, N'E7048', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4a6cc8c8-99e9-497f-bd3e-cf8aef4a03ac', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Supernat rem/Plasma added', N'PLT AFR PAS-E LUKOPR SR/PA', N'E024', N'@D5 D5 E8', 200, N'E6878', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4ab5a83d-f5af-43a4-bb64-76bbe99660da', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|4th container', N'PLT AFR Wash PAS-E LUKOPR', N'E027', N'@D5 D5 H6', 200, N'E6771', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4ac0c333-c596-4c7a-b5cc-a54e4639a882', N'Apheresis PLATELETS|ACD-A/XX/20-24C|For mnf:noninjectable|ResLeu:<5log6', N'PLT AFR ACD-A MNFNI LUKOPR', N'E024', N'@73 A3 D4', 200, N'E6649', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4acf0b28-efd6-4a3c-821e-12ac42fa9495', N'Apheresis FRESH FROZEN PLASMA|ACD-A/XX/<-25C|For mnf:injectable|Frozen <=24h|Quar:>=112d/retested', N'FFP AFR ACD-A MNFI QUAR', N'E012', N'@AI A2 G7 I4', 225, N'E6819', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4c0b9d34-8e39-435b-a957-bb8bd23aa245', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-C IRD BUFCT 5DNR', N'E022', N'@B5 C2 G2 J5', 200, N'E6701', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4c506557-7ff3-4d1a-a65b-d6bf6bbe4fce', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Plasma reduced/Plasma added|Buffy coat plts prep|From 8 donors', N'PLT Pool CPD OPN PLS Red/PA BUFCT 8DNR', N'E022', N'@A7 B2 EE G2 J8', 200, N'E6901', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4cd8892e-457a-4529-8824-c3fefbc31441', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|ResLeu:<5log6|3rd container', N'PLT AFR ACD-A>PAS-C LUKOPR', N'E024', N'@BH D4 H5', 200, N'E7004', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4d433b80-6964-43b8-b1b6-e05472e9ea38', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|4th container', N'PLT AFR Wash PAS-E IRD LUKOPR', N'E027', N'@D5 C2 D5 H6', 200, N'E6775', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4db5e1e0-7aba-462f-a56e-4494c8bfcd65', N'Washed POOLED PLATELETS|None/XX/20-24C|Irradiated|Buffy coat plts prep|From 3 donors', N'PLT Pool Wash None IRD BUFCT 3DNR', N'E023', N'@71 C2 G2 J3', 200, N'E6727', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4dcf35bd-72ac-405b-b8db-626d55ec31a5', N'Apheresis RED BLOOD CELLS|ACD-A>AS3/XX/refg|Open|Irradiated|ResLeu:<5log6|Plasma added|2nd container', N'RBC AFR ACD-A>AS3 OPN IRD LUKOPR PA', N'E009', N'@AZ B2 C2 D4 E4 H4', 350, N'E6792', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4dfe247f-b0fe-44f9-8fa9-db786138dcd0', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|From 5 donors', N'PLT Pool Thaw DMSO OPN LUKOPR PLS Red/PA 5DNR', N'E026', N'@77 B2 D5 EE J5', 200, N'E7140', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4e3c9ba8-6a73-4dda-a7ef-a3eddff77348', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part F or 6)', N'PLASMA AFR None IRD ', N'E012', N'@84 C2 HA', 225, N'49616', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4eeb8dad-f2bd-42cb-b25a-a9f7fe18b464', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool CPD OPN IRD LUKOPR BUFCT 4DNR', N'E022', N'@A7 B2 C2 D5 G2 J4', 200, N'E6918', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'4ef30369-9505-40db-abf0-fd156e06b2e1', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|Buffy coat plts prep|From 8 donors', N'PLT Pool CPD OPN LUKOPR PLS Red/PA BUFCT 8DNR', N'E022', N'@A7 B2 D5 EE G2 J8', 200, N'E6915', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'50b4a9d6-2394-4a05-b4ef-438e6e8188dc', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part F or 6)', N'PLASMA AFR Thaw None', N'E013', N'@AS HA', 225, N'49216', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'50cbfbc9-d6b5-4a04-9e73-7ba1b88f25fc', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Plasma reduced/Plasma added|From 7 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR PLS Red/PA 7DNR', N'E026', N'@77 B2 C2 D5 EE J7', 200, N'E7147', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'50fdd78a-0574-498c-ab12-56007ff01f54', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|From 3 donors', N'PLT Pool Thaw DMSO OPN 3DNR', N'E026', N'@77 B2 J3', 200, N'E7108', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'517abf09-2038-465a-b3fd-0593aa8b4e9e', N'PLASMA|CPDA-1/XX/<=-18C|For mnf:noninjectable restr use|Cryo reduced', N'PLASMA CPDA-1 MNFRNI CRYORed', N'E010', N'@36 A5 E3', 225, N'E7029', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5204f5bd-7eaf-43e9-a78c-154b46cb2af7', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|ResLeu:<1log6|From 5 donors', N'PLT Pool Froz DMSO OPN LUKOPR 5DNR', N'E025', N'@CP B2 D5 J5', 200, N'E7105', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'52b8fdf6-a5c0-4490-b309-2e62415e78e2', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|ResLeu:<5log6|<3log11 plts', N'PLT AFR ACD-A>PAS-C LUKOPR', N'E024', N'@BH D4 JI', 200, N'E7011', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'53247182-30e5-4521-a32a-32acae544342', N'Liquid PLASMA|CP2D/XX/rt|Not for tx or mnf', N'PLASMA Liq CP2D NMT', N'E011', N'@CU A4', 225, N'E6871', 624, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'53717aeb-ef39-4e31-a813-33f8f14c2287', N'RED BLOOD CELLS|SAGM/350mL/refg|Irradiated', N'RBC SAGM IRD', N'E002', N'@D4 C2', 350, N'E7166', 672, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'53e557a5-6cd5-4894-bb57-14b28de56c4c', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|From 7 donors', N'PLT Pool Thaw DMSO OPN IRD PLS Red/PA 7DNR', N'E026', N'@77 B2 C2 EE J7', 200, N'E7137', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'53e8a9c8-c1ca-4d75-81c1-af850b94e900', N'POOLED PLATELETS|CP2D/XX/20-24C|ResLeu:<5log6|From 6 donors|Bacterial monitoring', N'PLT Pool CP2D LUKOPR 6DNR BacMtr', N'E022', N'@B9 D4 J6 N2', 200, N'E7151', 120, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5430f17d-03c2-4f4f-8a35-46febeba495d', N'Washed POOLED PLATELETS|None/XX/20-24C|Irradiated|Buffy coat plts prep|From 2 donors', N'PLT Pool Wash None IRD BUFCT 2DNR', N'E023', N'@71 C2 G2 J2', 200, N'E6726', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'543b9e1e-e4f1-478e-8eda-ec9fa42a221f', N'PLASMA|CPDA-1/450mL/<=-18C|Frozen >24h', N'PLASMA CPDA-1', N'E010', N'@BK G8', 225, N'E6824', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'544edaa8-d549-420b-94ab-526c91342d13', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED (maybe used for second bag of double or triple plateletpheresis)', N'PLT AFR Wash None IRD LUKOPR', N'E027', N'@71 B2 C2 D2 H4', 200, N'12950', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'54720c3f-d8db-4960-9560-c99be9428687', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Supernat rem/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-E OPN SR/PA BUFCT 4DNR', N'E022', N'@D5 B2 E8 G2 J4', 200, N'E6946', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'54759437-b600-452e-ba58-a34b891ab07d', N'Washed POOLED PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep', N'PLT Pool Wash PAS-E LUKOPR BUFCT', N'E023', N'@D5 D5 G2', 200, N'E6782', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5489d11d-78dd-4b84-b1b3-f38a4db6768c', N'POOLED PLATELETS|PAS-E/XX/20-24C|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-E BUFCT 4DNR', N'E022', N'@D5 G2 J4', 200, N'E6853', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'54bd378f-079f-4024-8ae5-fec06f06dded', N'FRESH FROZEN PLASMA|CPD/450mL/<-25C|Frozen <=15h|Methylene blue-treated', N'FFP CPD Methyl Blue', N'E010', N'@A2 G5 K3', 225, N'E6822', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'55058223-e9c4-40dd-94c2-6784d2eeb642', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-C IRD LUKOPR BUFCT 8DNR', N'E022', N'@B5 C2 D5 G2 J8', 200, N'E6711', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'552ca0b0-cc4b-44a5-9d8e-de37a4f5b456', N'CRYOPRECIPITATE|CPDA-1/450mL/<-25C|Riboflavin-treated', N'CRYO CPDA-1 Ribo', N'E028', N'@CV K6', 15, N'E7073', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'56046a28-3b80-40ed-bdd0-e832967e6f7f', N'POOLED PLATELETS|CP2D/XX/20-24C|ResLeu:<5log6|From 7 donors|Bacterial monitoring', N'PLT Pool CP2D LUKOPR 7DNR BacMtr', N'E022', N'@B9 D4 J7 N2', 200, N'E7152', 120, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5619ecb1-4769-4584-96c9-4ce3dfa3f894', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-E IRD LUKOPR SR/PA BUFCT 6DNR', N'E022', N'@D5 C2 D5 E8 G2 J6', 200, N'E6962', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'572e1503-7e71-4cbc-808d-fbe7e1226913', N'Apheresis FRESH FROZEN PLASMA|NaCitrate/XX/<-30C|ResLeu:<5log6|50mL|Riboflavin-treated', N'FFP AFR NaCit LUKOPR Ribo', N'E012', N'@CH D4 FA K6', 225, N'E7041', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'58564a7b-2aed-4c6e-b07b-40ee9a420e34', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|ResLeu:<1log6|Buffy coat plts prep|From 5 donors|Riboflavin-treated', N'PLT Pool Froz DMSO OPN LUKOPR BUFCT 5DNR Ribo', N'E025', N'@CP B2 D5 G2 J5 K6', 200, N'E7067', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5866844b-cff6-4cd9-accc-a89c44f2ba6e', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-E IRD LUKOPR BUFCT 7DNR', N'E022', N'@D5 C2 D5 G2 J7', 200, N'E6956', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'590a7c75-6796-4145-bb4f-5542f4200cdc', N'Washed Thawed Apheresis PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6', N'PLT AFR Thaw Wash DMSO OPN LUKOPR', N'E027', N'@77 B2 D5', 200, N'E7164', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'59122650-7a89-4b00-8ea1-bc277a0a8014', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|From 5 donors', N'PLT Pool Thaw DMSO OPN LUKOPR 5DNR', N'E026', N'@77 B2 D5 J5', 200, N'E7120', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5915d868-c3dd-49f4-86b8-85f64e4010c5', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|From 3 donors', N'PLT Pool Thaw DMSO OPN IRD PLS Red/PA 3DNR', N'E026', N'@77 B2 C2 EE J3', 200, N'E7133', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5976c759-9257-4bda-a46f-8b109f7029ff', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 7 donors', N'PLT Pool CPD OPN LUKOPR BUFCT 7DNR', N'E022', N'@A7 B2 D5 G2 J7', 200, N'E6893', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'59e7f3ba-dd1e-42f2-8b95-22a30dec84b9', N'RED BLOOD CELLS|CPD>AS1/500mL/refg|Open|ResLeu:<5log6|Plasma added', N'RBC CPD>AS1 OPN LUKOPR PA', N'E002', N'@19 B2 D4 E4', 350, N'E6786', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5a0d91ff-bc4a-4d6f-ba3c-eee463a6a481', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|Bacterial test', N'PLT Pool PAS-E IRD LUKOPR BUFCT BacTest', N'E022', N'@D5 C2 D5 G2 N3', 200, N'E6779', 120, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5b608937-6a38-4f29-a9f7-8c3dd391d499', N'Frozen Apheresis PLATELETS|None/XX/<=-18C|For mnf:noninjectable|ResLeu:<5log6|1st container', N'PLT AFR Froz None MNFNI LUKOPR', N'E025', N'@84 A3 D4 H3', 200, N'E6809', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5c01c36e-94fe-4922-8ac0-c147d3ebf460', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|Buffy coat plts prep|From 4 donors', N'PLT Pool Thaw DMSO OPN IRD BUFCT 4DNR', N'E026', N'@77 B2 C2 G2 J4', 200, N'E7084', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5c97a440-d98c-4a5c-a7cf-d1c22a34cab0', N'RED BLOOD CELLS|CPD>AS5/450mL/refg|Open|Irradiated|ResLeu:<5log6|Supernat rem/Plasma added', N'RBC CPD>AS5 OPN IRD LUKOPR SR/PA', N'E002', N'@22 B2 C2 D4 E8', 350, N'E7047', 24, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5cadbb64-d539-4a7c-8386-f97ab4a47020', N'Deglycerolized Apheresis RED BLOOD CELLS|AS3/XX/refg|Open|Irradiated|ResLeu:<5log6|1st container', N'RBC AFR Deg AS3 OPN IRD LUKOPR', N'E051', N'@B6 B2 C2 D4 H3', 200, N'E6655', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5ceac1b8-5dae-4bac-84fc-853791666506', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|Buffy coat plts prep|From 7 donors', N'PLT Pool CPD OPN LUKOPR PLS Red/PA BUFCT 7DNR', N'E022', N'@A7 B2 D5 EE G2 J7', 200, N'E6914', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5d14fcf4-7046-4895-9c9d-927144f4735e', N'POOLED PLATELETS|CPD/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 6 donors|Riboflavin-treated', N'PLT Pool CPD LUKOPR BUFCT 6DNR Ribo', N'E022', N'@A7 D5 G2 J6 K6', 200, N'E6688', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5d9ee142-9368-4c0c-9582-e5b4b06cea2b', N'Deglycerolized Apheresis RED BLOOD CELLS|AS3/XX/refg|Open|ResLeu:<5log6|2nd container', N'RBC AFR Deg AS3 OPN LUKOPR', N'E051', N'@B6 B2 D4 H4', 200, N'E6654', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5e34e3fa-3c13-4351-a984-2b9e7e64e0cb', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Plasma reduced/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool Thaw DMSO OPN PLS Red/PA BUFCT 5DNR', N'E026', N'@77 B2 EE G2 J5', 200, N'E7091', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5e3db72d-da2a-495f-932b-fe0ebdaa0011', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|1st container|Psoralen-treated', N'PLT AFR Wash PAS-E LUKOPR Psoraln', N'E027', N'@D5 D5 H3 K5', 200, N'E7025', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5e5f6a8d-cb44-4795-ab6b-33369aeeccc1', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool Thaw DMSO OPN LUKOPR BUFCT 5DNR', N'E026', N'@77 B2 D5 G2 J5', 200, N'E7087', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5ec6e434-3012-4425-97d6-6b013a749475', N'Washed POOLED PLATELETS|None/XX/20-24C|Irradiated|Buffy coat plts prep|From 8 donors', N'PLT Pool Wash None IRD BUFCT 8DNR', N'E023', N'@71 C2 G2 J8', 200, N'E6732', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5eddfeb7-615f-4d48-86b8-e257d6d043bb', N'Apheresis PLATELETS|ACD-A/XX/20-24C|Open|ResLeu:<5log6|2nd container|<3log11 plts', N'PLT AFR ACD-A OPN LUKOPR', N'E024', N'@73 B2 D4 H4 JI', 200, N'E6844', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5ede15b8-1cb0-4f84-9374-7ded621038f4', N'Apheresis RED BLOOD CELLS|CP2D>AS3/XX/refg|Open|ResLeu:<5log6|Plasma added|2nd container', N'RBC AFR CP2D>AS3 OPN LUKOPR PA', N'E009', N'@33 B2 D4 E4 H4', 350, N'E6795', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5f0ef3a0-790d-4531-b3f5-0b87a71fdc2d', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|From 5 donors', N'PLT Pool Thaw DMSO OPN 5DNR', N'E026', N'@77 B2 J5', 200, N'E7110', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5f4f5429-ffe7-44f9-a1dc-02b7a768f72b', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|Irradiated|ResLeu:<5log6|3rd container', N'PLT AFR ACD-A>PAS-C IRD LUKOPR', N'E024', N'@BH C2 D4 H5', 200, N'E7008', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'5fcfca0a-df0e-43a8-a2f2-06ff8195ada0', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-C IRD LUKOPR BUFCT 2DNR', N'E022', N'@B5 C2 D5 G2 J2', 200, N'E6705', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'600d3693-030e-47d9-b1fd-8da3b6c9fa41', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-E OPN IRD SR/PA BUFCT 8DNR', N'E022', N'@D5 B2 C2 E8 G2 J8', 200, N'E6943', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6039ec7a-69ba-4abb-93ae-58a224735428', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|1st container|Bacterial monitoring', N'PLT AFR PAS-E IRD LUKOPR BacMtr', N'E024', N'@D5 C2 D5 H3 N2', 200, N'E6748', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'60539950-9efd-4cf0-ba61-5680c8513dbe', N'Thawed Apheresis PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Riboflavin-treated', N'PLT AFR Thaw DMSO OPN LUKOPR Ribo', N'E026', N'@77 B2 D5 K6', 200, N'E7064', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'605e0158-0d04-4877-90ac-76343125a09b', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added', N'PLT AFR PAS-E OPN IRD LUKOPR SR/PA', N'E024', N'@D5 B2 C2 D5 E8', 200, N'E6881', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'606cec4e-cc00-4291-b8c3-31b0551d6947', N'RED BLOOD CELLS|CPD>AS5/500mL/refg|Open|ResLeu:<5log6|Plasma added', N'RBC CPD>AS5 OPN LUKOPR PA', N'E002', N'@23 B2 D4 E4', 350, N'E6787', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'61e01a98-a840-4996-adfb-92fca097b7f3', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|From 5 donors', N'PLT Pool Froz DMSO OPN 5DNR', N'E025', N'@CP B2 J5', 200, N'E7100', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'621dee8b-d883-4996-b9ff-8da29cb6164d', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part C or 3)', N'PLASMA AFR Thaw None', N'E013', N'@AS H5', 225, N'49213', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'62679262-8193-4021-ae03-2f1d75e29038', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool Wash None OPN LUKOPR BUFCT 4DNR', N'E023', N'@71 B2 D5 G2 J4', 200, N'E6836', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'627eaf10-0051-45df-9d92-3b45efd4d424', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|From 7 donors', N'PLT Pool Froz DMSO OPN 7DNR', N'E025', N'@CP B2 J7', 200, N'E7102', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'62d0177a-5099-4a42-842e-f73928f04079', N'PLASMA|CP2D/XX/<=-18C|For mnf:noninjectable|Frozen <=72h', N'PLASMA CP2D MNFNI', N'E010', N'@37 A3 GD', 225, N'E6648', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'632abc72-2422-4dbe-bd8d-b45300a7d294', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED (variable contents:<3 X log11 plts)', N'PLT AFR Wash None IRD LUKOPR', N'E027', N'@71 B2 C2 D2 JI', 200, N'12941', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6340b21d-a005-4c94-9567-09c7bb02d4d9', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Buffy coat plts prep|From 6 donors|Riboflavin-treated', N'PLT Pool CPD IRD BUFCT 6DNR Ribo', N'E022', N'@A7 C2 G2 J6 K6', 200, N'E6681', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6369398e-1507-47da-8b24-030dd3c9843e', N'POOLED PLATELETS|CP2D/XX/20-24C|ResLeu:<5log6|From 5 donors|Bacterial monitoring', N'PLT Pool CP2D LUKOPR 5DNR BacMtr', N'E022', N'@B9 D4 J5 N2', 200, N'E7150', 120, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'639677df-0cad-4984-9cb1-85f4575c6969', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|3rd container|Psoralen-treated', N'PLT AFR PAS-E LUKOPR Psoraln', N'E024', N'@D5 D5 H5 K5', 200, N'E7019', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6449e3fe-fd62-43bb-bb30-e561b4e2aa61', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool Wash None OPN LUKOPR BUFCT 5DNR', N'E023', N'@71 B2 D5 G2 J5', 200, N'E6837', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'64aa2daf-755c-4035-952d-5b4fe6c2fc73', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep', N'PLT Pool PAS-E IRD LUKOPR BUFCT', N'E022', N'@D5 C2 D5 G2', 200, N'E6777', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6505c4de-95fb-4b46-969d-cc5855344a3a', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-C IRD LUKOPR BUFCT 5DNR', N'E022', N'@B5 C2 D5 G2 J5', 200, N'E6708', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'65818514-1c3e-43ca-b7d2-984800fb8a56', N'Apheresis RED BLOOD CELLS|ACD-A>AS3/XX/refg|Open|ResLeu:<5log6|Plasma added|2nd container', N'RBC AFR ACD-A>AS3 OPN LUKOPR PA', N'E009', N'@AZ B2 D4 E4 H4', 350, N'E6790', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'65a0de47-659f-4ec9-9282-60efa3f9eaa8', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-C IRD SR/PA BUFCT 7DNR', N'E022', N'@B5 C2 E8 G2 J7', 200, N'E6717', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'65e66ff2-945e-4224-bf34-eac16da7b5f3', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|From 3 donors', N'PLT Pool Thaw DMSO OPN LUKOPR 3DNR', N'E026', N'@77 B2 D5 J3', 200, N'E7118', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6662b5df-1ad9-42c1-9a15-5d4d0f84ce30', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container|Bacterial monitoring', N'PLT AFR PAS-E IRD LUKOPR BacMtr', N'E024', N'@D5 C2 D5 H5 N2', 200, N'E6750', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6677be0a-88c8-4739-9058-52e696de8ddd', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 6 donors', N'PLT Pool CPD OPN IRD PLS Red/PA BUFCT 6DNR', N'E022', N'@A7 B2 C2 EE G2 J6', 200, N'E6906', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'66b577b3-3e93-4d00-9d27-a93ab98e77fe', N'Apheresis PLATELETS|ACD-A/XX/20-24C|Open|ResLeu:<5log6|Plasma reduced|3rd container|<3log11 plts', N'PLT AFR ACD-A OPN LUKOPR PLS Red', N'E024', N'@73 B2 D4 E5 H5 JI', 200, N'E6806', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'66db1d7c-a125-4ead-a4ab-7b8c5193312d', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Plasma reduced/Plasma added|From 4 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR PLS Red/PA 4DNR', N'E026', N'@77 B2 C2 D5 EE J4', 200, N'E7144', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'68a29f83-a433-4a86-8e04-301673a0b0eb', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool CPD OPN LUKOPR BUFCT 6DNR', N'E022', N'@A7 B2 D5 G2 J6', 200, N'E6892', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'698ab7b5-9ab9-4fc3-91df-ad18f9125bfc', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|From 7 donors', N'PLT Pool Thaw DMSO OPN IRD 7DNR', N'E026', N'@77 B2 C2 J7', 200, N'E7117', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6a554650-6897-44eb-bf40-3e490bb42e1f', N'PLASMA|CPDA-1/450mL/<-25C|Cryo reduced|Quar:>=112d/retested', N'PLASMA CPDA-1 CRYORed QUAR', N'E010', N'@CV E3 I4', 225, N'E7075', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6b816a42-8cf1-403f-955a-881f8b5c6a6b', N'Frozen Apheresis PLATELETS|DMSO/XX/<-65C|Open|ResLeu:<5log6|1st container', N'PLT AFR Froz DMSO OPN LUKOPR', N'E025', N'@76 B2 D4 H3', 200, N'E6866', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6c60f87a-068e-4876-a0b1-8514aa490c24', N'Apheresis RED BLOOD CELLS|CP2D>AS3/XX/refg|Open|ResLeu:<5log6|Plasma added', N'RBC AFR CP2D>AS3 OPN LUKOPR PA', N'E009', N'@33 B2 D4 E4', 350, N'E6793', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6c6955b1-197c-4940-9738-f2f9c7deed6c', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool CPD OPN IRD PLS Red/PA BUFCT 4DNR', N'E022', N'@A7 B2 C2 EE G2 J4', 200, N'E6904', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6eff401b-6279-409a-8587-4be8e5c52898', N'POOLED PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-E LUKOPR BUFCT 8DNR', N'E022', N'@D5 D5 G2 J8', 200, N'E6864', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'6f2d7acd-4388-4db8-99a8-5d650f06dbc2', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-C IRD LUKOPR BUFCT 4DNR', N'E022', N'@B5 C2 D5 G2 J4', 200, N'E6707', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'70498679-0e2f-435f-beac-b782a0d9ff18', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|From 5 donors', N'PLT Pool Thaw DMSO OPN IRD PLS Red/PA 5DNR', N'E026', N'@77 B2 C2 EE J5', 200, N'E7135', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'70bf0def-f6e5-428e-8e90-d289f2278711', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED (may be used for second bag of double or triple plateletpheresis)', N'PLT AFR PAS-C LUKOPR', N'E024', N'@B5 D2 H4', 200, N'12920', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'70c438fc-b24f-4a71-8e2a-61aece40590a', N'POOLED PLATELETS|CPD/XX/20-24C|Buffy coat plts prep|From 8 donors', N'PLT Pool CPD BUFCT 8DNR', N'E022', N'@A7 G2 J8', 200, N'E6887', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'719cd020-dffd-49c9-b1d0-1346f064de67', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-E OPN LUKOPR SR/PA BUFCT 4DNR', N'E022', N'@D5 B2 D5 E8 G2 J4', 200, N'E6988', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'725542e4-c1dc-4242-85c7-f9ceb4640d34', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|4th container', N'PLT AFR PAS-E IRD LUKOPR', N'E024', N'@D5 C2 D5 H6', 200, N'E6743', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'72b7b724-8457-4b81-838c-5976244be8fa', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|3rd container', N'PLT AFR ACD-A>PAS-C', N'E024', N'@BH H5', 200, N'E6996', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'72f060d1-0db4-4382-8897-3c1dfe7fe31d', N'Thawed POOLED PLASMA|CPDA-1/XX/refg|Open', N'PLASMA Pool Thaw CPDA-1 OPN', N'E011', N'@10 B2', 500, N'E7046', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'72f4115c-3aad-4a73-b933-c6cf765d7425', N'Thawed FRESH FROZEN PLASMA|CP2D/XX/refg|ResLeu:<5log6', N'FFP Thaw CP2D LUKOPR', N'E011', N'@13 D4', 225, N'E6657', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'73cc021a-ed4f-44c6-95a4-ae2330add909', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|Supernat rem/Plasma added', N'PLT AFR PAS-E OPN IRD SR/PA', N'E024', N'@D5 B2 C2 E8', 200, N'E6877', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'73cfeae5-aae7-493f-bb92-ff2b34b75a80', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool Thaw DMSO LUKOPR BUFCT 6DNR', N'E026', N'@77 D5 G2 J6', 200, N'E7059', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'742f7e65-9dcc-4f4d-9302-628ccaf4af1e', N'Washed Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool Thaw Wash DMSO OPN IRD LUKOPR BUFCT 4DNR', N'E027', N'@77 B2 C2 D5 G2 J4', 200, N'E7162', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'74791fc3-87d8-4eba-bbbe-36ed2c4d44fd', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Supernat rem/Plasma added|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-E OPN SR/PA BUFCT 7DNR', N'E022', N'@D5 B2 E8 G2 J7', 200, N'E6949', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'75390bb1-f609-4523-bdb2-c3cbeb0f6e25', N'Frozen Apheresis PLATELETS|None/XX/<=-18C|For mnf:noninjectable|ResLeu:<5log6', N'PLT AFR Froz None MNFNI LUKOPR', N'E025', N'@84 A3 D4', 200, N'E6808', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'75746a8a-6a15-4daf-992a-b8f5f29b7a36', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|1st container|Psoralen-treated', N'PLT AFR PAS-E LUKOPR Psoraln', N'E024', N'@D5 D5 H3 K5', 200, N'E7017', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'75e3e8f1-667c-47fc-94ec-b2474c754e80', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-E OPN IRD LUKOPR BUFCT 7DNR', N'E022', N'@D5 B2 C2 D5 G2 J7', 200, N'E6984', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7620f99e-8817-42f3-803a-dbb29e7aaad0', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-E IRD BUFCT 5DNR', N'E022', N'@D5 C2 G2 J5', 200, N'E6926', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7681e461-c24d-45da-a2eb-9790d1bcbf82', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|From 4 donors', N'PLT Pool Thaw DMSO OPN IRD PLS Red/PA 4DNR', N'E026', N'@77 B2 C2 EE J4', 200, N'E7134', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7706156c-79c9-47e5-8b68-74e8e0ab1c71', N'Washed Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool Thaw Wash DMSO OPN IRD LUKOPR BUFCT 5DNR', N'E027', N'@77 B2 C2 D5 G2 J5', 200, N'E7163', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'77a0c696-0ee5-45cc-b2a9-89a52250a270', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Supernat rem/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-E OPN SR/PA BUFCT 5DNR', N'E022', N'@D5 B2 E8 G2 J5', 200, N'E6947', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'783456e3-473e-4586-b431-925a33543ac4', N'POOLED PLATELETS|CPD/XX/20-24C|ResLeu:<1log6|Plasma reduced|Buffy coat plts prep|From 5 donors', N'PLT Pool CPD LUKOPR PLS Red BUFCT 5DNR', N'E022', N'@A7 D5 E5 G2 J5', 200, N'E7155', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'78e90670-8716-4ea6-9699-8ffe8b9da0d2', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool Thaw DMSO OPN IRD PLS Red/PA BUFCT 4DNR', N'E026', N'@77 B2 C2 EE G2 J4', 200, N'E7092', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'792a0d60-dd3e-49a5-91e1-c45bd3bc3984', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-C IRD LUKOPR BUFCT 7DNR', N'E022', N'@B5 C2 D5 G2 J7', 200, N'E6710', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'797b13a8-d96b-447d-a425-c905c31269c9', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-C IRD LUKOPR SR/PA BUFCT 6DNR', N'E022', N'@B5 C2 D5 E8 G2 J6', 200, N'E6723', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'79cf6c22-d558-4276-af21-e36e76818e5e', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container', N'PLT AFR Wash PAS-E IRD LUKOPR', N'E027', N'@D5 C2 D5 H4', 200, N'E6773', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7a375479-abe7-4218-8a2d-ccd795b98fb3', N'Apheresis RED BLOOD CELLS|CP2D>AS3/XX/refg|Open|ResLeu:<5log6|Plasma added|1st container', N'RBC AFR CP2D>AS3 OPN LUKOPR PA', N'E009', N'@33 B2 D4 E4 H3', 350, N'E6794', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7aa5d34e-cf7b-4957-a7f2-b00c1b47d801', N'PLASMA|CPD/450mL/<-25C|Frozen >24h', N'PLASMA CPD', N'E010', N'@A2 G8', 225, N'E6825', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7b292c64-e6ec-4f2b-9380-7c3729582efe', N'Frozen Apheresis PLATELETS|DMSO/XX/<-65C|Open|ResLeu:<5log6|2nd container', N'PLT AFR Froz DMSO OPN LUKOPR', N'E025', N'@76 B2 D4 H4', 200, N'E6867', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7b2e2a03-bbca-4427-92bd-03da8358292c', N'POOLED PLATELETS|CPD/XX/20-24C|Buffy coat plts prep|From 3 donors', N'PLT Pool CPD BUFCT 3DNR', N'E022', N'@A7 G2 J3', 200, N'E6883', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7b67ae2c-0bff-4fdc-aa5e-920734483f2f', N'Thawed Apheresis POOLED FRESH FROZEN PLASMA|NaCitrate/XX/refg|Open', N'FFP AFR Pool Thaw NaCit OPN', N'E013', N'@55 B2', 225, N'E7030', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7bcc572e-1b1b-4811-8778-dac66cfffaaa', N'Washed POOLED PLATELETS|None/XX/20-24C|Irradiated|Buffy coat plts prep|From 6 donors', N'PLT Pool Wash None IRD BUFCT 6DNR', N'E023', N'@71 C2 G2 J6', 200, N'E6730', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7c270a19-c6c3-48a6-9c08-b50b1ebe3e37', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part D or 4)', N'PLASMA AFR None', N'E012', N'@84 H6', 225, N'49214', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7c6558f0-2565-4e21-ad59-760f0329f507', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-C IRD BUFCT 6DNR', N'E022', N'@B5 C2 G2 J6', 200, N'E6702', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7c868cbb-6ffc-428c-a468-32364ff8da5a', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|From 6 donors', N'PLT Pool Thaw DMSO OPN IRD PLS Red/PA 6DNR', N'E026', N'@77 B2 C2 EE J6', 200, N'E7136', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7d12a058-15a1-4eca-a421-9be40c5b3bf7', N'Apheresis RED BLOOD CELLS|CP2D>AS3/XX/refg|Open|Irradiated|ResLeu:<5log6|Plasma added|1st container', N'RBC AFR CP2D>AS3 OPN IRD LUKOPR PA', N'E009', N'@33 B2 C2 D4 E4 H3', 350, N'E6797', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7d95d4cb-9bc5-4146-9518-ba88c26c8f55', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|From 3 donors', N'PLT Pool Thaw DMSO OPN LUKOPR PLS Red/PA 3DNR', N'E026', N'@77 B2 D5 EE J3', 200, N'E7138', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7e362181-0953-4e72-8179-791257a0a252', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool CPD IRD PLS Red/PA BUFCT 4DNR', N'E022', N'@A7 C2 EE G2 J4', 200, N'E6672', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7e6d7e96-9c80-49c9-8306-4e94143df668', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-E OPN LUKOPR BUFCT 2DNR', N'E022', N'@D5 B2 D5 G2 J2', 200, N'E6972', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7e845e05-b5e1-41ef-81e1-d898a7442fec', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|3rd container|Bacterial monitoring', N'PLT AFR PAS-E LUKOPR BacMtr', N'E024', N'@D5 D5 H5 N2', 200, N'E6746', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7f0c051b-42ad-4867-be17-d9a3f826c650', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool CPD IRD PLS Red/PA BUFCT 5DNR', N'E022', N'@A7 C2 EE G2 J5', 200, N'E6673', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7f40e3b5-2404-4714-8a62-85599dd55fe1', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|2nd container|Bacterial monitoring', N'PLT AFR PAS-E LUKOPR BacMtr', N'E024', N'@D5 D5 H4 N2', 200, N'E6745', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'7fa85586-48a1-423e-9bdd-bd39c009ce04', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-E OPN IRD LUKOPR BUFCT 8DNR', N'E022', N'@D5 B2 C2 D5 G2 J8', 200, N'E6985', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'806905a8-4bf6-45a9-86ce-5972f8490332', N'Frozen Apheresis PLATELETS|DMSO/XX/<-65C|Open|ResLeu:<5log6', N'PLT AFR Froz DMSO OPN LUKOPR', N'E025', N'@76 B2 D4', 200, N'E6865', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'80aac576-47df-4c60-8703-890b0fcd74cc', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-C IRD LUKOPR SR/PA BUFCT 2DNR', N'E022', N'@B5 C2 D5 E8 G2 J2', 200, N'E6719', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'81133366-5750-434e-b34e-01fadafa7e95', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-E IRD LUKOPR SR/PA BUFCT 2DNR', N'E022', N'@D5 C2 D5 E8 G2 J2', 200, N'E6958', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'81609445-bd24-412d-a307-8215d58155b8', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Supernat reduced|1st container', N'PLT AFR PAS-E LUKOPR SUPRed', N'E024', N'@D5 D5 EA H3', 200, N'E6760', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8178dbc8-becb-460b-97e7-a2fe1d991865', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 4 donors|Riboflavin-treated', N'PLT Pool Thaw DMSO OPN LUKOPR BUFCT 4DNR Ribo', N'E026', N'@77 B2 D5 G2 J4 K6', 200, N'E7068', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'817c961e-aa30-477a-9e66-d46e700b1821', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|Buffy coat plts prep|From 5 donors', N'PLT Pool Froz DMSO OPN BUFCT 5DNR', N'E025', N'@CP B2 G2 J5', 200, N'E7079', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'82138d9c-7c23-48c7-86d7-c542df3cdf82', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool CPD OPN IRD LUKOPR BUFCT 6DNR', N'E022', N'@A7 B2 C2 D5 G2 J6', 200, N'E6920', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'82cb72f9-3aed-4123-b58d-a5b961d02e6a', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-E IRD SR/PA BUFCT 7DNR', N'E022', N'@D5 C2 E8 G2 J7', 200, N'E6935', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'82db4fbf-17b2-4005-b248-b3a35de6c2d7', N'POOLED PLATELETS|CPD/XX/20-24C|ResLeu:<2log5|Buffy coat plts prep|From 4 donors', N'PLT Pool CPD LUKOPR BUFCT 4DNR', N'E022', N'@A7 D8 G2 J4', 200, N'E6847', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'82f1de88-4b38-4b66-8eb5-22a1cea94026', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-C IRD BUFCT 7DNR', N'E022', N'@B5 C2 G2 J7', 200, N'E6703', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'83d34938-5033-41eb-ac10-e9cbc73d3b7b', N'Apheresis RED BLOOD CELLS|ACD-A>AS3/XX/refg|Open|ResLeu:<5log6|Plasma added|1st container', N'RBC AFR ACD-A>AS3 OPN LUKOPR PA', N'E009', N'@AZ B2 D4 E4 H3', 350, N'E6789', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'84ab3e91-7cc9-448e-87a3-5c11a048e02d', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part B or 2)', N'PLASMA AFR Thaw None IRD ', N'E013', N'@AS C2 H4', 225, N'49612', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'85afc7c2-cbca-407f-8f79-46d3443781d0', N'PLASMA|CPD/500mL/<=-18C|For mnf:injectable|ResLeu:NS|Cryo reduced', N'PLASMA CPD MNFI LUKOPR CRYORed', N'E010', N'@AN A2 D2 E3', 225, N'E6804', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'863b1eab-4e65-4a5b-b1f3-ca51fd8a00d2', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C', N'PLT AFR ACD-A>PAS-C', N'E024', N'@BH', 200, N'E6993', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'86400163-e30b-48b0-8d6c-2bae2e247412', N'Apheresis PLATELETS|ACD-A/XX/20-24C|Open|ResLeu:<1log6|Supernat rem/Plasma added', N'PLT AFR ACD-A OPN LUKOPR SR/PA', N'E024', N'@73 B2 D5 E8', 200, N'E6872', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'86c74ea7-d287-4f94-a380-41db0c44c4e0', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-C IRD BUFCT 3DNR', N'E022', N'@B5 C2 G2 J3', 200, N'E6699', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'86da531d-4ad1-47ec-bb13-a12d0694c7ff', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|1st container', N'PLT AFR PAS-E IRD LUKOPR', N'E024', N'@D5 C2 D5 H3', 200, N'E6740', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'86ef6bc9-ef4f-4359-9424-c1138c5742ac', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-E OPN IRD LUKOPR BUFCT 3DNR', N'E022', N'@D5 B2 C2 D5 G2 J3', 200, N'E6980', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8716ae12-6de6-4ccb-810f-fdba2f511d4a', N'CRYOPRECIPITATE|CPD/350mL/<-30C', N'CRYO CPD', N'E028', N'@CZ', 15, N'E7037', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8794d9df-e79e-4765-b7a4-d27bf61b3ad7', N'PLASMA|CPDA-1/450mL/<-25C|Cryo reduced|Riboflavin-treated', N'PLASMA CPDA-1 CRYORed Ribo', N'E010', N'@CV E3 K6', 225, N'E7077', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'87c9e999-56ab-4fb0-a6fc-929d736ac1c7', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container', N'PLT AFR Wash PAS-E IRD LUKOPR', N'E027', N'@D5 C2 D5 H5', 200, N'E6774', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'880ee9b7-13e5-4fbe-aae8-813749d42814', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-E IRD LUKOPR BUFCT 4DNR', N'E022', N'@D5 C2 D5 G2 J4', 200, N'E6953', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'89826014-130e-465e-a4fe-0e619d2d9759', N'Frozen Apheresis PLATELETS|DMSO/XX/<-80C|Open|ResLeu:<1log6|Riboflavin-treated', N'PLT AFR Froz DMSO OPN LUKOPR Ribo', N'E025', N'@CP B2 D5 K6', 200, N'E7063', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8ab43a11-dd0a-45e3-a112-9395d6895a14', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|From 3 donors', N'PLT Pool Thaw DMSO OPN IRD 3DNR', N'E026', N'@77 B2 C2 J3', 200, N'E7113', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8b27836d-4859-4898-ac7c-0160858b08a9', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 2 donors', N'PLT Pool CPD IRD PLS Red/PA BUFCT 2DNR', N'E022', N'@A7 C2 EE G2 J2', 200, N'E6670', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8b6287c9-c738-4c53-8d8d-f0f4dc4904b2', N'POOLED PLATELETS|CPD/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 5 donors|Riboflavin-treated', N'PLT Pool CPD LUKOPR BUFCT 5DNR Ribo', N'E022', N'@A7 D5 G2 J5 K6', 200, N'E6687', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8b8940cd-19b7-4c9f-9739-5b96fd94fe0e', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-E OPN LUKOPR BUFCT 5DNR', N'E022', N'@D5 B2 D5 G2 J5', 200, N'E6975', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8bc4eda9-879f-42a9-9caa-1519b13a7031', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|ResLeu:<5log6|2nd container', N'PLT AFR ACD-A>PAS-C LUKOPR', N'E024', N'@BH D4 H4', 200, N'E7003', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8bd234ae-2ac0-428f-bda9-5ad622d55060', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool Froz DMSO OPN LUKOPR BUFCT 4DNR', N'E025', N'@CP B2 D5 G2 J4', 200, N'E7080', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8bd8dfac-de40-4966-8b75-72d6f80ce043', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Supernat reduced|4th container', N'PLT AFR PAS-E LUKOPR SUPRed', N'E024', N'@D5 D5 EA H6', 200, N'E6763', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8bf7eb38-3245-4d89-a465-045ece87a0f7', N'Washed RED BLOOD CELLS|AS3/500mL/refg|ResLeu:<5log6', N'RBC Wash AS3 LUKOPR', N'E003', N'@D7 D4', 200, N'E6813', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8bfcdab0-e6fd-4c37-9027-bad5a3275998', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|From 3 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR 3DNR', N'E026', N'@77 B2 C2 D5 J3', 200, N'E7123', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8c437051-187b-4be1-bb10-d40432ab5403', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|1st container', N'PLT AFR Wash PAS-E IRD LUKOPR', N'E027', N'@D5 C2 D5 H3', 200, N'E6772', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8d53dee4-8f81-4f7c-9803-0c9bd82fbc0d', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-E IRD LUKOPR BUFCT 5DNR', N'E022', N'@D5 C2 D5 G2 J5', 200, N'E6954', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8de6dca9-3523-49b7-8ba5-d91c23008524', N'PLASMA|CPD/450mL/<=-18C|Frozen >24h', N'PLASMA CPD', N'E010', N'@AM G8', 225, N'E6823', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8df1a0ed-75c8-40c3-b6d9-538e9e9cdfb8', N'POOLED PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-E LUKOPR BUFCT 4DNR', N'E022', N'@D5 D5 G2 J4', 200, N'E6860', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8e52bd4b-e0ac-4505-80ff-3cc8d3b71b43', N'POOLED PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-E LUKOPR BUFCT 5DNR', N'E022', N'@D5 D5 G2 J5', 200, N'E6861', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8ea9242f-ade0-49d9-a93f-4347ce994960', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-E IRD SR/PA BUFCT 8DNR', N'E022', N'@D5 C2 E8 G2 J8', 200, N'E6936', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8ed59f2d-6d5a-4b35-91af-ffb7727d377b', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED (may be used for second bag of double or triple plateletpheresis)(variable contents:<3 X log11 plts)', N'PLT AFR PAS-C LUKOPR', N'E024', N'@B5 D2 H4 JI', 200, N'12921', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8fcbf13e-c681-4557-854f-14f58451692b', N'POOLED PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-E LUKOPR BUFCT 3DNR', N'E022', N'@D5 D5 G2 J3', 200, N'E6859', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'8fe02304-cda2-487f-b498-85ac4ff815ef', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part G or 7)', N'PLASMA AFR None IRD ', N'E012', N'@84 C2 HB', 225, N'49617', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'90061372-da8f-4f55-911a-dcc858192c3d', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-E OPN IRD SR/PA BUFCT 4DNR', N'E022', N'@D5 B2 C2 E8 G2 J4', 200, N'E6939', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'90799eee-86fe-4409-bb52-70f3fae1756d', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-E OPN LUKOPR SR/PA BUFCT 2DNR', N'E022', N'@D5 B2 D5 E8 G2 J2', 200, N'E6986', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'90b8c3d0-7388-476f-9259-afa0427435ea', N'POOLED PLATELETS|PAS-E/XX/20-24C|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-E BUFCT 6DNR', N'E022', N'@D5 G2 J6', 200, N'E6855', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'90c02bbe-884b-4c59-908a-83711a81eecc', N'RED BLOOD CELLS|CPD/500mL/refg|ResLeu:<5log6|Plasma added', N'RBC CPD LUKOPR PA', N'E002', N'@04 D4 E4', 250, N'E7032', 504, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'91265fd7-a501-4793-9248-bda80ff8e3ea', N'PLASMA|CP2D/XX/<=-20C|For mnf:injectable restr use|Frozen <=72h', N'PLASMA CP2D MNFRI', N'E010', N'@BS A6 GD', 225, N'E6842', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'912af517-7fab-42e4-8282-f6cfb6eb7717', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part F or 6)', N'PLASMA AFR Thaw None IRD ', N'E013', N'@AS C2 HA', 225, N'49616', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'91830e91-e801-4052-9aa1-bd3248481c7d', N'Apheresis PLASMA|NaCitrate/XX/<=-18C|Cryo reduced|Quar:>=112d/retested', N'PLASMA AFR NaCit CRYORed QUAR', N'E012', N'@48 E3 I4', 225, N'E6663', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'91979c3d-de86-4a4b-ad62-9c21bf24a6c4', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part G or 7)', N'PLASMA AFR Thaw None IRD ', N'E013', N'@AS C2 HB', 225, N'49617', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'91c16248-7a8c-4cc6-8eeb-ed490506e418', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 4 donors|Riboflavin-treated', N'PLT Pool CPD IRD LUKOPR BUFCT 4DNR Ribo', N'E022', N'@A7 C2 D5 G2 J4 K6', 200, N'E6693', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'925e8b80-8915-4bf5-99f6-a26257b951c6', N'Thawed Apheresis PLASMA|NaCitrate/XX/<37C|For mnf:noninjectable', N'PLASMA AFR Thaw NaCit MNFNI', N'E013', N'@D6 A3', 225, N'E6735', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'92dcfe24-735b-4df3-93b7-40ab50c4c9f0', N'Washed POOLED PLATELETS|None/XX/20-24C|Irradiated|Buffy coat plts prep|From 7 donors', N'PLT Pool Wash None IRD BUFCT 7DNR', N'E023', N'@71 C2 G2 J7', 200, N'E6731', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'931c230f-b020-4149-b25b-9459e7ee44b2', N'Washed Apheresis PLATELETS|None/XX/20-24C|Irradiated|ResLeu:<1log6', N'PLT AFR Wash None IRD LUKOPR', N'E027', N'@71 C2 D5', 200, N'E6667', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9394bd60-b41e-4f7e-9052-fb8acade0aef', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Supernat reduced|1st container|Psoralen-treated', N'PLT AFR PAS-E LUKOPR SUPRed Psoraln', N'E024', N'@D5 D5 EA H3 K5', 200, N'E7021', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'93b3650e-d652-42fb-9a27-f70af6b446d8', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 7 donors', N'PLT Pool CPD IRD PLS Red/PA BUFCT 7DNR', N'E022', N'@A7 C2 EE G2 J7', 200, N'E6675', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'94463eab-90f6-45bc-88e6-d5d3792df773', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part E or 5)', N'PLASMA AFR Thaw None', N'E013', N'@AS H9', 225, N'49215', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9462813f-d60e-41fc-9d1e-f7c85f787e28', N'POOLED PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-E LUKOPR BUFCT 6DNR', N'E022', N'@D5 D5 G2 J6', 200, N'E6862', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'947fda5c-d283-4a41-89ec-ec9a510f60ec', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Plasma reduced/Plasma added|Buffy coat plts prep|From 2 donors', N'PLT Pool CPD OPN PLS Red/PA BUFCT 2DNR', N'E022', N'@A7 B2 EE G2 J2', 200, N'E6895', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'957b86a5-9d0d-4c74-bffc-d1c146925f90', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-E OPN IRD LUKOPR BUFCT 6DNR', N'E022', N'@D5 B2 C2 D5 G2 J6', 200, N'E6983', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9638907a-f796-4782-98f5-242193ac295c', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|From 6 donors', N'PLT Pool Froz DMSO OPN 6DNR', N'E025', N'@CP B2 J6', 200, N'E7101', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'96744e5a-6e6f-4b09-9c76-2846a015b3cb', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-E IRD SR/PA BUFCT 2DNR', N'E022', N'@D5 C2 E8 G2 J2', 200, N'E6930', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'96e028e9-3b96-4a61-b7ee-3525f5734ccf', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED (variable contents:<3 X log11 plts)', N'PLT AFR PAS-C IRD LUKOPR', N'E024', N'@B5 C2 D2 JI', 200, N'12941', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9719cffe-b8f9-4633-86dc-25714baa9ed4', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-C IRD LUKOPR SR/PA BUFCT 5DNR', N'E022', N'@B5 C2 D5 E8 G2 J5', 200, N'E6722', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'99d20d68-5cc9-47b8-9d0d-bcffdb3c612b', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part A or 1)', N'PLASMA AFR Thaw None', N'E013', N'@AS H3', 225, N'49211', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9c837182-93f2-4464-bd7d-6038133ba47c', N'FRESH FROZEN PLASMA|CPD/XX/<-30C|ResLeu:<5log6|50mL|Riboflavin-treated', N'FFP CPD LUKOPR Ribo', N'E010', N'@A4 D4 FA K6', 225, N'E7042', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9c8d190f-334b-4e73-b9fa-805318acfd54', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part G or 7)', N'PLASMA AFR None', N'E012', N'@84 HB', 225, N'49217', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9cdca9db-3977-4923-b137-57b9845151fe', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Supernat reduced|3rd container|Psoralen-treated', N'PLT AFR PAS-E LUKOPR SUPRed Psoraln', N'E024', N'@D5 D5 EA H5 K5', 200, N'E7023', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9d7b9bb8-6ba4-4715-acd5-d92adbfcfab8', N'Apheresis PLATELETS|ACD-A/XX/20-24C|Irradiated|ResLeu:<5log6|Plasma reduced|3rd container|<3log11 plts', N'PLT AFR ACD-A IRD LUKOPR PLS Red', N'E024', N'@73 C2 D4 E5 H5 JI', 200, N'E6812', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9d9f4372-a9c0-48ed-8961-5f4b700336a3', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part A or 1)', N'PLASMA AFR Thaw None IRD ', N'E013', N'@AS C2 H3', 225, N'49611', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9e6a27aa-7cca-4e9e-b8f9-ee3de512890b', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Plasma reduced/Plasma added|From 5 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR PLS Red/PA 5DNR', N'E026', N'@77 B2 C2 D5 EE J5', 200, N'E7145', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9e931359-575b-4096-bc60-e2095c6a9fdf', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Buffy coat plts prep|From 5 donors', N'PLT Pool Thaw DMSO OPN BUFCT 5DNR', N'E026', N'@77 B2 G2 J5', 200, N'E7083', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9ec5cbad-7172-4b8f-ab78-8bc759909db5', N'Apheresis PLATELETS|PAS-D/XX/20-24C|Irradiated|ResLeu:<1log6', N'PLT AFR PAS-D IRD LUKOPR', N'E024', N'@CI C2 D5', 200, N'E6665', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9f05e4c2-8294-4e55-8c1c-f59480b48ba0', N'POOLED PLATELETS|CPD/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 4 donors|Riboflavin-treated', N'PLT Pool CPD LUKOPR BUFCT 4DNR Ribo', N'E022', N'@A7 D5 G2 J4 K6', 200, N'E6686', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'9f31cd22-c3e2-4726-8721-8998e7767cde', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Buffy coat plts prep|From 5 donors|Riboflavin-treated', N'PLT Pool CPD IRD BUFCT 5DNR Ribo', N'E022', N'@A7 C2 G2 J5 K6', 200, N'E6680', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a0a14d69-e6db-45ee-8645-a04fe7002d60', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-E IRD SR/PA BUFCT 6DNR', N'E022', N'@D5 C2 E8 G2 J6', 200, N'E6934', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a0c74c8d-cd88-481f-9d5a-769990f98709', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|ResLeu:<1log6|From 7 donors', N'PLT Pool Froz DMSO OPN LUKOPR 7DNR', N'E025', N'@CP B2 D5 J7', 200, N'E7107', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a1aef2a2-592a-475d-88aa-ee360a5bcb48', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Plasma reduced/Plasma added|Buffy coat plts prep|From 3 donors', N'PLT Pool CPD OPN PLS Red/PA BUFCT 3DNR', N'E022', N'@A7 B2 EE G2 J3', 200, N'E6896', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a25ad28b-167c-4d5c-a289-99fdf9e5dcb4', N'Apheresis RED BLOOD CELLS|CP2D>AS3/XX/refg|Open|Irradiated|ResLeu:<5log6|Plasma added', N'RBC AFR CP2D>AS3 OPN IRD LUKOPR PA', N'E009', N'@33 B2 C2 D4 E4', 350, N'E6796', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a31ed8fc-8aad-4901-810a-9f47e6450672', N'POOLED PLATELETS|CPD/XX/20-24C|Buffy coat plts prep|From 4 donors', N'PLT Pool CPD BUFCT 4DNR', N'E022', N'@A7 G2 J4', 200, N'E6647', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a3461abb-ea83-4de1-b5ea-c07c14945a85', N'CRYOPRECIPITATE|CPD/450mL/<-25C|Riboflavin-treated', N'CRYO CPD Ribo', N'E028', N'@A2 K6', 15, N'E7072', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a3aeda62-5f8d-49bf-b409-a2a7b8e21cc8', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED (maybe used for second bag of double or triple plateletpheresis)', N'PLT AFR PAS-C IRD LUKOPR', N'E024', N'@B5 C2 D2 H4', 200, N'12950', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a3e58541-9859-4b4c-bd27-28de04d5b886', N'RED BLOOD CELLS|SAGM/450mL/refg|ResLeu:<5log6|Plasma added', N'RBC SAGM LUKOPR PA', N'E002', N'@24 D4 E4', 350, N'E7045', 1008, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a41a2f76-fe22-4384-952b-131d409121fa', N'Washed Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Buffy coat plts prep|From 4 donors', N'PLT Pool Thaw Wash DMSO OPN BUFCT 4DNR', N'E027', N'@77 B2 G2 J4', 200, N'E7156', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a6bec947-93e4-444b-ae3b-0eee17c65e61', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Buffy coat plts prep|From 4 donors', N'PLT Pool Thaw DMSO OPN BUFCT 4DNR', N'E026', N'@77 B2 G2 J4', 200, N'E7082', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a7582789-5b7e-40ac-8bf8-0e9b82246927', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part H or 8)', N'PLASMA AFR Thaw None', N'E013', N'@AS HC', 225, N'49218', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a7aede0e-813c-4654-b491-ea79fa36f43e', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|Irradiated|1st container', N'PLT AFR ACD-A>PAS-C IRD', N'E024', N'@BH C2 H3', 200, N'E6998', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a81445bb-49d3-42ea-b3d8-68933325cb93', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part H or 8)', N'PLASMA AFR None', N'E012', N'@84 HC', 225, N'49218', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a87f0df9-d7f8-4951-9506-03384413e8b0', N'Frozen RED BLOOD CELLS|Gly40%/450mL/<-65C|Open|ResLeu:<1log6,WB filtr', N'RBC Froz Gly40% OPN LUKOPR', N'E004', N'@AT B2 DA', 200, N'E7036', 87600, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a932b1ef-14ee-4abf-9cd2-13b57eaed945', N'Frozen Apheresis PLATELETS|DMSO/XX/<-65C|Open|Irradiated|ResLeu:<5log6|2nd container', N'PLT AFR Froz DMSO OPN IRD LUKOPR', N'E025', N'@76 B2 C2 D4 H4', 200, N'E7015', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a9f0f123-9746-4965-8eb2-28e3137198eb', N'PLASMA|CPD/450mL/<-25C|Cryo reduced|Quar:>=112d/retested', N'PLASMA CPD CRYORed QUAR', N'E010', N'@A2 E3 I4', 225, N'E7074', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'a9fb5f78-3aa7-4e08-b4c6-94a4df0129d3', N'Deglycerolized RED BLOOD CELLS|None/450mL/refg|Open|ResLeu:<1log6,WB filtr', N'RBC Deg None OPN LUKOPR', N'E006', N'@17 B2 DA', 200, N'E7148', 24, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'aa526de5-c6c8-4ed2-9302-2e65f5dd6a35', N'PLASMA|CPD/350mL/<-30C|Cryo reduced', N'PLASMA CPD CRYORed', N'E010', N'@CZ E3', 225, N'E7038', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'aa68a4a9-5af6-49bc-9dc6-360daaec3ec4', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|1st container|Bacterial monitoring', N'PLT AFR PAS-E LUKOPR BacMtr', N'E024', N'@D5 D5 H3 N2', 200, N'E6744', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'aa810d1c-b26a-48b2-8709-75cf946fd54b', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced|1st container', N'PLT AFR PAS-E IRD LUKOPR SUPRed', N'E024', N'@D5 C2 D5 EA H3', 200, N'E6764', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'aab80f8d-4552-49c6-8dc3-f98083a2dc32', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Supernat rem/Plasma added|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-E OPN SR/PA BUFCT 2DNR', N'E022', N'@D5 B2 E8 G2 J2', 200, N'E6944', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ad4d4b37-8b9a-4d71-8a9d-fe4c227019cf', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|ResLeu:<5log6|Plasma reduced|Buffy coat plts prep|From 4 donors', N'PLT Pool CPD IRD LUKOPR PLS Red BUFCT 4DNR', N'E022', N'@A7 C2 D4 E5 G2 J4', 200, N'E6818', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'af66db6d-8d02-46fb-bc20-8eec29af3981', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-E OPN IRD SR/PA BUFCT 2DNR', N'E022', N'@D5 B2 C2 E8 G2 J2', 200, N'E6937', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b02caa73-2d39-4454-9361-a84870678431', N'POOLED PLATELETS|CPD/XX/20-24C|ResLeu:<5log6|Plasma reduced|Buffy coat plts prep|From 4 donors', N'PLT Pool CPD LUKOPR PLS Red BUFCT 4DNR', N'E022', N'@A7 D4 E5 G2 J4', 200, N'E6817', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b0f0b2b1-fae8-47f8-acf9-2635286e4060', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Plasma reduced/Plasma added|From 6 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR PLS Red/PA 6DNR', N'E026', N'@77 B2 C2 D5 EE J6', 200, N'E7146', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b0f59cbe-ba1a-4d8c-8de4-ca1234f15bf3', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Plasma reduced/Plasma added|From 7 donors', N'PLT Pool Thaw DMSO OPN PLS Red/PA 7DNR', N'E026', N'@77 B2 EE J7', 200, N'E7132', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b12ff0bd-268d-4385-a240-1f8b1e72a181', N'Washed Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool Thaw Wash DMSO OPN LUKOPR BUFCT 5DNR', N'E027', N'@77 B2 D5 G2 J5', 200, N'E7161', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b19430d9-3085-4393-9e23-00b08a653166', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Supernat rem/Plasma added|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-E OPN SR/PA BUFCT 6DNR', N'E022', N'@D5 B2 E8 G2 J6', 200, N'E6948', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b2853fbb-eb41-4c16-8df6-8aafa9361ff1', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-E OPN IRD SR/PA BUFCT 7DNR', N'E022', N'@D5 B2 C2 E8 G2 J7', 200, N'E6942', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b2c67266-2d78-4789-9f3e-14079ad76e19', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|From 4 donors', N'PLT Pool Froz DMSO OPN 4DNR', N'E025', N'@CP B2 J4', 200, N'E7099', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b3064096-0253-457a-9819-301f72cdf8d4', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED', N'PLT AFR Wash None OPN LUKOPR', N'E027', N'@71 B2 D2', 200, N'12910', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b37ce3c3-575a-449c-affb-4a88e1239b21', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|Supernat rem/Plasma added', N'PLT AFR PAS-E IRD SR/PA', N'E024', N'@D5 C2 E8', 200, N'E6876', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b4b2b7fa-441b-4ea9-904b-ebfd7361b731', N'Washed POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep', N'PLT Pool Wash PAS-E IRD LUKOPR BUFCT', N'E023', N'@D5 C2 D5 G2', 200, N'E6783', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b4c8632f-8455-470b-9654-ba692a29f749', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|2nd container', N'PLT AFR ACD-A>PAS-C', N'E024', N'@BH H4', 200, N'E6995', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b527a245-907f-47a3-a31c-e48cb7a3e6d7', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|2nd container|Psoralen-treated', N'PLT AFR Wash PAS-E LUKOPR Psoraln', N'E027', N'@D5 D5 H4 K5', 200, N'E7026', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b6c5c2eb-4c58-4e10-bde4-c074a789765d', N'Apheresis PLATELETS|ACD-A/XX/20-24C|Open|ResLeu:<5log6|Plasma reduced|<3log11 plts', N'PLT AFR ACD-A OPN LUKOPR PLS Red', N'E024', N'@73 B2 D4 E5 JI', 200, N'E6807', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b6ee3dbe-dfdd-4e3c-9711-0f4d06a30997', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|Irradiated|ResLeu:<5log6', N'PLT AFR ACD-A>PAS-C IRD LUKOPR', N'E024', N'@BH C2 D4', 200, N'E7005', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b7b41ac3-1736-4bdb-ba06-6bc0b7de57a2', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|Irradiated|2nd container', N'PLT AFR ACD-A>PAS-C IRD', N'E024', N'@BH C2 H4', 200, N'E6999', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'b8e990a0-5dc8-491d-bd0a-9ae97d796ecf', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED (for third bag of triple plateletpheresis)(variable contents: < 3X 1011)', N'PLT AFR PAS-C IRD LUKOPR', N'E024', N'@B5 C2 D2 H5 JI', 200, N'12961', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ba11079a-5674-45ba-a079-664eac2b751c', N'Apheresis FRESH FROZEN PLASMA|ACD-A/XX/<=-18C|Open|>=400mL<600mL', N'FFP AFR ACD-A OPN LowVol', N'E012', N'@47 B2 F6', 225, N'E7167', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'bb5384cd-de60-46e3-aca5-6547af293ce7', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-E OPN LUKOPR BUFCT 6DNR', N'E022', N'@D5 B2 D5 G2 J6', 200, N'E6976', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'bc0386e5-7370-42ff-8689-192ba45e1537', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED (for third bag of triple plateletpheresis)', N'PLT AFR Wash None IRD LUKOPR', N'E027', N'@71 B2 C2 D2 H5', 200, N'12960', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'bd787d76-5ed5-4445-a32a-5da06720f799', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-C IRD LUKOPR BUFCT 6DNR', N'E022', N'@B5 C2 D5 G2 J6', 200, N'E6709', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'bdbed804-11cc-405d-888e-52a7c875a5f8', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 2 donors', N'PLT Pool Wash None OPN LUKOPR BUFCT 2DNR', N'E023', N'@71 B2 D5 G2 J2', 200, N'E6834', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'bde2fe6a-c432-41da-851f-21a400eec015', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part C or 3)', N'PLASMA AFR None', N'E012', N'@84 H5', 225, N'49213', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'be7eae1b-ad72-4dee-bdc6-5823bf4d3338', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|2nd container|Psoralen-treated', N'PLT AFR PAS-E LUKOPR Psoraln', N'E024', N'@D5 D5 H4 K5', 200, N'E7018', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'be81a37b-b4b9-4c19-a428-500ee525140a', N'Washed Thawed Apheresis PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6', N'PLT AFR Thaw Wash DMSO OPN IRD LUKOPR', N'E027', N'@77 B2 C2 D5', 200, N'E7165', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'bef07d2b-fe1e-4105-a153-8be5f28bd5db', N'Apheresis PLATELETS|ACD-A/XX/20-24C|For mnf:noninjectable|ResLeu:<5log6|1st container', N'PLT AFR ACD-A MNFNI LUKOPR', N'E024', N'@73 A3 D4 H3', 200, N'E6650', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'bef5a795-c397-45aa-aa9e-e616b0db58ee', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|Buffy coat plts prep|From 3 donors', N'PLT Pool CPD OPN LUKOPR PLS Red/PA BUFCT 3DNR', N'E022', N'@A7 B2 D5 EE G2 J3', 200, N'E6910', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'bf218304-0985-446e-890e-50d1756fab5e', N'PLASMA|CPD/450mL/<-25C|Cryo reduced|Riboflavin-treated', N'PLASMA CPD CRYORed Ribo', N'E010', N'@A2 E3 K6', 225, N'E7076', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c1302255-ebc8-46eb-8b2a-638ae32b944f', N'Washed Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Buffy coat plts prep|From 5 donors', N'PLT Pool Thaw Wash DMSO OPN BUFCT 5DNR', N'E027', N'@77 B2 G2 J5', 200, N'E7157', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c141e52a-8eeb-4f28-b5ac-8a71aa189fef', N'FRESH FROZEN PLASMA|CPD/XX/<-30C|ResLeu:<5log6|Riboflavin-treated', N'FFP CPD LUKOPR Ribo', N'E010', N'@A4 D4 K6', 225, N'E7043', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c1de5f4f-7f4b-4440-9266-54aacd642ef0', N'Frozen Apheresis PLATELETS|None/XX/<=-18C|For mnf:noninjectable|ResLeu:<5log6|3rd container', N'PLT AFR Froz None MNFNI LUKOPR', N'E025', N'@84 A3 D4 H5', 200, N'E6811', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c2813625-0097-4027-9ae1-069449f56e91', N'POOLED PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-E LUKOPR BUFCT 2DNR', N'E022', N'@D5 D5 G2 J2', 200, N'E6858', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c3069684-c2ef-46a1-89d4-de886e07fffa', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|ResLeu:<1log6|From 3 donors', N'PLT Pool Froz DMSO OPN LUKOPR 3DNR', N'E025', N'@CP B2 D5 J3', 200, N'E7103', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c330cd23-0a70-4660-a709-e8ddb371e1ec', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool Froz DMSO LUKOPR BUFCT 4DNR', N'E025', N'@CP D5 G2 J4', 200, N'E7054', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c3761b1c-1817-4721-9240-81d69a550aa0', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Supernat reduced|4th container|Psoralen-treated', N'PLT AFR PAS-E LUKOPR SUPRed Psoraln', N'E024', N'@D5 D5 EA H6 K5', 200, N'E7024', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c394cb8e-f4c8-40a5-8ace-f965c71cd4e4', N'POOLED PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-D LUKOPR BUFCT 6DNR', N'E022', N'@CI D5 G2 J6', 200, N'E6668', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c4c54914-8f5e-4991-a36b-b104a53e5201', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Buffy coat plts prep|From 4 donors|Riboflavin-treated', N'PLT Pool CPD IRD BUFCT 4DNR Ribo', N'E022', N'@A7 C2 G2 J4 K6', 200, N'E6679', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c52c5d94-7fa2-4c61-8685-7f0f5c03fda1', N'LEUKOCYTES|CPD/500mL/<37C|For mnf:injectable', N'LEUK CPD MNFI', N'E038', N'@BI A2', 200, N'E7031', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c558376d-aad7-4b7d-8ff9-e82cfc2f1c96', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Plasma reduced/Plasma added|Buffy coat plts prep|From 6 donors', N'PLT Pool CPD OPN PLS Red/PA BUFCT 6DNR', N'E022', N'@A7 B2 EE G2 J6', 200, N'E6899', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c56d00ab-c593-4f09-89f0-c5085f79bcb2', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|From 6 donors', N'PLT Pool Thaw DMSO OPN IRD 6DNR', N'E026', N'@77 B2 C2 J6', 200, N'E7116', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c5d7d0b1-768a-4332-b93a-e9d9c51a18c7', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced|4th container', N'PLT AFR PAS-E IRD LUKOPR SUPRed', N'E024', N'@D5 C2 D5 EA H6', 200, N'E6767', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c6913964-b676-4ee5-8373-b37365198a20', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED (variable contents:<3 X log11 plts)', N'PLT AFR PAS-C LUKOPR', N'E024', N'@B5 D2 JI', 200, N'12911', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c69d7cb8-387d-49a8-bd6a-a2f08f358e56', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY (by pheresis)', N'PLASMA AFR Thaw None', N'E013', N'@AS', 225, N'18641', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c6dd8349-e99e-4f0c-82fa-d6dc29985fbf', N'PLASMA|CPDA-1/450mL/<-25C|Frozen >24h', N'PLASMA CPDA-1', N'E010', N'@CV G8', 225, N'E6826', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c7c3a567-db85-4950-821f-08956431b604', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container|Bacterial test', N'PLT AFR PAS-E IRD LUKOPR BacTest', N'E024', N'@D5 C2 D5 H5 N3', 200, N'E6758', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c88b1c62-eed5-480d-8a40-f7dd1f16ce0d', N'PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Supernat rem/Plasma added', N'PLT PAS-E OPN LUKOPR SR/PA', N'E020', N'@D5 B2 D5 E8', 50, N'E6879', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c8c9f308-1d53-4f1b-b17a-148c5877d9d8', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool CPD OPN LUKOPR PLS Red/PA BUFCT 4DNR', N'E022', N'@A7 B2 D5 EE G2 J4', 200, N'E6911', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c9302e77-e2ff-4030-af28-7ac62a9d98db', N'FRESH FROZEN PLASMA|CPD/450mL/<-25C|Frozen <=15h|Riboflavin-treated', N'FFP CPD Ribo', N'E010', N'@A2 G5 K6', 225, N'E7060', 8760, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c93778c7-182e-403b-a7d9-1c87f2ac731d', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-E IRD SR/PA BUFCT 3DNR', N'E022', N'@D5 C2 E8 G2 J3', 200, N'E6931', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'c9be32a1-8fe7-48f8-a11d-59ef72cb106b', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Buffy coat plts prep|From 8 donors|Riboflavin-treated', N'PLT Pool CPD IRD BUFCT 8DNR Ribo', N'E022', N'@A7 C2 G2 J8 K6', 200, N'E6683', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ca211458-ed8f-4c39-a526-9e0997fffa12', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool Froz DMSO LUKOPR BUFCT 6DNR', N'E025', N'@CP D5 G2 J6', 200, N'E7056', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ca780d74-a69d-4d90-b439-894f894f80d8', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED', N'PLT AFR PAS-C LUKOPR', N'E024', N'@B5 D2', 200, N'12910', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'cabbbf88-db97-42db-8feb-b8638d6e3bc4', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Plasma reduced/Plasma added|Buffy coat plts prep|From 7 donors', N'PLT Pool CPD OPN PLS Red/PA BUFCT 7DNR', N'E022', N'@A7 B2 EE G2 J7', 200, N'E6900', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'cb0bde4f-bd48-4e0a-b502-125e932e4daf', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED (for third bag of triple plateletpheresis)', N'PLT AFR Wash None OPN LUKOPR', N'E027', N'@71 B2 D2 H5', 200, N'12930', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'cb69acd3-dcad-4c66-bdf9-1d6e3b9904be', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR BUFCT 4DNR', N'E026', N'@77 B2 C2 D5 G2 J4', 200, N'E7088', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'cb762690-cfd4-4aba-8a8e-03d889ac7394', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part F or 6)', N'PLASMA AFR None', N'E012', N'@84 HA', 225, N'49216', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'cc939073-82ac-4bb8-a2fc-0dc74562c5c2', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part C or 3)', N'PLASMA AFR None IRD ', N'E012', N'@84 C2 H5', 225, N'49613', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'cd11b9c4-7795-4ea0-8335-9ec73f61ab92', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-E IRD LUKOPR BUFCT 8DNR', N'E022', N'@D5 C2 D5 G2 J8', 200, N'E6957', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'cdee7fc1-4dcb-48cf-92ff-3d9126374111', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Plasma reduced/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool Thaw DMSO OPN PLS Red/PA BUFCT 4DNR', N'E026', N'@77 B2 EE G2 J4', 200, N'E7090', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ce2c9418-a1fb-4143-92d3-4bab4e3dc053', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|2nd container|Bacterial monitoring', N'PLT AFR PAS-E IRD LUKOPR BacMtr', N'E024', N'@D5 C2 D5 H4 N2', 200, N'E6749', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ce5b930a-7f58-440d-a06b-01c2c71e4831', N'Apheresis PLASMA|CP2D/XX/<=-18C|Not for tx or mnf', N'PLASMA AFR CP2D NMT', N'E012', N'@37 A4', 225, N'E6803', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ce8a45fc-4349-4c6f-b09b-f5c85b4c1cbb', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool CPD OPN LUKOPR BUFCT 4DNR', N'E022', N'@A7 B2 D5 G2 J4', 200, N'E6890', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ce95d202-9d84-40cb-8b19-73001ff855d5', N'POOLED PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Supernat reduced|Buffy coat plts prep', N'PLT Pool PAS-E LUKOPR SUPRed BUFCT', N'E022', N'@D5 D5 EA G2', 200, N'E6780', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'cf5477a7-6fad-4b32-a859-ab9b3e8102a1', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|Buffy coat plts prep|From 4 donors', N'PLT Pool Froz DMSO OPN BUFCT 4DNR', N'E025', N'@CP B2 G2 J4', 200, N'E7078', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'cf9e0cb3-c2b4-4149-851f-3dbb35ad1dbe', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 6 donors', N'PLT Pool CPD IRD PLS Red/PA BUFCT 6DNR', N'E022', N'@A7 C2 EE G2 J6', 200, N'E6674', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd03d5eca-33f9-4144-8102-b841f689b4e2', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part B or 2)', N'PLASMA AFR None', N'E012', N'@84 H4', 225, N'49212', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd07a4696-e43f-49c2-8ba3-a7e269fbd153', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|Irradiated|ResLeu:<5log6|1st container', N'PLT AFR ACD-A>PAS-C IRD LUKOPR', N'E024', N'@BH C2 D4 H3', 200, N'E7006', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd350cfd1-cac6-43c1-86df-270eddc1f59c', N'POOLED PLATELETS|PAS-E/XX/20-24C|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-E BUFCT 2DNR', N'E022', N'@D5 G2 J2', 200, N'E6851', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd378ec43-9820-46a7-80f7-ace3515502aa', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-E OPN LUKOPR SR/PA BUFCT 7DNR', N'E022', N'@D5 B2 D5 E8 G2 J7', 200, N'E6991', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd38a4dd1-893d-4cb3-949e-91cbba944845', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-E OPN IRD SR/PA BUFCT 5DNR', N'E022', N'@D5 B2 C2 E8 G2 J5', 200, N'E6940', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd39ef13c-7739-4aee-84f6-04824bd7d5ab', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Supernat reduced|2nd container|Psoralen-treated', N'PLT AFR PAS-E LUKOPR SUPRed Psoraln', N'E024', N'@D5 D5 EA H4 K5', 200, N'E7022', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd3eaa796-8e59-4308-979d-74045d814725', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-C IRD SR/PA BUFCT 5DNR', N'E022', N'@B5 C2 E8 G2 J5', 200, N'E6715', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd44048d5-a561-436e-917f-dcaf1b925bb0', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 7 donors|Riboflavin-treated', N'PLT Pool CPD IRD LUKOPR BUFCT 7DNR Ribo', N'E022', N'@A7 C2 D5 G2 J7 K6', 200, N'E6696', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd44320d8-b56a-410f-9bf8-3942ddb4e0f3', N'Washed Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|3rd container|Psoralen-treated', N'PLT AFR Wash PAS-E LUKOPR Psoraln', N'E027', N'@D5 D5 H5 K5', 200, N'E7027', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd45fdc53-a0bb-474f-bcca-afcf23e40867', N'Apheresis PLATELETS|ACD-A/XX/20-24C|For mnf:noninjectable|ResLeu:<5log6|2nd container', N'PLT AFR ACD-A MNFNI LUKOPR', N'E024', N'@73 A3 D4 H4', 200, N'E6651', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd4ac99a9-7f3a-4847-a831-4348306e9ebd', N'Thawed Apheresis PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Riboflavin-treated', N'PLT AFR Thaw DMSO OPN IRD LUKOPR Ribo', N'E026', N'@77 B2 C2 D5 K6', 200, N'E7065', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd4fcf3cc-36f3-4915-b1ab-bb49bfd9929b', N'POOLED PLATELETS|CPD/XX/20-24C|Buffy coat plts prep|From 2 donors', N'PLT Pool CPD BUFCT 2DNR', N'E022', N'@A7 G2 J2', 200, N'E6882', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd5d6d730-7087-475a-a68f-03fffe868c56', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-C IRD SR/PA BUFCT 4DNR', N'E022', N'@B5 C2 E8 G2 J4', 200, N'E6714', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd6671e37-6323-4c79-97d0-1e64ba8ebbec', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|ResLeu:<1log6|Buffy coat plts prep|From 4 donors|Riboflavin-treated', N'PLT Pool Froz DMSO OPN LUKOPR BUFCT 4DNR Ribo', N'E025', N'@CP B2 D5 G2 J4 K6', 200, N'E7066', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd7228a57-bffc-4021-91da-d59f0843272f', N'POOLED PLATELETS|CPD/XX/20-24C|ResLeu:<1log6|Plasma reduced|Buffy coat plts prep|From 4 donors', N'PLT Pool CPD LUKOPR PLS Red BUFCT 4DNR', N'E022', N'@A7 D5 E5 G2 J4', 200, N'E7154', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd7839b1d-4b4d-4004-9498-1eccf8d80eec', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-E IRD BUFCT 6DNR', N'E022', N'@D5 C2 G2 J6', 200, N'E6927', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd7c23d53-c943-40d6-ba06-79ea315680a0', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|ResLeu:<1log6|From 6 donors', N'PLT Pool Froz DMSO OPN LUKOPR 6DNR', N'E025', N'@CP B2 D5 J6', 200, N'E7106', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd7e4343d-19f3-421c-be7e-68c29013be8e', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Plasma reduced/Plasma added|From 4 donors', N'PLT Pool Thaw DMSO OPN PLS Red/PA 4DNR', N'E026', N'@77 B2 EE J4', 200, N'E7129', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'd8add2dd-d155-4bd8-9253-eef972c2076e', N'PLASMA|CP2D/XX/<=-20C|For mnf:injectable restr use|Frozen <=24h', N'PLASMA CP2D MNFRI', N'E010', N'@BS A6 G7', 225, N'E6841', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'da05c735-fb24-4525-9eb4-d03f59a97ff3', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|From 6 donors', N'PLT Pool Thaw DMSO OPN LUKOPR 6DNR', N'E026', N'@77 B2 D5 J6', 200, N'E7121', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'da1a7d7c-e1e5-4da4-aada-1e1e4f7f8f7a', N'POOLED PLATELETS|PAS-E/XX/20-24C|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-E BUFCT 7DNR', N'E022', N'@D5 G2 J7', 200, N'E6856', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'da64205d-4f65-4c2a-98cc-a57981274885', N'POOLED PLATELETS|PAS-E/XX/20-24C|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-E BUFCT 5DNR', N'E022', N'@D5 G2 J5', 200, N'E6854', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'da6e6f5e-8431-45b1-8190-57ff2db13a8b', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat reduced|Buffy coat plts prep', N'PLT Pool PAS-E IRD LUKOPR SUPRed BUFCT', N'E022', N'@D5 C2 D5 EA G2', 200, N'E6781', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'daad0f60-e83e-43ee-b814-684a42caa930', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-E IRD LUKOPR SR/PA BUFCT 8DNR', N'E022', N'@D5 C2 D5 E8 G2 J8', 200, N'E6964', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'daef18ad-4ef3-4a64-b65c-f79350c34741', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-E OPN LUKOPR BUFCT 3DNR', N'E022', N'@D5 B2 D5 G2 J3', 200, N'E6973', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'db8266e3-d8f6-436e-b036-8120039ed13a', N'Liquid Apheresis PLASMA|NaCitrate/XX/<37C|For mnf:noninjectable|Open', N'PLASMA AFR Liq NaCit MNFNI OPN', N'E011', N'@D6 A3 B2', 225, N'E6870', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'dbdf0c94-371d-4766-897d-5ae75b47ad29', N'PLATELETS|CP2D/500mL/20-24C|Open|Plasma reduced', N'PLT CP2D OPN PLS Red', N'E020', N'@69 B2 E5', 50, N'E6805', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'dc2d90ab-5f46-4452-a3e8-8007c18f34f1', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-E OPN LUKOPR SR/PA BUFCT 5DNR', N'E022', N'@D5 B2 D5 E8 G2 J5', 200, N'E6989', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'dc49ef02-b638-43b1-889f-20ee58900620', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|3rd container|Bacterial test', N'PLT AFR PAS-E LUKOPR BacTest', N'E024', N'@D5 D5 H5 N3', 200, N'E6754', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'dc683b42-d659-499c-a181-8d6117217056', N'Thawed POOLED CRYOPRECIPITATE|CP2D/XX/rt|Open|ResLeu:NS', N'CRYO Pool Thaw CP2D OPN LUKOPR', N'E031', N'@CU B2 D2', 100, N'E6815', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'dcbac751-35bd-45dc-bc90-d30fdcb657cc', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool PAS-E IRD LUKOPR BUFCT 6DNR', N'E022', N'@D5 C2 D5 G2 J6', 200, N'E6955', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ddd5e534-b7fc-473a-ba7e-2ee0a98a7ed4', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|1st container|Bacterial test', N'PLT AFR PAS-E IRD LUKOPR BacTest', N'E024', N'@D5 C2 D5 H3 N3', 200, N'E6756', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'de3897be-6853-44b7-9d89-ffa36c261550', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-E IRD LUKOPR BUFCT 2DNR', N'E022', N'@D5 C2 D5 G2 J2', 200, N'E6951', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'de752c40-d789-4ba5-8fde-af6177c775e0', N'RED BLOOD CELLS|CPDA-1/500mL/refg|ResLeu:<5log6|Plasma reduced', N'RBC CPDA-1 LUKOPR PLS Red', N'E002', N'@08 D4 E5', 250, N'E7035', 840, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'dedf8470-3a02-45cc-9257-c0e8f44db49c', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 6 donors|Riboflavin-treated', N'PLT Pool CPD IRD LUKOPR BUFCT 6DNR Ribo', N'E022', N'@A7 C2 D5 G2 J6 K6', 200, N'E6695', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e052ef26-d4c8-49ee-a250-0c45b29f8037', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool CPD OPN LUKOPR PLS Red/PA BUFCT 5DNR', N'E022', N'@A7 B2 D5 EE G2 J5', 200, N'E6912', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e1c55174-917a-485c-aea9-1f25bc6889d9', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|From 4 donors', N'PLT Pool Thaw DMSO OPN IRD 4DNR', N'E026', N'@77 B2 C2 J4', 200, N'E7114', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e1f6c6c2-6705-4b34-87a6-4197d4f62471', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-C IRD LUKOPR SR/PA BUFCT 3DNR', N'E022', N'@B5 C2 D5 E8 G2 J3', 200, N'E6720', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e2fb1c51-7c2f-4238-b975-6780c56dc3b9', N'PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part E or 5)', N'PLASMA AFR None', N'E012', N'@84 H9', 225, N'49215', 8760, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e2fc8780-7659-4250-bd39-9dc47d48743e', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|Irradiated|Buffy coat plts prep|From 4 donors', N'PLT Pool Wash None OPN IRD BUFCT 4DNR', N'E023', N'@71 B2 C2 G2 J4', 200, N'E6829', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e3eb0b60-06c6-4859-993c-043af8e61c97', N'RED BLOOD CELLS|CP2D>AS3/500mL/refg|Open|ResLeu:<5log6|Plasma added', N'RBC CP2D>AS3 OPN LUKOPR PA', N'E002', N'@21 B2 D4 E4', 350, N'E6785', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e415dcaa-af0d-4258-b918-be2177287b07', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|For mnf:injectable|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool Froz DMSO MNFI LUKOPR BUFCT 5DNR', N'E025', N'@CP A2 D5 G2 J5', 200, N'E7052', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e4368771-4ed2-45c6-a70a-412db16915f9', N'Apheresis PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|3rd container', N'PLT AFR PAS-E IRD LUKOPR', N'E024', N'@D5 C2 D5 H5', 200, N'E6742', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e47016b8-14d2-494b-9be6-94cc0ffa8af7', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 8 donors', N'PLT Pool Wash None OPN LUKOPR BUFCT 8DNR', N'E023', N'@71 B2 D5 G2 J8', 200, N'E6840', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e4e3b50a-1c55-43e9-880f-a5ec5f160911', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Plasma reduced/Plasma added|From 3 donors', N'PLT Pool Thaw DMSO OPN PLS Red/PA 3DNR', N'E026', N'@77 B2 EE J3', 200, N'E7128', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e530c1ec-d5b9-48f2-9c52-4ec566c85512', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|2nd container', N'PLT AFR PAS-E LUKOPR', N'E024', N'@D5 D5 H4', 200, N'E6737', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e56b14d4-7413-4b45-a28f-907d343dcb6e', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool CPD OPN IRD PLS Red/PA BUFCT 5DNR', N'E022', N'@A7 B2 C2 EE G2 J5', 200, N'E6905', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e57c38c0-02bb-4038-9c29-3a93b14bd4a7', N'Apheresis RED BLOOD CELLS|ACD-A>AS3/XX/refg|Open|ResLeu:<5log6|Plasma added', N'RBC AFR ACD-A>AS3 OPN LUKOPR PA', N'E009', N'@AZ B2 D4 E4', 350, N'E6788', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e709c080-450f-455c-b47b-635ebb5ad895', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|From 5 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR 5DNR', N'E026', N'@77 B2 C2 D5 J5', 200, N'E7125', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e729488e-ab2a-45f1-b176-61c195bd1803', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool Thaw DMSO OPN IRD PLS Red/PA BUFCT 5DNR', N'E026', N'@77 B2 C2 EE G2 J5', 200, N'E7093', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e827e083-6409-4960-9f26-01187b5bbe8b', N'Apheresis PLATELETS|ACD-A/XX/20-24C|Irradiated|Supernat rem/Plasma added', N'PLT AFR ACD-A IRD SR/PA', N'E024', N'@73 C2 E8', 200, N'E6733', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e84b9a66-bd74-44cd-912e-6cb2d578c3b7', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Supernat reduced|1st container|Riboflavin-treated', N'PLT AFR PAS-E LUKOPR SUPRed Ribo', N'E024', N'@D5 D5 EA H3 K6', 200, N'E7039', 120, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e89b6a8a-2fae-467b-a506-6508e8d0bea1', N'PLASMA|CPD/450mL/<=-18C|For mnf:noninjectable', N'PLASMA CPD MNFNI', N'E010', N'@AM A3', 225, N'E6850', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e8e394c1-94fe-41ee-8330-f7bce5312f17', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-E IRD BUFCT 4DNR', N'E022', N'@D5 C2 G2 J4', 200, N'E6925', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e93dc7e6-8893-4c0d-bcce-45ebaa4a39b8', N'Apheresis RED BLOOD CELLS|ACD-A>AS3/XX/refg|Open|Irradiated|ResLeu:<5log6|Plasma added|1st container', N'RBC AFR ACD-A>AS3 OPN IRD LUKOPR PA', N'E009', N'@AZ B2 C2 D4 E4 H3', 350, N'E6791', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e9790fbd-0a12-4744-a09e-0dc3888e9393', N'Frozen Apheresis PLATELETS|None/XX/<=-18C|For mnf:noninjectable|ResLeu:<5log6|2nd container', N'PLT AFR Froz None MNFNI LUKOPR', N'E025', N'@84 A3 D4 H4', 200, N'E6810', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e9e3cfd9-9473-477b-9a23-a5fedef02545', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED (may be used for second bag of double or triple plateletpheresis)(variable contents:<3 X log11 plts)', N'PLT AFR Wash None OPN LUKOPR', N'E027', N'@71 B2 D2 H4 JI', 200, N'12921', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'e9f05919-aec4-4791-acc7-56ec60d47fb5', N'Frozen Apheresis PLATELETS|DMSO/XX/<-65C|Open|Irradiated|ResLeu:<5log6', N'PLT AFR Froz DMSO OPN IRD LUKOPR', N'E025', N'@76 B2 C2 D4', 200, N'E7013', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ea93c208-2b72-44f8-85a3-967d6b15eebc', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Plasma reduced/Plasma added|Buffy coat plts prep|From 8 donors', N'PLT Pool CPD IRD PLS Red/PA BUFCT 8DNR', N'E022', N'@A7 C2 EE G2 J8', 200, N'E6676', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'eb0a85bb-ce8e-4565-addf-8f80c93c3fba', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 5 donors', N'PLT Pool PAS-E IRD LUKOPR SR/PA BUFCT 5DNR', N'E022', N'@D5 C2 D5 E8 G2 J5', 200, N'E6961', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'eb57fcf1-a056-4065-b460-a6340a37745c', N'POOLED PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-E LUKOPR BUFCT 7DNR', N'E022', N'@D5 D5 G2 J7', 200, N'E6863', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'eb5a9003-6469-4098-9c95-d1b8b2b17942', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|Irradiated|ResLeu:<5log6|<3log11 plts', N'PLT AFR ACD-A>PAS-C IRD LUKOPR', N'E024', N'@BH C2 D4 JI', 200, N'E7012', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ebac12a9-4cc9-4e8e-aef8-eb73aafb4881', N'Apheresis FRESH FROZEN PLASMA|NaCitrate/XX/<-25C|For mnf:injectable|Frozen <=24h|Quar:>=112d/retested', N'FFP AFR NaCit MNFI QUAR', N'E012', N'@CQ A2 G7 I4', 225, N'E6821', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ec0af8b7-711f-4078-ad11-f214580c92b3', N'PLASMA|CPD/450mL/<=-18C|Cryo reduced|Quar:>=112d/retested', N'PLASMA CPD CRYORed QUAR', N'E010', N'@AM E3 I4', 225, N'E6660', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ec54a5e8-7669-42de-9df9-045153d760a8', N'PLASMA|CPDA-1/450mL/<=-18C|Cryo reduced', N'PLASMA CPDA-1 CRYORed', N'E010', N'@BK E3', 225, N'E6661', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ed398902-253c-4345-86fe-5757d8e8ced7', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|From 4 donors', N'PLT Pool Thaw DMSO OPN 4DNR', N'E026', N'@77 B2 J4', 200, N'E7109', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ed76f7e9-09bd-4712-bad3-9710ec85a7ee', N'Washed Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|Buffy coat plts prep|From 4 donors', N'PLT Pool Thaw Wash DMSO OPN IRD BUFCT 4DNR', N'E027', N'@77 B2 C2 G2 J4', 200, N'E7158', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'edb351d1-22de-4c12-baa7-8e272b33a8ba', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-E IRD LUKOPR SR/PA BUFCT 4DNR', N'E022', N'@D5 C2 D5 E8 G2 J4', 200, N'E6960', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'edb53585-7ae8-409f-892f-61c80fbc7e26', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED', N'PLT AFR PAS-C IRD LUKOPR', N'E024', N'@B5 C2 D2', 200, N'12940', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'edea1207-bca1-4ade-ad3d-659365362264', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-E IRD BUFCT 7DNR', N'E022', N'@D5 C2 G2 J7', 200, N'E6928', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'edf3d1a7-e10c-4a50-b1ea-05ae2bee375a', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|From 6 donors', N'PLT Pool Thaw DMSO OPN 6DNR', N'E026', N'@77 B2 J6', 200, N'E7111', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ee457878-d1c5-4a29-a403-19d3d2683069', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-E OPN LUKOPR SR/PA BUFCT 8DNR', N'E022', N'@D5 B2 D5 E8 G2 J8', 200, N'E6992', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ee75cfa4-39cd-4b7d-badf-57ef2ce5745b', N'Washed Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|Buffy coat plts prep|From 5 donors', N'PLT Pool Thaw Wash DMSO OPN IRD BUFCT 5DNR', N'E027', N'@77 B2 C2 G2 J5', 200, N'E7159', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'eeacd5b2-6cb7-4b77-a6cb-5b08e698cc61', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|From 5 donors', N'PLT Pool Thaw DMSO OPN IRD 5DNR', N'E026', N'@77 B2 C2 J5', 200, N'E7115', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'eed60c30-214e-4842-a4d8-65f1755d718c', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-C IRD BUFCT 4DNR', N'E022', N'@B5 C2 G2 J4', 200, N'E6700', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ef0b896d-ea93-438e-960e-546a7e4e3526', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-E IRD BUFCT 2DNR', N'E022', N'@D5 C2 G2 J2', 200, N'E6923', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f00a3fb8-b68c-4d3b-a8ca-d1a06f830f66', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED (variable contents:<3 X log11 plts)', N'PLT AFR Wash None OPN LUKOPR', N'E027', N'@71 B2 D2 JI', 200, N'12911', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f02606e7-1bd4-460e-89e9-97dd301fa844', N'Thawed Apheresis PLASMA|ACD-A/XX/refg|Open|2nd container', N'PLASMA AFR Thaw ACD-A OPN', N'E013', N'@54 B2 H4', 225, N'E6800', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f12f185d-de87-444e-ae06-c3d072bb49f9', N'Thawed Apheresis PLASMA|ACD-A/XX/refg|Open|1st container', N'PLASMA AFR Thaw ACD-A OPN', N'E013', N'@54 B2 H3', 225, N'E6799', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f163b672-623e-48b2-8550-739c2e6cbc13', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED IRRADIATED (by pheresis)(-18C or colder)(Part D or 4)', N'PLASMA AFR Thaw None IRD ', N'E013', N'@AS C2 H6', 225, N'49614', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f1b219c0-a46f-448d-a571-f82760cb6c18', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 7 donors', N'PLT Pool PAS-C IRD LUKOPR SR/PA BUFCT 7DNR', N'E022', N'@B5 C2 D5 E8 G2 J7', 200, N'E6724', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f1cc2e8f-10e3-49ae-a9c2-608152b65566', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 3 donors', N'PLT Pool Wash None OPN LUKOPR BUFCT 3DNR', N'E023', N'@71 B2 D5 G2 J3', 200, N'E6835', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f2bd18cf-8e89-48de-9043-f7ed196bf3bf', N'Thawed Apheresis PLASMA|ACD-A/XX/refg|Open|4th container', N'PLASMA AFR Thaw ACD-A OPN', N'E013', N'@54 B2 H6', 225, N'E6802', 24, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f2d1af7c-7a8f-4c52-9563-fb3feafe9224', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED IRRADIATED', N'PLT AFR Wash None IRD LUKOPR', N'E027', N'@71 B2 C2 D2', 200, N'12940', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f385b2fd-4d20-405e-b992-ac5da6cae9c0', N'Apheresis PLATELETS|ACD-A/XX/20-24C|For mnf:noninjectable|ResLeu:<5log6|3rd container', N'PLT AFR ACD-A MNFNI LUKOPR', N'E024', N'@73 A3 D4 H5', 200, N'E6652', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f45e6b7a-2b13-445a-9c3d-a12471ec4322', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|Irradiated|ResLeu:<5log6|2nd container', N'PLT AFR ACD-A>PAS-C IRD LUKOPR', N'E024', N'@BH C2 D4 H4', 200, N'E7007', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f46a43dc-1ce0-4c78-8836-69538bedcdb7', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 5 donors', N'PLT Pool Thaw DMSO LUKOPR BUFCT 5DNR', N'E026', N'@77 D5 G2 J5', 200, N'E7058', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f4b3a0b8-ce3b-4937-a268-8828b168d0a6', N'POOLED PLATELETS|CPD/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 2 donors', N'PLT Pool CPD OPN LUKOPR BUFCT 2DNR', N'E022', N'@A7 B2 D5 G2 J2', 200, N'E6888', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f51a508d-04b9-41f2-880d-f43ff940a717', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|From 7 donors', N'PLT Pool Thaw DMSO OPN 7DNR', N'E026', N'@77 B2 J7', 200, N'E7112', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f568cf02-eb96-47d3-a1b6-f7b6efd1016a', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Plasma reduced/Plasma added|From 3 donors', N'PLT Pool Thaw DMSO OPN IRD LUKOPR PLS Red/PA 3DNR', N'E026', N'@77 B2 C2 D5 EE J3', 200, N'E7143', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f5f0742c-3ab9-4a1b-8def-c3c22ac8db2e', N'PLASMA|CPDA-1/450mL/<=-18C|Cryo reduced|Quar:>=112d/retested', N'PLASMA CPDA-1 CRYORed QUAR', N'E010', N'@BK E3 I4', 225, N'E6662', 8760, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f633c1c4-560f-48fa-9c37-f2d5df9bcd11', N'POOLED PLATELETS|CPD/XX/20-24C|Open|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 8 donors', N'PLT Pool CPD OPN IRD LUKOPR BUFCT 8DNR', N'E022', N'@A7 B2 C2 D5 G2 J8', 200, N'E6922', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f6792a00-9818-473a-a4d6-5c72f215a892', N'PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED (for third bag of triple plateletpheresis)', N'PLT AFR PAS-C LUKOPR', N'E024', N'@B5 D2 H5', 200, N'12930', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f6868bdd-78b4-495a-88ec-4249472676d5', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Supernat rem/Plasma added|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-E OPN LUKOPR SR/PA BUFCT 3DNR', N'E022', N'@D5 B2 D5 E8 G2 J3', 200, N'E6987', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f6fbff90-cba0-454b-b4f9-2d3726d5534b', N'POOLED PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep', N'PLT Pool PAS-E LUKOPR BUFCT', N'E022', N'@D5 D5 G2', 200, N'E6776', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f7283136-c0ce-4ff3-889c-f777ceb7d3fb', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Supernat rem/Plasma added|Buffy coat plts prep|From 8 donors', N'PLT Pool PAS-C IRD SR/PA BUFCT 8DNR', N'E022', N'@B5 C2 E8 G2 J8', 200, N'E6718', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f773170b-efa1-4dd2-8091-96c20a60fa57', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6', N'PLT AFR PAS-E LUKOPR', N'E024', N'@D5 D5', 200, N'E6874', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f783f694-7d9f-4f3b-927f-55cdd6e9271c', N'Apheresis PLATELETS|ACD-A>PAS-C/XX/20-24C|Irradiated', N'PLT AFR ACD-A>PAS-C IRD', N'E024', N'@BH C2', 200, N'E6997', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f7d7fc39-a0b5-46fd-95cc-f1a4326a909c', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|Buffy coat plts prep|From 3 donors|Riboflavin-treated', N'PLT Pool CPD IRD BUFCT 3DNR Ribo', N'E022', N'@A7 C2 G2 J3 K6', 200, N'E6678', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f7ef361d-3f64-4640-9e24-e5ac436fb651', N'POOLED PLATELETS|PAS-E/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 4 donors', N'PLT Pool PAS-E OPN LUKOPR BUFCT 4DNR', N'E022', N'@D5 B2 D5 G2 J4', 200, N'E6974', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f8e6cf9d-3e9b-4f44-989b-8e6adecb1381', N'POOLED PLATELETS|CPD/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 2 donors|Riboflavin-treated', N'PLT Pool CPD IRD LUKOPR BUFCT 2DNR Ribo', N'E022', N'@A7 C2 D5 G2 J2 K6', 200, N'E6691', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f906a7a3-6479-4275-8867-4681fb22dc78', N'POOLED PLATELETS|CPD/XX/20-24C|ResLeu:<1log6|Buffy coat plts prep|From 2 donors|Riboflavin-treated', N'PLT Pool CPD LUKOPR BUFCT 2DNR Ribo', N'E022', N'@A7 D5 G2 J2 K6', 200, N'E6684', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f9819088-ba54-4c19-b4f0-5401981d43ba', N'Washed PAS-C PLATELETS PHERESIS LEUKOCYTES REDUCED (may be used for second bag of double or triple plateletpheresis)', N'PLT AFR Wash None OPN LUKOPR', N'E027', N'@71 B2 D2 H4', 200, N'12920', 4, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'f9ee0693-a6e9-4438-a0a0-333295b0d4e3', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|Supernat reduced|2nd container', N'PLT AFR PAS-E LUKOPR SUPRed', N'E024', N'@D5 D5 EA H4', 200, N'E6761', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'fa494d96-5efd-4d85-87ae-392d175532c5', N'Apheresis PLATELETS|PAS-D/XX/20-24C|ResLeu:<1log6', N'PLT AFR PAS-D LUKOPR', N'E024', N'@CI D5', 200, N'E6664', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'fa6516c4-3b37-44ac-a7d4-1d54dbee4ac8', N'POOLED PLATELETS|CP2D/XX/20-24C|ResLeu:<5log6|From 8 donors|Bacterial monitoring', N'PLT Pool CP2D LUKOPR 8DNR BacMtr', N'E022', N'@B9 D4 J8 N2', 200, N'E7153', 120, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'fbf90cca-f2da-46be-895f-4fec90f5e661', N'Washed POOLED PLATELETS|None/XX/20-24C|Open|ResLeu:<1log6|Buffy coat plts prep|From 6 donors', N'PLT Pool Wash None OPN LUKOPR BUFCT 6DNR', N'E023', N'@71 B2 D5 G2 J6', 200, N'E6838', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'fd73f4fd-afd3-41c4-80f6-ce777fa814bf', N'Frozen POOLED PLATELETS|DMSO/XX/<-80C|Open|From 3 donors', N'PLT Pool Froz DMSO OPN 3DNR', N'E025', N'@CP B2 J3', 200, N'E7098', 87600, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'fe600f58-48d3-42e8-b9ed-484786f122e8', N'POOLED PLATELETS|PAS-C/XX/20-24C|Irradiated|Buffy coat plts prep|From 2 donors', N'PLT Pool PAS-C IRD BUFCT 2DNR', N'E022', N'@B5 C2 G2 J2', 200, N'E6698', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'fe7ce130-0c5e-4efd-aad7-9723da18b541', N'POOLED PLATELETS|CPD/XX/20-24C|Buffy coat plts prep|From 7 donors', N'PLT Pool CPD BUFCT 7DNR', N'E022', N'@A7 G2 J7', 200, N'E6886', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'fe813de3-92e6-4788-8ede-3b06f34bd665', N'Thawed PLASMA FROZEN WITHIN 24 HOURS AFTER PHLEBOTOMY HELD AT ROOM TEMPERATURE UP TO 24 HRS AFTER PHLEBOTOMY DIVIDED (by pheresis)(-18C)(Part B or 2)', N'PLASMA AFR Thaw None', N'E013', N'@AS H4', 225, N'49212', 120, 0, 0, 0, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'feca6a26-dce9-4cd3-bb00-ca7eabee3d65', N'POOLED PLATELETS|PAS-E/XX/20-24C|Irradiated|ResLeu:<1log6|Buffy coat plts prep|From 3 donors', N'PLT Pool PAS-E IRD LUKOPR BUFCT 3DNR', N'E022', N'@D5 C2 D5 G2 J3', 200, N'E6952', 4, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ff788458-2c05-4835-a3bd-2c8c86c84c9b', N'Apheresis PLATELETS|PAS-E/XX/20-24C|ResLeu:<1log6|3rd container', N'PLT AFR PAS-E LUKOPR', N'E024', N'@D5 D5 H5', 200, N'E6738', 120, 0, 0, 1, NULL, N'A', '20100819 08:27:00.000', N'vhaishsmithm', 0)
INSERT INTO dbo.BloodProduct (BloodProductGuid, ProductName, ProductShortName, ProductTypeCode, ProductAttributeForm, ProductVolume, ProductCode, MaximumStorageTime, CrossmatchRequiredIndicator, ProductModificationIndicator, ProductCodeIndicator, RetireDate, RecordStatusCode, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'ffe06f1e-3ca1-4f7b-80b9-f667f4c52001', N'Thawed POOLED PLATELETS|DMSO/XX/20-24C|Open|ResLeu:<1log6|Plasma reduced/Plasma added|From 4 donors', N'PLT Pool Thaw DMSO OPN LUKOPR PLS Red/PA 4DNR', N'E026', N'@77 B2 D5 EE J4', 200, N'E7139', 4, 0, 0, 1, NULL, N'A', '20101201 15:05:41.750', N'vhaishsmithm', 0)

-- Add 3 rows to dbo.ProductAttribute
INSERT INTO dbo.ProductAttribute (ProductAttributeId, ProductAttributeText, ProductAttributePrintName, ModificationAllowedIndicator, RetireDate, SortColumn, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'@D5', N'PAS-E/XX/20-24C', N'PAS-E', 0, NULL, 0, '20100820 14:52:17.000', N'vhaishsmithm', 0)
INSERT INTO dbo.ProductAttribute (ProductAttributeId, ProductAttributeText, ProductAttributePrintName, ModificationAllowedIndicator, RetireDate, SortColumn, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'@D6', N'NaCitrate/XX/<37C', N'NaCit', 1, NULL, 0, '20100820 14:52:17.000', N'vhaishsmithm', 0)
INSERT INTO dbo.ProductAttribute (ProductAttributeId, ProductAttributeText, ProductAttributePrintName, ModificationAllowedIndicator, RetireDate, SortColumn, LastUpdateDate, LastUpdateUser, LastUpdateFunctionId) VALUES (N'@D7', N'AS3/500mL/refg', N'AS3', 1, NULL, 0, '20100820 14:52:17.000', N'vhaishsmithm', 0)


/*****************************************************
**	Update VbecsVersion
*****************************************************/
DECLARE @SUser	VARCHAR(45)

SET @SUser = (SELECT SUSER_SNAME())

INSERT INTO dbo.VbecsVersion
(Version, PatchSuccessfulIndicator, LastUpdateDate, LastUpdateUser)
VALUES('1.5.2.2', 0, GETUTCDATE(), RIGHT(@SUser, LEN(@SUser) - CHARINDEX('\', @SUser)))